@@ -53,6 +53,7 @@ void tharea::start_insert() {
   if (this->type == TT_AREA_TYPE_U) {
     if (this->m_subtype_str == NULL)
       ththrow(("missing subtype specification for area of user defined type"))
+    this->db->db2d.register_u_symbol(this->get_class_id(), this->m_subtype_str);
   }
 }
 
@@ -78,19 +79,19 @@ int tharea::get_cmd_nargs()
 }
 
 
-char * tharea::get_cmd_end()
+const char * tharea::get_cmd_end()
 {
   return "endarea";
 }
 
 
-char * tharea::get_cmd_name()
+const char * tharea::get_cmd_name()
 {
   return "area";
 }
 
 
-thcmd_option_desc tharea::get_cmd_option_desc(char * opts)
+thcmd_option_desc tharea::get_cmd_option_desc(const char * opts)
 {
 //  int id = thmatch_token(opts, thtt_area_opt);
 //  if (id == TT_AREA_UNKNOWN)
@@ -239,7 +240,15 @@ bool tharea::export_mp(class thexpmapmpxs * out)
     bl = bl->next_line;
     blnum++;
   }
-  fprintf(out->file,"%s(buildcycle(",out->symset->get_mp_macro(macroid));
+
+  if (this->type == TT_AREA_TYPE_U) {
+    out->symset->export_mp_symbol_options(out->file, -1);
+    fprintf(out->file,"a_u_%s(buildcycle(",this->m_subtype_str);
+    this->db->db2d.use_u_symbol(this->get_class_id(), this->m_subtype_str);
+  } else {
+    out->symset->export_mp_symbol_options(out->file, macroid);
+    fprintf(out->file,"%s(buildcycle(",out->symset->get_mp_macro(macroid));
+  }
 
   this->first_line->line->export_path_mp(out);
   bl = this->first_line->next_line;
@@ -249,11 +258,9 @@ bool tharea::export_mp(class thexpmapmpxs * out)
     bl = bl->next_line;
   }
   
-  if (this->type == TT_AREA_TYPE_U)
-    fprintf(out->file,"),\"%s\");\n", (this->m_subtype_str == NULL) ? "" : this->m_subtype_str);
-  else
-    fprintf(out->file,"));\n");
+  fprintf(out->file,"));\n");
   
+  th2ddataobject::export_mp_end(out);
   return(false);  
   
 }
