@@ -34,6 +34,7 @@
 #include <set>
 #include <vector>
 #include <string>
+#include <sstream>
 
 #include <cstring>
 #include <cstdio>
@@ -47,6 +48,8 @@ using namespace std;
 
 // extern list<scraprecord> SCRAPLIST;
 
+#define IOerr(F) ((string)"Can't open file "+F+"!\n").c_str()
+
 map<string,string> RGB, ALL_FONTS, ALL_PATTERNS;
 typedef set<unsigned char> FONTCHARS;
 map<string,FONTCHARS> USED_CHARS;
@@ -58,7 +61,7 @@ int convert_mode;
 
 void read_hash() {
   ifstream F("data.pl");
-  if(!F) therror(("???"));
+  if(!F) therror((IOerr("data.pl")));
   string line, tmp = "";
   char buf[100];
   scraprecord S;
@@ -235,12 +238,13 @@ string process_pdf_string(string s, string font) {
 //                30 -- legend
 //                31 -- northarrow, scalebar
 
-void distill_eps(string name, string fname, string cname, int mode, ofstream& TEX) {
+void distill_eps(string name, string fname, string cname, int mode, ofstream& TEX, double r = -1, double g = -1, double b = -1) {
   string form_id;
   string tok, lastmovex, lastmovey, buffer;
   string font, patt, fntmatr;
+  string pattcolor = "0 0 0";
   bool comment = true, concat = false, 
-       already_transp = false, transp_used = false;
+       already_transp = false, transp_used = false, before_group_transp = false;
   double llx = 0, lly = 0, urx = 0, ury = 0, HS = 0.0, VS = 0.0;
   double dx, dy;
   char x[20],y[20];
@@ -249,9 +253,14 @@ void distill_eps(string name, string fname, string cname, int mode, ofstream& TE
   list<scraprecord>::iterator J;
   
   convert_mode = mode;
+  
+  ostringstream text_attr;
+  if (LAYOUT.colored_text && r >= 0 && g >= 0 && b >= 0) {
+    text_attr << "0.1 w " << r << " " << g << " " << b << " rg 2 Tr ";
+  };
 
   ifstream F(fname.c_str());
-  if(!F) therror(("???"));
+  if(!F) therror((IOerr(fname)));
   while(F >> tok) {
     if (comment) {                      // File header
       if ((tok == "%%BoundingBox:") && (mode > 0)) {
@@ -324,7 +333,7 @@ void distill_eps(string name, string fname, string cname, int mode, ofstream& TE
 	if ((mode <= 11) && (cname != "")) { // beginning of boundary cl.path definition
           TEX << "\\PL{q}";          // for F and G scraps
           ifstream G(cname.c_str());
-          if(!G) therror(("???"));
+          if(!G) therror((IOerr(cname)));
           while(G >> buffer) {
             if ((buffer == "m") || (buffer == "l") || (buffer == "c")) {
               print_queue(thstack,llx,lly,buffer,TEX);
@@ -401,6 +410,7 @@ void distill_eps(string name, string fname, string cname, int mode, ofstream& TE
         thstack.clear();
       }
       else if (tok == "setgray") {
+        if (mode == 0) continue;            // ignore color for uncolored patterns
         if (already_transp) {  // transp off
           print_str("/GS0 gs",TEX);
           already_transp = false;
@@ -409,6 +419,7 @@ void distill_eps(string name, string fname, string cname, int mode, ofstream& TE
         thstack.clear();
       }
       else if (tok == "setrgbcolor") {
+        if (mode == 0) continue;            // ignore color for uncolored patterns
         if ((!((thstack[0] == "0.00002") && (thstack[1] == "0.00018"))) 
               && already_transp) {           // transp off
           print_str("/GS0 gs",TEX);
@@ -424,7 +435,7 @@ void distill_eps(string name, string fname, string cname, int mode, ofstream& TE
               ALL_PATTERNS.insert(make_pair(patt,u2str(patt_id)));
               patt_id++;
             }
-            print_str("/CS1 cs /"+patt+" scn",TEX);
+            print_str("/CS1 cs "+pattcolor+" /"+patt+" scn",TEX);
           }
           else if (thstack[1] == "0.00018") {     // transparency
             transp_used = true;
@@ -467,10 +478,14 @@ void distill_eps(string name, string fname, string cname, int mode, ofstream& TE
       else if (tok == "gsave") {
         print_str("q",TEX);
         thstack.clear();
+        if (already_transp) before_group_transp = true;
+        else before_group_transp = false;
       }
       else if (tok == "grestore") {
         print_str("Q",TEX);
         thstack.clear();
+        if (before_group_transp) already_transp = true;
+        else already_transp = false;
       }
       else if (tok == "translate") {
         print_str("1 0 0 1 "+thstack[0]+" "+thstack[1]+" cm",TEX);
@@ -524,6 +539,7 @@ void distill_eps(string name, string fname, string cname, int mode, ofstream& TE
           sprintf(y, "%.1f", atof(lastmovey.c_str())-lly);
           print_str("1 0 0 1 "+string(x)+" "+string(y)+" Tm",TEX);
         }
+	print_str(text_attr.str(),TEX);
         TEX << "\\PL{" << buffer << " Tj}%" << endl;
         print_str("ET",TEX);
         concat = false;
@@ -543,6 +559,10 @@ void distill_eps(string name, string fname, string cname, int mode, ofstream& TE
         concat = true;
         thstack.clear();
       }
+      else if (tok == "THsetpatterncolor") {
+        pattcolor = thstack[0] + " " + thstack[1] + " " + thstack[2];
+        thstack.clear();
+      }
       else {
         thstack.push_back(tok);
       }
@@ -590,7 +610,7 @@ void distill_eps(string name, string fname, string cname, int mode, ofstream& TE
                  "\\space 0 R ";
         }
         TEX << ">> ";
-        TEX << "/ColorSpace << /CS1 [/Pattern /DeviceGray] >> ";
+        TEX << "/ColorSpace << /CS1 [/Pattern /DeviceRGB] >> ";
       }
       TEX << "} ";
     }
@@ -605,19 +625,19 @@ void convert_scraps() {
   unsigned char c;
  
   ofstream TEX("th_formdef.tex");
-  if(!TEX) therror(("???"));
+  if(!TEX) therror((IOerr("th_formdef.tex")));
   TEX.setf(ios::fixed, ios::floatfield);
   TEX.precision(2);
   
   for(list<scraprecord>::iterator I = SCRAPLIST.begin(); 
                                   I != SCRAPLIST.end(); I++) {
 //    cout << "*" << flush;
-    if (I->F != "") distill_eps(I->name, I->F, I->C, 10, TEX);
+    if (I->F != "") distill_eps(I->name, I->F, I->C, 10, TEX, I->r, I->g, I->b);
     if (I->G != "") distill_eps(I->name, I->G, I->C, 11, TEX);
     if (I->B != "") distill_eps(I->name, I->B, "", 12, TEX);
     if (I->I != "") distill_eps(I->name, I->I, "", 13, TEX);
-    if (I->E != "") distill_eps(I->name, I->E, "", 14, TEX);
-    if (I->X != "") distill_eps(I->name, I->X, "", 20, TEX);
+    if (I->E != "") distill_eps(I->name, I->E, "", 14, TEX, I->r, I->g, I->b);
+    if (I->X != "") distill_eps(I->name, I->X, "", 20, TEX, I->r, I->g, I->b);
   }
 
   // similarly with legend (distill_eps( , , , 30, TEX))
@@ -647,7 +667,7 @@ void convert_scraps() {
   TEX.close();
 
   ofstream F("th_fontdef.tex");
-  if(!F) therror(("???"));
+  if(!F) therror((IOerr("th_fontdef.tex")));
   F.setf(ios::fixed, ios::floatfield);
   F.precision(2);
   for (map<string,string>::iterator I = ALL_FONTS.begin(); 
@@ -682,7 +702,7 @@ void convert_scraps() {
   }
   F << "\\endgroup" << endl;
   ifstream P("patterns.dat");
-  if(!P) therror(("???"));
+  if(!P) therror((IOerr("patterns.dat")));
   char buf[200];
   char delim[] = ":";
   string line,num,pfile,bbox,xstep,ystep,matr;
@@ -696,12 +716,12 @@ void convert_scraps() {
     map<string,string>::iterator I = ALL_PATTERNS.find(num);
     if (I != ALL_PATTERNS.end()) {
       F << "\\immediate\\pdfobj stream attr {/Type /Pattern" << endl;
-      F << "/PaintType 1 /PatternType 1 /TilingType 1" << endl;
+      F << "/PaintType 2 /PatternType 1 /TilingType 1" << endl;
       F << "/Matrix " << matr << endl;
       F << "/BBox " << bbox << endl;
       F << "/XStep " << xstep << endl;
       F << "/YStep " << ystep << endl;
-      F << "/Resources << /ProcSet [/PDF ] >> } {0 g 0 G " << endl;
+      F << "/Resources << /ProcSet [/PDF ] >> } {" << endl;
       distill_eps("", pfile , "", 0, F);
       F << "} \\newcount \\" << tex_Pname((*I).second) << 
            "\\" << tex_Pname((*I).second) << "=\\pdflastobj" << endl;
@@ -785,7 +805,7 @@ void convert_scraps() {
 
 void read_rgb() {
   ifstream F("rgbcolors.dat");
-  if(!F) therror(("???"));
+  if(!F) therror((IOerr("rgbcolors.dat")));
   char buf[100];
   string line, color, value;
   while(F.getline(buf,100,'\n')) {
