@@ -29,10 +29,16 @@
 #include "thexception.h"
 #include "thchenc.h"
 #include "thsurvey.h"
+#include "thconfig.h"
+#include "thparse.h"
+#include "thcsdata.h"
+#include "thproj.h"
 
 thdataobject::thdataobject()
 {
   this->db = NULL;
+  this->cs = TTCS_LOCAL;
+  this->id = 0;
   
   this->name = "";
   this->title = "";
@@ -103,7 +109,7 @@ int thdataobject::get_cmd_nargs()
 }
 
 
-char * thdataobject::get_cmd_end()
+const char * thdataobject::get_cmd_end()
 {
   return NULL;
 }
@@ -119,19 +125,20 @@ bool thdataobject::get_cmd_ends_match(char * cmd) {
 }
 
 
-char * thdataobject::get_cmd_name()
+const char * thdataobject::get_cmd_name()
 {
   return NULL;
 }
 
 
-thcmd_option_desc thdataobject::get_cmd_option_desc(char * opts)
+thcmd_option_desc thdataobject::get_cmd_option_desc(const char * opts)
 {
   int id = thmatch_token(opts, thtt_dataobject_opt);
   switch (id) {
     case TT_DATAOBJECT_AUTHOR:
     case TT_DATAOBJECT_COPYRIGHT:
     case TT_DATAOBJECT_STNS:
+    case TT_DATAOBJECT_ATTR:
       return thcmd_option_desc(id, 2);
     default:
       return thcmd_option_desc(id);
@@ -149,6 +156,28 @@ void thdataobject::set(thcmd_option_desc cod, char ** args, int argenc, unsigned
       else 
         ththrow(("invalid keyword -- %s", *args));
       break;
+
+    case TT_DATAOBJECT_CS:
+      switch (this->get_class_id()) {
+        case TT_DATA_CMD:
+        case TT_LAYOUT_CMD:
+        case TT_SURFACE_CMD:
+        case TT_IMPORT_CMD:
+        case TT_SCRAP_CMD:
+          break;
+        default:
+          ththrow(("coordinate system specification not valid for %s", this->get_cmd_name()))
+          break;  
+      }
+      {
+        int id = thcasematch_token(*args, thtt_cs);
+        if (id == TTCS_UNKNOWN) {
+          ththrow(("unknown coordinate system -- %s", *args));
+        }
+        this->cs = id;
+        this->cs_source = this->db->csrc;
+      }
+      break;
     
     case TT_DATAOBJECT_STNS:
       switch (this->get_class_id()) {
@@ -226,6 +255,18 @@ void thdataobject::set(thcmd_option_desc cod, char ** args, int argenc, unsigned
           break;
       }
       break;
+
+    case TT_DATAOBJECT_ATTR:
+      switch (this->get_class_id()) {
+        case TT_LAYOUT_CMD:
+          ththrow(("attribute specification not valid for layout"))
+          break;
+      }
+      if (cod.nargs != 2)
+        ththrow(("invalid attribute specification -- should be <name> <value>"))
+      thencode(&(this->db->buff_enc), args[1], argenc);
+      this->parse_attribute(args[0], this->db->buff_enc.get_buffer());
+      break;
         
     default:
       ththrow(("unknown option -- %s", args[0]));
@@ -252,13 +293,13 @@ int thdataobject::get_context()
 }
 
 
-char * thdataobject::get_name()
+const char * thdataobject::get_name()
 {
   return this->name;
 }
 
 
-char * thdataobject::get_title()
+const char * thdataobject::get_title()
 {
   return this->title;
 }
@@ -360,3 +401,108 @@ void thdataobject::start_insert()
 }
 
 
+
+void thdataobject::convert_cs(char * src_x, char * src_y, double & dst_x, double & dst_y)
+{
+  // 1. Check kompatibility with output CS.
+  if (thcfg.outcs_def.is_valid()) {
+    if (((this->cs == TTCS_LOCAL) && (thcfg.outcs != TTCS_LOCAL)) ||
+      ((this->cs != TTCS_LOCAL) && (thcfg.outcs == TTCS_LOCAL)))
+      ththrow(("mixing local and global coordinate systems not allowed -- conflict with cs specification at %s [%d]", thcfg.outcs_def.name, thcfg.outcs_def.line));
+  };
+
+  // 1. Conversion to numbers.
+  int sv;
+  double tx(0.0), ty(0.0), tz(0.0), dst_z(0.0);
+  bool initcs(false);
+  if ((this->cs != TTCS_LOCAL) && thcsdata_table[this->cs].dms) {
+    thparse_double_dms(sv, tx, src_x);
+    tx /= 180.0 / THPI;
+  } else {
+    thparse_double(sv, tx, src_x);
+  }
+  if (sv != TT_SV_NUMBER)
+    ththrow(("invalid X coordinate -- %s", src_x));
+
+  if ((this->cs != TTCS_LOCAL) && thcsdata_table[this->cs].dms) {
+    thparse_double_dms(sv, ty, src_y);
+    ty /= 180.0 / THPI;
+  } else {
+    thparse_double(sv, ty, src_y);
+  }
+  if (sv != TT_SV_NUMBER)
+    ththrow(("invalid Y coordinate -- %s", src_y));
+
+  if ((this->cs != TTCS_LOCAL) && thcsdata_table[this->cs].swap) {
+    tz = tx;
+    tx = ty;
+    ty = tz;
+    tz = 0.0;
+  }
+
+  if ((this->cs != TTCS_LOCAL) && thcsdata_table[this->cs].dms) {
+    if ((tx < - THPI) || (tx > THPI))
+      ththrow(("longitude out of range -- %s", thcsdata_table[this->cs].swap ? src_y : src_x));
+
+    if ((ty < (- THPI / 2)) || (ty > (THPI / 2)))
+      ththrow(("latitude out of range -- %s", thcsdata_table[this->cs].swap ? src_x : src_y));
+  }
+
+  if (!thcfg.outcs_def.is_valid()) {
+    if ((this->cs != TTCS_LOCAL) && (!thcsdata_table[this->cs].output)) {
+      // TODO: get NS
+      double dumx, dumy, dumz;
+      int south = 0;
+      thcs2cs(thcsdata_table[this->cs].params, thcsdata_table[TTCS_LAT_LONG].params, tx, ty, tz, dumx, dumy, dumz);
+      if (dumy < 0.0)
+        south = 1;
+      thcfg.outcs = TTCS_UTM1N + 2 * (thcs2zone(thcsdata_table[this->cs].params, tx, ty, tz) - 1) + south;
+    } else {
+      thcfg.outcs = this->cs;
+    }
+    if (this->cs_source.is_valid())
+      thcfg.outcs_def = this->cs_source;
+    else
+      thcfg.outcs_def = this->source;
+    initcs = true;
+  }
+
+  if (this->cs == TTCS_LOCAL) {
+    dst_x = tx;
+    dst_y = ty;
+    dst_z = tz;
+  } else {
+    thcs2cs(thcsdata_table[this->cs].params, thcsdata_table[thcfg.outcs].params, tx, ty, tz, dst_x, dst_y, dst_z);
+  }
+
+  if (thcfg.outcs != TTCS_LOCAL) {
+    if (initcs) {
+      thcfg.outcs_sumx = dst_x;
+      thcfg.outcs_sumy = dst_y;
+      thcfg.outcs_sumz = dst_z;
+      thcfg.outcs_sumn = 1.0;
+    } else {
+      thcfg.outcs_sumx += dst_x;
+      thcfg.outcs_sumy += dst_y;
+      thcfg.outcs_sumz += dst_z;
+      thcfg.outcs_sumn += 1.0;
+    }
+  }
+  
+}
+
+
+void thdataobject::parse_attribute(char * name, char * value) {
+
+  // check name
+  if ((name == NULL) || (strlen(name) == 0))
+    ththrow(("epmty attribute name not allowed"))
+  if (name[0] == '_')
+    ththrow(("attribute name starting with '_' not allowed"))
+  if (!th_is_attr_name(name))
+    ththrow(("invalid characters in attribute name -- %s", name))
+
+  this->db->attr.insert_attribute(name, value, long(this->id));
+}
+
+
