@@ -50,7 +50,7 @@ enum {
   TT_SCRAP_CMD, TT_ENDSCRAP_CMD,
   TT_POINT_CMD, TT_LINE_CMD, TT_AREA_CMD,
   TT_JOIN_CMD, TT_MAP_CMD, TT_SURFACE_CMD,
-  TT_DATA_CMD, TT_IMPORT_CMD,
+  TT_DATA_CMD, TT_IMPORT_CMD, TT_SKETCH_CMD,
 };
 
 
@@ -72,6 +72,7 @@ static const thstok thtt_commands[] = {
   {"map", TT_MAP_CMD},
   {"point", TT_POINT_CMD},
   {"scrap", TT_SCRAP_CMD},
+  {"sketch", TT_SKETCH_CMD},
   {"surface", TT_SURFACE_CMD},
   {"survey", TT_SURVEY_CMD},
 	{NULL, TT_UNKNOWN_CMD},
@@ -88,6 +89,9 @@ enum {
   TT_DATAOBJECT_TITLE = 1002,
   TT_DATAOBJECT_AUTHOR = 1003,
   TT_DATAOBJECT_COPYRIGHT = 1004,
+  TT_DATAOBJECT_STNS = 1005,
+  TT_DATAOBJECT_CS = 1006,
+  TT_DATAOBJECT_ATTR = 1007,
 };
 
 
@@ -96,15 +100,17 @@ enum {
  */
  
 static const thstok thtt_dataobject_opt[] = {
+  {"attr", TT_DATAOBJECT_ATTR},
   {"author", TT_DATAOBJECT_AUTHOR},
   {"copyright", TT_DATAOBJECT_COPYRIGHT},
+  {"cs", TT_DATAOBJECT_CS},
   {"id", TT_DATAOBJECT_NAME},
+  {"station-names", TT_DATAOBJECT_STNS},
   {"title", TT_DATAOBJECT_TITLE},
 	{NULL, TT_DATAOBJECT_UNKNOWN},
 };
 
 
-
 /**
  * Option description class.
  */
@@ -196,7 +202,7 @@ class thdataobject_copyright {
   
   public:
 
-  char * name;  ///< Author's name
+  const char * name;  ///< Author's name
   unsigned long rev;  ///< Object's revision.
   
   /**
@@ -210,7 +216,7 @@ class thdataobject_copyright {
    * Default constructor.
    */
    
-  thdataobject_copyright(char * nn, unsigned long rv) : name(nn), rev(rv) {}
+  thdataobject_copyright(const char * nn, unsigned long rv) : name(nn), rev(rv) {}
   
   
 };
@@ -237,13 +243,16 @@ class thdataobject {
 
   class thdatabase * db;
 
-  char * name,  ///< Object name.
-    * title;  ///< Object title.
+  const char * name,  ///< Object name.
+    * title,  ///< Object title.
+    * stnpref, ///< Station names prefix;
+    * stnsuff; ///< Station names suffix;
   
   unsigned long id;  ///< Object identifier.
   
   bool selected,  ///< Whether object is selected.
     tmp_bool;  ///< Temporary variable for some algorithms
+    
   unsigned long selected_number,  ///< Number of selection.
     tmp_ulong;  ///< Temporary variable for some algorithms
   
@@ -252,6 +261,7 @@ class thdataobject {
   class thsurvey * fsptr;  ///< Father survey ptr.
   
   unsigned long revision;  ///< Object revision.
+	thobjectsrc source;  ///< Object source.
 
   thperson dotmp_person;  ///< Temporary person.
   thdate dotmp_date;  ///< Temporary date.
@@ -260,6 +270,8 @@ class thdataobject {
   thdo_author_map_type author_map;  ///< Author map.
   thdo_copyright_map_type copyright_map;  ///< Copyright map.
 
+  int cs;  ///< Coordinate system.
+	thobjectsrc cs_source;  ///< Coordinate system source.
 
   public:
 
@@ -295,7 +307,7 @@ class thdataobject {
    * Return class name.
    */
    
-  virtual char * get_class_name() {return "thdataobject";};
+  virtual const char * get_class_name() {return "thdataobject";};
   
   
   /**
@@ -316,7 +328,7 @@ class thdataobject {
    * Return command end option.
    */
    
-  virtual char * get_cmd_end();
+  virtual const char * get_cmd_end();
   
   
   /**
@@ -337,14 +349,14 @@ class thdataobject {
    * Return command name.
    */
    
-  virtual char * get_cmd_name();
+  virtual const char * get_cmd_name();
   
   
   /**
    * Return option description.
    */
    
-  virtual thcmd_option_desc get_cmd_option_desc(char * opts);
+  virtual thcmd_option_desc get_cmd_option_desc(const char * opts);
   
   
   /**
@@ -369,14 +381,14 @@ class thdataobject {
    * Return object name.
    */
    
-  virtual char * get_name();
+  virtual const char * get_name();
   
   
   /**
    * Return object title.
    */
    
-  char * get_title();
+  const char * get_title();
   
   
   /**
@@ -444,6 +456,19 @@ class thdataobject {
    */
  
   bool is_in_survey(thsurvey * psearch);
+
+  /**
+   * Convert coordinates according to current CS specification.
+   */
+
+  void convert_cs(char * src_x, char * src_y, double & dst_x, double & dst_y);
+
+  /**
+   * Parse object attributes.
+   */
+
+  void parse_attribute(char * name, char * value);
+
 };
 
 
