@@ -34,10 +34,16 @@
 #include "thdataleg.h"
 #include "thdb3d.h"
 #include "thdataleg.h"
+#include "thattr.h"
 #include <map>
 #include <vector>
 #include <list>
 
+enum {
+  TT_TEMPSTATION_NONE,
+  TT_TEMPSTATION_WALL,
+  TT_TEMPSTATION_FEATURE,
+};
 
 struct thdb1d_loop_leg {
   thdb1d_loop_leg * next_leg, * prev_leg;
@@ -123,9 +129,8 @@ class thdb1ds {
   double x, y, z;   // coordinates
   double xx;        // coordinate for extended elevation
   
-  char * name,  ///< Station name.
-    * comment,  ///< Station comment.
-    * code;     ///< Station continuation code.
+  const char * name,  ///< Station name.
+    * comment;  ///< Station comment.
   
   class thsurvey * survey;  ///< Station survey.
   class thdataobject * fixcontext;  ///< Fix context.
@@ -135,35 +140,40 @@ class thdb1ds {
   unsigned data_priority, ///< 0 - undefined, 1 - equate, 2 - leg, 3 - fix
     data_slength;  ///< Survey specification length
   
-  unsigned char flags,  ///< Station flags.
-    mark,  ///< Mark type.
-    extend;  ///< Extend flags: normal, reverse, left, right, break
+  char temps;           ///< Station temporary status.
+  unsigned long flags;  ///< Station flags.
+  int mark,             ///< Mark type.
+    extend;             ///< Extend flags: normal, reverse, left, right, break
   bool mark_station;
   
   bool adjusted, fixed;
   int placed;
   double sdx, sdy, sdz;
   
+  double explored;
+  
   /**
    * Default constructor.
    */
    
-  thdb1ds() : uid(0), x(0), y(0), z(0), name(NULL), comment(NULL), code(NULL), survey(NULL), fixcontext(NULL), 
-    data(NULL), data_priority(0), data_slength(0), 
+  thdb1ds() : uid(0), x(0), y(0), z(0), name(NULL), comment(NULL), survey(NULL), fixcontext(NULL), 
+    data(NULL), data_priority(0), data_slength(0), temps(TT_TEMPSTATION_NONE),
     flags(TT_STATIONFLAG_NONE), mark(TT_DATAMARK_TEMP), extend(TT_EXTENDFLAG_NORMAL), 
-    adjusted(false), fixed(false), placed(0), sdx(0.0), sdy(0.0), sdz(0.0) {}
+    adjusted(false), fixed(false), placed(0), sdx(0.0), sdy(0.0), sdz(0.0),
+    explored(thnan) {}
   
 
   /**
    * Default constructor with 2 parameters.
    */
    
-  thdb1ds(char * n, class thsurvey * ps) : uid(0), x(0), y(0), z(0), name(n), 
-    comment(NULL), code(NULL), survey(ps), fixcontext(NULL),  
-    data(NULL), data_priority(0), data_slength(0), 
+  thdb1ds(const char * n, class thsurvey * ps) : uid(0), x(0), y(0), z(0), name(n), 
+    comment(NULL), survey(ps), fixcontext(NULL),  
+    data(NULL), data_priority(0), data_slength(0), temps(TT_TEMPSTATION_NONE),
     flags(TT_STATIONFLAG_NONE),
     mark(TT_DATAMARK_TEMP), extend(TT_EXTENDFLAG_NORMAL), mark_station(false), 
-    adjusted(false), fixed(false), placed(0), sdx(0.0), sdy(0.0), sdz(0.0) {}
+    adjusted(false), fixed(false), placed(0), sdx(0.0), sdy(0.0), sdz(0.0),
+    explored(thnan) {}
     
   
   /**
@@ -172,6 +182,21 @@ class thdb1ds {
   
   void set_parent_data(class thdata * pd, unsigned pd_priority, unsigned pd_slength);
   
+
+  /**
+   * Is station temporary.
+   */
+  bool is_temporary() {return (temps != TT_TEMPSTATION_NONE);}
+
+  /**
+   * Set temporary status.
+   */
+  void set_temporary(char ts) {temps = ts;}
+
+  /**
+   * Set temporary status.
+   */
+  void set_temporary(const char * name);
   
 };
 
@@ -237,6 +262,8 @@ class thdb1d {
   
   thdb3ddata d3_data, d3_surface, d3_walls;
 
+  thattr m_station_attr;
+
   double min_year, max_year;
   
   bool d3_data_parsed;
