@@ -53,6 +53,10 @@
 #include <time.h>
 #include <string>
 
+extern const thstok thtt__texts [];
+
+int therion_exit_state = 2;
+
 bool thverbose_mode = true;
 
 bool thtext_inline = false;
@@ -62,13 +66,14 @@ char * thexecute_cmd = NULL;
 const char * thhelp_text =
       "\ntherion [-q] [-L] [-l log-file]\n"
       "\t[-s source-file] [-p search-path]\n"
-      "\t[-g|-u] [-i] [-d] [-x] [cfg-file]\n\n"
+      "\t[-d] [-x] [cfg-file]\n\n"
       "therion [-h|--help]\n"
       "        [-v|--version]\n"
       "        [--print-encodings]\n"
       "        [--print-init-file]\n"
       "        [--print-tex-encodings]\n"
-      "        [--print-environment]\n\n";
+      "        [--print-environment]\n"
+      "        [--print-symbols]\n\n";
 
 const char * thversion_text = THVERSION;
 const char * thversion_format = "therion %s";
@@ -101,19 +106,21 @@ void thprintf2err(const char *format, ...)
 
 
 void thprint_environment() {
-  thprintf("INIT=%s\n",thcfg.get_initialization_path());
-  thprintf("SOURCE=%s\n",thcfg.get_search_path());
-//  thprintf("CAVERN=%s\n",thini.get_path_cavern());
+  thprintf("\n\nINIT=%s\n",thcfg.get_initialization_path());
+  thprintf("SOURCE=%s\n\n",thcfg.get_search_path());
+  thprintf("CAVERN=%s\n",thini.get_path_cavern());
   thprintf("METAPOST=%s\n",thini.get_path_mpost());
   thprintf("PDFTEX=%s\n",thini.get_path_pdftex());
+  thprintf("IDENTIFY=%s\n",thini.get_path_identify());
+  thprintf("CONVERT=%s\n",thini.get_path_convert());
 }
 
 
 void thprint_xtherion() {
   bool already_exported;
   int i, j, l;
-  char * lngstr, *trnstr;
-  thbuffer tsrc, tdst;
+  const char * lngstr, *trnstr, * tsrc;
+  thbuffer tdst;
   std::string tss;
   thprintf("set xth(point_types) {\n");
   for(i = 0; thtt_point_types[i].tok != TT_POINT_TYPE_UNKNOWN; i++) {
@@ -126,25 +133,6 @@ void thprint_xtherion() {
     }
     if (!already_exported) {
       thprintf("\t%s\n",thtt_point_types[i].s);
-      tsrc  = "point ";
-      tsrc += thtt_point_types[i].s;
-      l = 0;
-      lngstr = thlang_getid(l);
-      while (strlen(lngstr) > 0) {
-        trnstr = thT(tsrc.get_buffer(), l);
-	if (strcmp(trnstr, tsrc.get_buffer()) != 0) {
-	  thdecode_tcl(&tdst, trnstr);
-    	  tss += "::msgcat::mcset ";
-	  tss += lngstr;
-	  tss += " \"";
-	  tss += tsrc.get_buffer();
-	  tss += "\" [encoding convertfrom utf-8 \"";
-	  tss += tdst.get_buffer();
-	  tss += "\"]\n";
-	}
-        l++;
-        lngstr = thlang_getid(l);
-      }      
     }
   }
   thprintf("}\n\nset xth(line_types) {\n");
@@ -158,25 +146,6 @@ void thprint_xtherion() {
     }
     if (!already_exported) {
       thprintf("\t%s\n",thtt_line_types[i].s);
-      tsrc  = "line ";
-      tsrc += thtt_line_types[i].s;
-      l = 0;
-      lngstr = thlang_getid(l);
-      while (strlen(lngstr) > 0) {
-        trnstr = thT(tsrc.get_buffer(), l);
-	if (strcmp(trnstr, tsrc.get_buffer()) != 0) {
-	  thdecode_tcl(&tdst, trnstr);
-    	  tss += "::msgcat::mcset ";
-	  tss += lngstr;
-	  tss += " \"";
-	  tss += tsrc.get_buffer();
-	  tss += "\" [encoding convertfrom utf-8 \"";
-	  tss += tdst.get_buffer();
-	  tss += "\"]\n";
-	}
-        l++;
-        lngstr = thlang_getid(l);
-      }      
     }
   }
   thprintf("}\n\nset xth(area_types) {\n");
@@ -190,28 +159,33 @@ void thprint_xtherion() {
     }
     if (!already_exported) {
       thprintf("\t%s\n",thtt_area_types[i].s);
-      tsrc  = "area ";
-      tsrc += thtt_area_types[i].s;
-      l = 0;
-      lngstr = thlang_getid(l);
-      while (strlen(lngstr) > 0) {
-        trnstr = thT(tsrc.get_buffer(), l);
-	if (strcmp(trnstr, tsrc.get_buffer()) != 0) {
-	  thdecode_tcl(&tdst, trnstr);
-    	  tss += "::msgcat::mcset ";
-	  tss += lngstr;
-	  tss += " \"";
-	  tss += tsrc.get_buffer();
-	  tss += "\" [encoding convertfrom utf-8 \"";
-	  tss += tdst.get_buffer();
-	  tss += "\"]\n";
-	}
-        l++;
-        lngstr = thlang_getid(l);
-      }      
     }
   }
   thprintf("}\n");
+
+  tss = "";
+  const thstok * x = thlang_get_text_table();
+  while (x->s != NULL) {
+    tsrc = x->s;
+    l = 0;
+    lngstr = thlang_getid(l);
+    while (strlen(lngstr) > 0) {
+      trnstr = thT(tsrc, l);
+      if (strcmp(trnstr, tsrc) != 0) {
+        thdecode_tcl(&tdst, trnstr);
+        tss += "::msgcat::mcset ";
+        tss += lngstr;
+        tss += " \"";
+        tss += tsrc;
+        tss += "\" [encoding convertfrom utf-8 \"";
+        tss += tdst.get_buffer();
+        tss += "\"]\n";
+      }
+      l++;
+      lngstr = thlang_getid(l);
+    }      
+    x = &(x[1]);
+  }
   thprintf("\n%s", tss.c_str());
 }
 
@@ -232,6 +206,7 @@ void thpause_exit() {
 
 void thexit(int exit_code)
 {
+  if (exit_code == EXIT_FAILURE) thcfg.xth_save();
   exit(exit_code);
 }  
 
@@ -239,7 +214,9 @@ void thexit(int exit_code)
 int main(int argc, char * argv[]) {
 
 #ifndef THDEBUG
+#ifndef THMSVC
   try {
+#endif
 #endif  
 
     time_t tmUserStart = time(NULL);
@@ -314,23 +291,23 @@ int main(int argc, char * argv[]) {
     }
     
     // load input
-    long sid, maxsid = thcfg.get_source_file_names()->get_size();
-    if (maxsid == 0)
+    thconfig_src_list * sources = thcfg.get_sources();
+    thconfig_src_list::iterator srcit;
+    if (sources->size() == 0)
       therror(("source files not specified"));
-    char ** srcn = thcfg.get_source_file_names()->get_buffer();
 #ifndef THDEBUG
     thprintf("reading source files ... ");
     thtext_inline = true;
 #endif 
 
 
-    for(sid = 0; sid < maxsid; sid++, srcn++) {
+    for(srcit = sources->begin(); srcit != sources->end(); srcit++) {
 
 #ifdef THDEBUG
-      thprintf("\nreading input -- %s\n", *srcn);
+      thprintf("\nreading input -- %s\n", srcit->fname);
 #endif 
 
-      thdbreader.read(*srcn, 
+      thdbreader.read(srcit->fname, srcit->startln, srcit->endln, 
         thcfg.get_search_path(), &thdb);
 
 #ifdef THDEBUG
@@ -399,14 +376,15 @@ int main(int argc, char * argv[]) {
     thdb.db1d.print_loops();
     thdb.db2d.log_distortions();
     
-#ifdef THMSVC
-    getchar();
-#endif
+//#ifdef THMSVC
+//    getchar();
+//#endif
 
     //exit the program
     return(EXIT_SUCCESS);
     
 #ifndef THDEBUG
+#ifndef THMSVC
   }
   catch(...)
   {
@@ -417,6 +395,7 @@ int main(int argc, char * argv[]) {
     }
   }
 #endif
+#endif
 
 }
 
