@@ -186,7 +186,7 @@ void thexpmodel::export_3d_file(class thdatabase * dbp)
   if (nlegs == 0)
     return;
   
-  char * fnm = this->get_output("cave.3d");
+  const char * fnm = this->get_output("cave.3d");
   
 #ifdef THDEBUG
   thprintf("\n\nwriting %s\n", fnm);
@@ -198,7 +198,9 @@ void thexpmodel::export_3d_file(class thdatabase * dbp)
   unsigned long i;
   img * pimg;
   img_output_version = 4;
-  pimg = img_open_write(fnm, "cave", 1);
+  thbuffer fnmb;
+  fnmb.strcpy("cave");
+  pimg = img_open_write(fnm, fnmb.get_buffer(), 1);
      
   if (!pimg) {
     thwarning(("can't open %s for output",fnm))
@@ -261,9 +263,10 @@ void thexpmodel::export_3d_file(class thdatabase * dbp)
       if (((tmps->flags & TT_STATIONFLAG_FIXED) != 0) && ((tmps->flags & TT_STATIONFLAG_NOTFIXED) == 0))
         x_exp |= img_SFLAG_FIXED;
       stnbuf.strcpy(dbp->db1d.station_vec[i].survey->get_reverse_full_name());
-      stnbuf.strcat(".");
+      if (strlen(stnbuf.get_buffer()) > 0) stnbuf.strcat(".");
       stnbuf.strcat(dbp->db1d.station_vec[i].name);
-      img_write_item(pimg, img_LABEL, x_exp, stnbuf,
+      if (!tmps->is_temporary())
+        img_write_item(pimg, img_LABEL, x_exp, stnbuf,
           dbp->db1d.station_vec[i].x, dbp->db1d.station_vec[i].y, dbp->db1d.station_vec[i].z);
     }
   }
@@ -285,6 +288,8 @@ void thexpmodel::export_3d_file(class thdatabase * dbp)
 // 12 - station name
 // 40 - survey comment
 // -9.9 - no dimension
+
+#define toft(x) ((x)/0.3048)
  
 void thexpmodel::export_plt_file(class thdatabase * dbp)
 {
@@ -296,7 +301,7 @@ void thexpmodel::export_plt_file(class thdatabase * dbp)
   if (nlegs == 0)
     return;
   
-  char * fnm = this->get_output("cave.plt");
+  const char * fnm = this->get_output("cave.plt");
   
 #ifdef THDEBUG
   thprintf("\n\nwriting %s\n", fnm);
@@ -357,13 +362,11 @@ if (var > max) max = var
 strncpy(station_name,dbp->db1d.station_vec[stid].name,8); \
 station_name[8] = 0
 
-#define toft(x) ((x)/0.3048)
-
-#define exppltdim(ffd,ttd) (thisnan((*tlegs)->reverse ? (*tlegs)->leg->ffd : (*tlegs)->leg->ttd) ? -9999.0 : toft((*tlegs)->reverse ? (*tlegs)->leg->ffd : (*tlegs)->leg->ttd))
+#define exppltdim(ffd,ttd) (((*tlegs)->reverse ? thisnan((*tlegs)->leg->ffd) : thisnan((*tlegs)->leg->ttd)) ? -9999.0 : toft((*tlegs)->reverse ? (*tlegs)->leg->ffd : (*tlegs)->leg->ttd))
   
   // now let's print header
-  fprintf(pltf,"Z\t%f\t%f\t%f\t%f\t%f\t%f\n\n",toft(ymin-avy),toft(ymax-avy),toft(xmin-avx),toft(xmax-avx),toft(zmin),toft(zmax));
-//  fprintf(pltf,"Z\t%f\t%f\t%f\t%f\t%f\t%f\n",toft(ymin-avy),toft(ymax-avy),toft(xmin-avx),toft(xmax-avx),toft(zmin-avz),toft(zmax-avz));
+  fprintf(pltf,"Z\t%.2f\t%.2f\t%.2f\t%.2f\t%.2f\t%.2f\n\n",toft(ymin-avy),toft(ymax-avy),toft(xmin-avx),toft(xmax-avx),toft(zmin),toft(zmax));
+//  fprintf(pltf,"Z\t%.2f\t%.2f\t%.2f\t%.2f\t%.2f\t%.2f\n",toft(ymin-avy),toft(ymax-avy),toft(xmin-avx),toft(xmax-avx),toft(zmin-avz),toft(zmax-avz));
   fprintf(pltf,"STHERION\n");
   fprintf(pltf,"NX\tD\t3\t12\t1997\tCtherion export\n");
   last_st = nstat;
@@ -373,7 +376,7 @@ station_name[8] = 0
       cur_st = dbp->db1d.station_vec[((*tlegs)->reverse ? (*tlegs)->leg->to.id : (*tlegs)->leg->from.id) - 1].uid - 1;
       if (cur_st != last_st) {
         copy_station_name(cur_st);
-        fprintf(pltf,"M\t%f\t%f\t%f\tS%s\tP\t%f\t%f\t%f\t%f\n",
+        fprintf(pltf,"M\t%.2f\t%.2f\t%.2f\tS%s\tP\t%.2f\t%.2f\t%.2f\t%.2f\n",
           toft(dbp->db1d.station_vec[cur_st].y - avy), toft(dbp->db1d.station_vec[cur_st].x - avx), 
           toft(dbp->db1d.station_vec[cur_st].z),station_name,
           exppltdim(to_left, from_left), 
@@ -384,7 +387,9 @@ station_name[8] = 0
       }
       last_st = dbp->db1d.station_vec[((*tlegs)->reverse ? (*tlegs)->leg->from.id : (*tlegs)->leg->to.id) - 1].uid - 1;
       copy_station_name(last_st);
-      fprintf(pltf,"D\t%f\t%f\t%f\tS%s\tP\t%f\t%f\t%f\t%f\n",
+      if (strcmp(station_name,"VA09") == 0) 
+        fflush(pltf);
+      fprintf(pltf,"D\t%.2f\t%.2f\t%.2f\tS%s\tP\t%.2f\t%.2f\t%.2f\t%.2f\n",
         toft(dbp->db1d.station_vec[last_st].y - avy), toft(dbp->db1d.station_vec[last_st].x - avx), 
         toft(dbp->db1d.station_vec[last_st].z),station_name,
         exppltdim(from_left, to_left), 
@@ -395,8 +400,8 @@ station_name[8] = 0
     }
   }
 
-  fprintf(pltf,"X\t%f\t%f\t%f\t%f\t%f\t%f\n\x1A",toft(ymin-avy),toft(ymax-avy),toft(xmin-avx),toft(xmax-avx),toft(zmin),toft(zmax));
-//  fprintf(pltf,"X\t%f\t%f\t%f\t%f\t%f\t%f\n",toft(ymin-avy),toft(ymax-avy),toft(xmin-avx),toft(xmax-avx),toft(zmin-avz),toft(zmax-avz));
+  fprintf(pltf,"X\t%.2f\t%.2f\t%.2f\t%.2f\t%.2f\t%.2f\n\x1A",toft(ymin-avy),toft(ymax-avy),toft(xmin-avx),toft(xmax-avx),toft(zmin),toft(zmax));
+//  fprintf(pltf,"X\t%.2f\t%.2f\t%.2f\t%.2f\t%.2f\t%.2f\n",toft(ymin-avy),toft(ymax-avy),toft(xmin-avx),toft(xmax-avx),toft(zmin-avz),toft(zmax-avz));
   fclose(pltf);
   
 #ifdef THDEBUG
@@ -409,7 +414,7 @@ station_name[8] = 0
 void thexpmodel::export_thm_file(class thdatabase * dbp)
 {
 
-  char * fnm = this->get_output("cave.thm");
+  const char * fnm = this->get_output("cave.thm");
   
 #ifdef THDEBUG
   thprintf("\n\nwriting %s\n", fnm);
@@ -577,7 +582,7 @@ void thexpmodel::export_thm_file(class thdatabase * dbp)
 
 
 void thexpmodel::export_vrml_file(class thdatabase * dbp) {
-  char * fnm;  
+  const char * fnm;  
 #ifdef THWIN32
   fnm = this->get_output("cave.wrl");
 #else
@@ -823,7 +828,7 @@ void thexpmodel::export_vrml_file(class thdatabase * dbp) {
 
 void thexpmodel::export_3dmf_file(class thdatabase * dbp) {
 
-  char * fnm = this->get_output("cave.3dmf");
+  const char * fnm = this->get_output("cave.3dmf");
   
 #ifdef THDEBUG
   thprintf("\n\nwriting %s\n", fnm);
@@ -1014,7 +1019,7 @@ void thexpmodel::export_3dmf_file(class thdatabase * dbp) {
 
 void thexpmodel::export_dxf_file(class thdatabase * dbp) {
 
-  char * fnm = this->get_output("cave.dxf");
+  const char * fnm = this->get_output("cave.dxf");
   
 #ifdef THDEBUG
   thprintf("\n\nwriting %s\n", fnm);
@@ -1354,7 +1359,7 @@ void thexpmodel::export_dxf_file(class thdatabase * dbp) {
 
 void thexpmodel::export_lox_file(class thdatabase * dbp) {
 
-  char * fnm = this->get_output("cave.lox");
+  const char * fnm = this->get_output("cave.lox");
 
   thdb_object_list_type::iterator obi;
   
@@ -1392,7 +1397,7 @@ void thexpmodel::export_lox_file(class thdatabase * dbp) {
   if (nstat > 0) {
     stnum = new long[nstat];
     for (i = 0; i < nstat; i++)
-      stnum[i] = -1; //(dbp->db1d.station_vec[i].survey->is_selected() ? 1 : -1);
+      stnum[i] = (dbp->db1d.station_vec[i].survey->is_selected() ? 1 : -1); //;-1
   }
 
   // prejde vsetky zamery, ktore ideme 
@@ -1446,13 +1451,11 @@ void thexpmodel::export_lox_file(class thdatabase * dbp) {
       sptr = (thsurvey*)(*obi);
       if (sptr->num1 > 0) {
         sptr->num1 = survnum++;
-
         expf_survey.m_id = sptr->num1;
-        expf_survey.m_parent = (sptr->fsptr != NULL) ? sptr->fsptr->num1 : sptr->num1;
+        expf_survey.m_parent = ((sptr->fsptr == NULL) || (sptr->fsptr->num1 < 0)) ? sptr->num1 : sptr->fsptr->num1;
         expf_survey.m_namePtr = expf.m_surveysData.AppendStr(sptr->name);
         expf_survey.m_titlePtr = expf.m_surveysData.AppendStr(sptr->title);
         expf.m_surveys.push_back(expf_survey);
-
       }
     }
     obi++;
@@ -1483,6 +1486,8 @@ void thexpmodel::export_lox_file(class thdatabase * dbp) {
         expf_station.m_flags |= LXFILE_STATION_FLAG_FIXED;
       if ((pst->flags & TT_STATIONFLAG_CONT) != 0)
         expf_station.m_flags |= LXFILE_STATION_FLAG_CONTINUATION;
+      if ((pst->flags & TT_STATIONFLAG_UNDERGROUND) == 0)
+        expf_station.m_flags |= LXFILE_STATION_FLAG_SURFACE;
       expf.m_stations.push_back(expf_station);
 
       survnum++;
@@ -1501,6 +1506,8 @@ void thexpmodel::export_lox_file(class thdatabase * dbp) {
       expf_shot.m_to = stnum[(*tlegs)->leg->to.id - 1];
       expf_shot.m_surveyId = (*tlegs)->survey->num1;
       expf_shot.m_flags = 0;
+      if (((*tlegs)->leg->flags & TT_LEGFLAG_SPLAY) != 0)
+        expf_shot.m_flags |= LXFILE_SHOT_FLAG_SPLAY;
       if (((*tlegs)->leg->flags & TT_LEGFLAG_SURFACE) != 0)
         expf_shot.m_flags |= LXFILE_SHOT_FLAG_SURFACE;
       if (((*tlegs)->leg->flags & TT_LEGFLAG_DUPLICATE) != 0)
@@ -1699,7 +1706,7 @@ void thexpmodel::export_kml_file(class thdatabase * dbp)
   }
 
   FILE * out;
-  char * fnm = this->get_output("cave.kml");
+  const char * fnm = this->get_output("cave.kml");
   out = fopen(fnm, "w");
   if (out == NULL) {
     thwarning(("can't open %s for output",fnm))
