@@ -136,7 +136,7 @@ void thexpshpf::close()
 }
 
 
-void thexpshpf::init(thexpshp * xshp, char * fnm, int type)
+void thexpshpf::init(thexpshp * xshp, const char * fnm, int type)
 {
   this->m_fnm = fnm;
   this->m_fpath = NULL;
@@ -166,7 +166,7 @@ thexpshp::thexpshp()
 
 
 
-bool thexpshp::open(char * dirname)
+bool thexpshp::open(const char * dirname)
 {
   // create directory
 #ifdef THWIN32
@@ -492,8 +492,9 @@ void thexpshp::xscrap2d(thscrap * scrap, thdb2dxm * xmap, thdb2dxs * xbasic)
     } else {
       this->m_fpoints.point_insert(slp->stx + scrap->proj->rshift_x,  slp->sty + scrap->proj->rshift_y,  slp->stz + scrap->proj->rshift_z);
       this->m_fpoints.object_insert();
-      this->m_flines.m_attributes.insert_attribute("_TYPE","station");
-      this->m_flines.m_attributes.insert_attribute("_NAME",slp->station->name);
+      this->m_fpoints.m_attributes.insert_attribute("_TYPE","station");
+      this->m_fpoints.m_attributes.insert_attribute("_NAME",slp->station->name);
+      this->m_fpoints.m_attributes.insert_attribute("_COMMENT",slp->station->comment);
 		}
     slp = slp->next_item;
   }
@@ -520,10 +521,6 @@ void thexpshp::xscrap2d(thscrap * scrap, thdb2dxm * xmap, thdb2dxs * xbasic)
               (thmatch_string(ppt->subtype, thtt_point_subtypes)) : ppt->m_subtype_str);
         this->m_fpoints.m_attributes.copy_attributes(thdb.attr.get_object(ppt->id));
         switch (ppt->type) {
-          case TT_POINT_TYPE_CONTINUATION:
-            if ((ppt->code != NULL) && (strlen(ppt->code) > 0))
-              this->m_fpoints.m_attributes.insert_attribute("_CODE",ppt->code);
-            break;
           case TT_POINT_TYPE_LABEL:
           case TT_POINT_TYPE_REMARK:
             if ((ppt->text != NULL) && (strlen(ppt->text) > 0))
@@ -602,7 +599,7 @@ void thexpmap::export_shp(class thdb2dxm * maps, class thdb2dprj * prj)
 
   thdb.db2d.process_areas_in_projection(prj);
 
-  char * fnm = this->get_output("cave.shp");
+  const char * fnm = this->get_output("cave.shp");
   thexpshp xs;
   xs.m_xproj = prj;
   xs.m_expmap = this;
@@ -653,7 +650,7 @@ void thexpmap::export_shp(class thdb2dxm * maps, class thdb2dprj * prj)
 void thexpmodel::export_shp_file(class thdatabase * dbp)
 {
 
-  char * fnm = this->get_output("cave.shp");
+  const char * fnm = this->get_output("cave.shp");
   thexpshp xs;
   xs.m_expmodel = this;
   if (!xs.open(fnm)) {
@@ -685,6 +682,7 @@ void thexpmodel::export_shp_file(class thdatabase * dbp)
     xs.m_fstations3D.m_attributes.insert_attribute("_CONTINUATION", long(((st->flags | TT_STATIONFLAG_CONT) != 0) ? 1 : 0));
     if ((st->comment != NULL) && (strlen(st->comment) > 0))
       xs.m_fstations3D.m_attributes.insert_attribute("_COMMENT", st->comment);
+    xs.m_fstations3D.m_attributes.copy_attributes(dbp->db1d.m_station_attr.get_object(i+1));
   }
 
   // shots
@@ -704,6 +702,7 @@ void thexpmodel::export_shp_file(class thdatabase * dbp)
     xs.m_fshots3D.m_attributes.insert_attribute("_SURVEY", lg->leg->psurvey->get_full_name());
     xs.m_fshots3D.m_attributes.insert_attribute("_SURFACE", long(((lg->leg->flags | TT_LEGFLAG_SURFACE) != 0) ? 1 : 0));
     xs.m_fshots3D.m_attributes.insert_attribute("_DUPLICATE", long(((lg->leg->flags | TT_LEGFLAG_DUPLICATE) != 0) ? 1 : 0));
+    xs.m_fshots3D.m_attributes.insert_attribute("_APPROXIMATE", long(((lg->leg->flags | TT_LEGFLAG_APPROXIMATE) != 0) ? 1 : 0));
   }
 
 
