@@ -52,11 +52,14 @@ struct thsst {
 
 thimport::thimport()
 {
-  // replace this by setting real properties initialization
   this->format = 0;
   this->fname = NULL;
   this->surveys = TT_IMPORT_SURVEYS_CREATE;
   this->filter = NULL;
+
+  this->calib_x = 0.0;
+  this->calib_y = 0.0;
+  this->calib_z = 0.0;
 }
 
 
@@ -93,21 +96,21 @@ int thimport::get_cmd_nargs()
 }
 
 
-char * thimport::get_cmd_end()
+const char * thimport::get_cmd_end()
 {
   // insert endcommand if multiline command
   return NULL;
 }
 
 
-char * thimport::get_cmd_name()
+const char * thimport::get_cmd_name()
 {
   // insert command name here
   return "import";
 }
 
 
-thcmd_option_desc thimport::get_cmd_option_desc(char * opts)
+thcmd_option_desc thimport::get_cmd_option_desc(const char * opts)
 {
   int id = thmatch_token(opts, thtt_import_opt);
   if (id == TT_IMPORT_UNKNOWN)
@@ -131,6 +134,10 @@ void thimport::set(thcmd_option_desc cod, char ** args, int argenc, unsigned lon
       if (this->format == TT_IMPORT_FMT_UNKNOWN)
         ththrow(("unknown import format -- %s", args[0]))
       break;
+
+    case TT_IMPORT_CALIB:
+      this->parse_calib(args[0], argenc);
+      break;
     
     case TT_IMPORT_SURVEYS:
       this->surveys = thmatch_token(args[0], thtt_import_surveys);
@@ -179,7 +186,7 @@ void thimport::set_file_name(char * fnm)
   impf_path += "/";
   impf_path += thdb.csrc.name;
   char * pp = impf_path.get_buffer();
-  for(i = strlen(pp); i >= 0; i--) {
+  for(i = (long)strlen(pp); i >= 0; i--) {
     if ((pp[i] == '/') || (pp[i] == '\\')) {
       break;
     } else
@@ -189,7 +196,7 @@ void thimport::set_file_name(char * fnm)
     impf_path = "/";
   impf_path += fnm;
   pp = impf_path.get_buffer();
-  for(i = strlen(pp); i >= 0; i--) {
+  for(i = (long)strlen(pp); i >= 0; i--) {
     if (pp[i] == '\\')
       pp[i] = '/';
   }
@@ -232,6 +239,7 @@ void thimport::import_file()
     this->db->lcsobjectptr = this->db->csurveyptr->loptr;
   tmpsrc = this->db->csrc;
   this->db->csrc = this->mysrc;
+  this->db->csrc.context = this;
   switch (this->format) {
     case TT_IMPORT_FMT_3D:
     case TT_IMPORT_FMT_PLT:
@@ -247,7 +255,7 @@ void thimport::import_file()
 }
 
 
-const char * thimport::station_name(const char * sn, const char separator, class thsst * sst)
+const char * thimport::station_name(const char * sn, const char separator, struct thsst * sst)
 {
 
   sst->survey = NULL;
@@ -297,7 +305,8 @@ const char * thimport::station_name(const char * sn, const char separator, class
           thsplit_strings(&psurv, sn, separator);
           prevsurvey = "";
           int prevctx;
-          thsurvey * csurvey = this->fsptr, * nsurvey, * prevcsptr = this->db->csurveyptr;
+          thsurvey * csurvey = this->fsptr, * nsurvey,
+            * prevcsptr = this->db->csurveyptr;
 //          thendsurvey * nendsurvey;
           for (active_survey = 0; active_survey < (ncs - 1); active_survey++) {
             nsurvey = NULL;
@@ -324,6 +333,8 @@ const char * thimport::station_name(const char * sn, const char separator, class
               // TODO - nastavit mu meno cez set
               nsurvey->name = this->db->strstore(cbf[active_survey]);
               this->db->insert(nsurvey);
+              this->db->csrc.context = this;
+              this->db->csurveylevel--;
 //              nendsurvey = (thendsurvey*) this->db->create("endsurvey", this->mysrc);
 //              this->db->insert(nendsurvey);
               this->db->csurveyptr = prevcsptr;
@@ -371,7 +382,7 @@ const char * thimport::station_name(const char * sn, const char separator, class
       }
       break;      
     default:
-      l = strlen(bx);
+      l = (long)strlen(bx);
       rv = buff;
       for(i = 0; i < l; i++) {
         if ((buff[i] == separator) && ((i + 1) < l)) {
@@ -447,7 +458,11 @@ void thimport::import_file_img()
 
   img_point imgpt;
   int result;
-  char * args [4], * stnm;
+  char * args [4], * stnm, a0[32], a1[32], a2[32], a3[32];
+  args[0] = a0;
+  args[1] = a1;
+  args[2] = a2;
+  args[3] = a3;
   size_t filterl = 0;
   if (this->filter != NULL)
     filterl = strlen(this->filter);
@@ -494,9 +509,9 @@ void thimport::import_file_img()
         if (strlen(stnm) < 1)
           break;
         if (svxs2ths.find(orig_name) == svxs2ths.end()) {
-          sprintf(xb.get_buffer(), "%.16g", imgpt.x);
-          sprintf(yb.get_buffer(), "%.16g", imgpt.y);
-          sprintf(zb.get_buffer(), "%.16g", imgpt.z);
+          sprintf(xb.get_buffer(), "%.16g", imgpt.x + this->calib_x);
+          sprintf(yb.get_buffer(), "%.16g", imgpt.y + this->calib_y);
+          sprintf(zb.get_buffer(), "%.16g", imgpt.z + this->calib_z);
           tmpsurvey = this->db->csurveyptr;
           new_name = this->station_name(stnm, pimg->separator, &tmpsst);
           // thprintf("%s -> %s\n", pimg->label, new_name.c_str());
@@ -527,14 +542,22 @@ void thimport::import_file_img()
           args[2] = yb.get_buffer();
           args[3] = zb.get_buffer();
           args[0] = n1.get_buffer();
+          tmpdata->cs = this->cs;
           tmpdata->set_data_fix(4, args);
           // ak bude entrance, vlozi aj station
           if ((pimg->flags & img_SFLAG_ENTRANCE) != 0) {
             args[0] = n2.get_buffer();
-            args[1] = "";
-            args[2] = "entrance";
+	    args[1] = strcpy(a1, "");
+            args[2] = strcpy(a2, "entrance");
             tmpdata->set_data_station(3, args, TT_UTF_8);
           }
+          if ((pimg->flags & img_SFLAG_FIXED) == 0) {
+            args[0] = n2.get_buffer();
+            args[1] = strcpy(a1, "");
+            args[2] = strcpy(a2, "not");
+            args[3] = strcpy(a3, "fixed");
+            tmpdata->set_data_station(4, args, TT_UTF_8);
+          }
           this->db->csurveyptr = tmpsurvey;
         }
         break;
@@ -638,9 +661,9 @@ void thimport::import_file_img()
     
 //    thprintf("%s - %s = %s - %s in %s\n", s1s.fullname.c_str(), s2s.fullname.c_str(), n1.get_buffer(), n2.get_buffer(), tmpdata->fsptr->full_name);
 
-    args[0] = "nosurvey";
-    args[1] = "from";
-    args[2] = "to";
+    args[0] = strcpy(a0, "nosurvey");
+    args[1] = strcpy(a1, "from");
+    args[2] = strcpy(a2, "to");
     tmpdata->set_data_data(3,args);
 
     args[0] = n1.get_buffer();
@@ -663,4 +686,27 @@ void thimport::import_file_img()
 }
 
 
+void thimport::parse_calib(char * spec, int enc)
+{
+  thmbuffer * mb;
+  char ** args;
+  mb = &this->db->mbuff_tmp;
+  thsplit_args(mb, spec);
+  args = mb->get_buffer();
+  double v[6];
+  int sv, i;
+  if (mb->get_size() != 6)
+    ththrow(("invalid import calibration -- \"%s\"", spec));
+  for(i = 0; i < 6; i++) {
+    thparse_double(sv, v[i], args[i]);
+    if (sv != TT_SV_NUMBER)
+      ththrow(("invalid number -- %s", args[i]))
+  }
+  this->calib_x = v[3] - v[0];
+  this->calib_y = v[4] - v[1];
+  this->calib_z = v[5] - v[2];
+}
+
+
+
 
