@@ -52,6 +52,7 @@ thline::thline()
   this->last_point = NULL;
   
   this->text = NULL;
+  this->m_height = thnan;
 }
 
 
@@ -80,19 +81,19 @@ int thline::get_cmd_nargs()
 }
 
 
-char * thline::get_cmd_end()
+const char * thline::get_cmd_end()
 {
   return "endline";
 }
 
 
-char * thline::get_cmd_name()
+const char * thline::get_cmd_name()
 {
   return "line";
 }
 
 
-thcmd_option_desc thline::get_cmd_option_desc(char * opts)
+thcmd_option_desc thline::get_cmd_option_desc(const char * opts)
 {
   int id = thmatch_token(opts, thtt_line_opt);
   if (id == TT_LINE_UNKNOWN)
@@ -147,6 +148,10 @@ void thline::set(thcmd_option_desc cod, char ** args, int argenc, unsigned long
     case TT_LINE_SUBTYPE:
       this->parse_subtype(*args);
       break;
+
+	case TT_LINE_HEIGHT:
+	  this->parse_height(*args);
+	  break;
     
     case TT_LINE_BORDER:
       this->parse_border(*args);
@@ -323,6 +328,10 @@ void thline::parse_subtype(char * ss)
         case TT_LINE_SUBTYPE_UNDERLYING:
         case TT_LINE_SUBTYPE_UNSURVEYED:
         case TT_LINE_SUBTYPE_PRESUMED:
+        case TT_LINE_SUBTYPE_OVERLYING:
+        case TT_LINE_SUBTYPE_PIT:
+        case TT_LINE_SUBTYPE_MOONMILK:
+        case TT_LINE_SUBTYPE_FLOWSTONE:
           tsok = true;
       }
       break;
@@ -473,7 +482,7 @@ void thline::insert_point_mark(char * ss)
   this->last_point->mark = this->db->strstore(ss,true);
 }
 
-thdb2dlp * thline::get_marked_station(char * mark)
+thdb2dlp * thline::get_marked_station(const char * mark)
 {
   if (strcmp(mark,"end") == 0) {
     if (this->reverse)
@@ -530,7 +539,7 @@ void thline::preprocess()
   if (this->fscrapptr->flip != TT_SCRAP_FLIP_NONE)
     tmpreverse = !tmpreverse;
   
-  if (tmpreverse) {
+  if (tmpreverse && (this->first_point->nextlp != NULL)) {
   
     // najprv prehodi podtypy
     c_item = this->first_point;
@@ -643,11 +652,19 @@ bool thline::export_mp(class thexpmapmpxs * out)
   if (this->first_point == NULL)
     return(false);
   bool postprocess = true, todraw, fsize, frot, anypt;  //, first
-  int from, to, cs, macroid = -1;
+  int from, to, cs, macroid = -1, omacroid = -1;
   double s1, r1;
   thdb2dlp * lp;
 
   th2ddataobject::export_mp(out);
+  if (out->file != NULL) {
+    if (thisnan(this->m_height)) {
+      fprintf(out->file,"ATTR__height := -1;\n");
+    } else {
+      fprintf(out->file,"ATTR__height := %.2f;\n", this->m_height);
+    }
+  }
+
   
   switch (this->type) {
     case TT_LINE_TYPE_WALL:
@@ -672,15 +689,20 @@ bool thline::export_mp(class thexpmapmpxs * out)
             thline_type_export_mp(TT_LINE_SUBTYPE_DEBRIS, SYML_WALL_DEBRIS)
             thline_type_export_mp(TT_LINE_SUBTYPE_BLOCKS, SYML_WALL_BLOCKS)
             thline_type_export_mp(TT_LINE_SUBTYPE_ICE, SYML_WALL_ICE)
+            thline_type_export_mp(TT_LINE_SUBTYPE_FLOWSTONE, SYML_WALL_FLOWSTONE)
+            thline_type_export_mp(TT_LINE_SUBTYPE_MOONMILK, SYML_WALL_MOONMILK)
             thline_type_export_mp(TT_LINE_SUBTYPE_UNDERLYING, SYML_WALL_UNDERLYING)
+            thline_type_export_mp(TT_LINE_SUBTYPE_OVERLYING, SYML_WALL_OVERLYING)
             thline_type_export_mp(TT_LINE_SUBTYPE_UNSURVEYED, SYML_WALL_UNSURVEYED)
             thline_type_export_mp(TT_LINE_SUBTYPE_PRESUMED, SYML_WALL_PRESUMED)
+            thline_type_export_mp(TT_LINE_SUBTYPE_PIT, SYML_WALL_PIT)
           }
           if (this->context >= 0) 
             macroid = this->context;
           if (out->symset->assigned[macroid]) {
             if (out->file == NULL)
               return(true);
+            out->symset->export_mp_symbol_options(out->file, macroid);
             fprintf(out->file,"%s(",out->symset->get_mp_macro(macroid));
             this->export_path_mp(out,from,to);
             fprintf(out->file,");\n");
@@ -703,6 +725,7 @@ bool thline::export_mp(class thexpmapmpxs * out)
 
     case TT_LINE_TYPE_LABEL:
       macroid = SYML_LABEL;
+      omacroid = macroid;
       if (this->context >= 0) 
         macroid = this->context;
       if ((this->text == NULL) || (!out->symset->assigned[macroid])) {
@@ -711,6 +734,7 @@ bool thline::export_mp(class thexpmapmpxs * out)
       } 
       if (out->file == NULL)
         return(true);
+      out->symset->export_mp_symbol_options(out->file, omacroid);
       fprintf(out->file,"l_label(btex ");
       switch (this->scale) {
         case TT_2DOBJ_SCALE_XL:
@@ -729,7 +753,7 @@ bool thline::export_mp(class thexpmapmpxs * out)
           fprintf(out->file,"\\thnormalsize ");
       }
       //thdecode(&(this->db->buff_enc),TT_ISO8859_2,this->text);
-      fprintf(out->file,"%s etex,",utf2tex(this->text, true));
+      fprintf(out->file,"%s etex,",ths2tex(this->text, out->layout->lang, true).c_str());
       this->export_path_mp(out);
       fprintf(out->file,");\n");
       postprocess = false;
@@ -744,6 +768,7 @@ bool thline::export_mp(class thexpmapmpxs * out)
       }
       if (out->file == NULL)
         return(true);
+      out->symset->export_mp_symbol_options(out->file, SYML_CONTOUR);
       fprintf(out->file,"%s(",out->symset->get_mp_macro(SYML_CONTOUR));
       this->export_path_mp(out);
       from = 0;
@@ -794,6 +819,7 @@ bool thline::export_mp(class thexpmapmpxs * out)
       }
       if (out->file == NULL)
         return(true);
+      out->symset->export_mp_symbol_options(out->file, SYML_SLOPE);
       fprintf(out->file,"%s(",out->symset->get_mp_macro(SYML_SLOPE));
       this->export_path_mp(out);
       fprintf(out->file,",%d",
@@ -868,6 +894,7 @@ bool thline::export_mp(class thexpmapmpxs * out)
       }
       if (out->file == NULL)
         return(true);
+      out->symset->export_mp_symbol_options(out->file, SYML_ARROW);
       fprintf(out->file,"%s(",out->symset->get_mp_macro(SYML_ARROW));
       this->export_path_mp(out);
       from = 0;
@@ -889,6 +916,7 @@ bool thline::export_mp(class thexpmapmpxs * out)
       }
       if (out->file == NULL)
         return(true);
+      out->symset->export_mp_symbol_options(out->file, SYML_SECTION);
       fprintf(out->file,"%s(",out->symset->get_mp_macro(SYML_SECTION));
       this->export_path_mp(out);
       anypt = false;
@@ -936,12 +964,14 @@ bool thline::export_mp(class thexpmapmpxs * out)
             thline_type_export_mp(TT_LINE_SUBTYPE_PRESUMED, SYML_BORDER_PRESUMED)
             thline_type_export_mp(TT_LINE_SUBTYPE_INVISIBLE, SYML_BORDER_INVISIBLE)
           }
+          omacroid = macroid;
           if (this->context >= 0) 
             macroid = this->context;
           if (out->symset->assigned[macroid]) {
             if (out->file == NULL)
               return(true);
-            fprintf(out->file,"%s(",out->symset->get_mp_macro(macroid));
+            out->symset->export_mp_symbol_options(out->file, omacroid);
+            fprintf(out->file,"%s(",out->symset->get_mp_macro(omacroid));
             this->export_path_mp(out,from,to);
             fprintf(out->file,");\n");
             if (out->layout->is_debug_joins()) {
@@ -978,12 +1008,14 @@ bool thline::export_mp(class thexpmapmpxs * out)
             thline_type_export_mp(TT_LINE_SUBTYPE_INTERMITTENT, SYML_WATERFLOW_INTERMITTENT)
             thline_type_export_mp(TT_LINE_SUBTYPE_CONJECTURAL, SYML_WATERFLOW_CONJECTURAL)
           }
+          omacroid = macroid;
           if (this->context >= 0) 
             macroid = this->context;
           if (out->symset->assigned[macroid]) {
             if (out->file == NULL)
               return(true);
-            fprintf(out->file,"%s(",out->symset->get_mp_macro(macroid));
+            out->symset->export_mp_symbol_options(out->file, omacroid);
+            fprintf(out->file,"%s(",out->symset->get_mp_macro(omacroid));
             this->export_path_mp(out,from,to);
             fprintf(out->file,");\n");
           }
@@ -1009,12 +1041,14 @@ bool thline::export_mp(class thexpmapmpxs * out)
           switch (cs) {
             thline_type_export_mp(TT_LINE_SUBTYPE_SURFACE, SYML_SURVEY_SURFACE)
           }
+          omacroid = macroid;
           if (this->context >= 0) 
             macroid = this->context;
           if (out->symset->assigned[macroid]) {
             if (out->file == NULL)
               return(true);
-            fprintf(out->file,"%s(",out->symset->get_mp_macro(macroid));
+            out->symset->export_mp_symbol_options(out->file, omacroid);
+            fprintf(out->file,"%s(",out->symset->get_mp_macro(omacroid));
             this->export_path_mp(out,from,to);
             fprintf(out->file,");\n");
           }
@@ -1034,17 +1068,22 @@ bool thline::export_mp(class thexpmapmpxs * out)
       this->export_path_mp(out);
       fprintf(out->file,");\n");
     } else {
+      omacroid = macroid;
       if (this->context >= 0) 
         macroid = this->context;
       if (out->symset->assigned[macroid]) {
         if (out->file == NULL)
           return(true);
-        fprintf(out->file,"%s(",out->symset->get_mp_macro(macroid));
+        if (this->type == TT_LINE_TYPE_U) {
+          out->symset->export_mp_symbol_options(out->file, -1);
+          fprintf(out->file,"l_u_%s(", this->m_subtype_str);
+          this->db->db2d.use_u_symbol(this->get_class_id(), this->m_subtype_str);
+        } else {
+          out->symset->export_mp_symbol_options(out->file, omacroid);
+          fprintf(out->file,"%s(",out->symset->get_mp_macro(omacroid));
+        }
         this->export_path_mp(out);
-        if (this->type == TT_LINE_TYPE_U)
-          fprintf(out->file,",\"%s\");\n", (this->m_subtype_str == NULL) ? "" : this->m_subtype_str);
-        else
-          fprintf(out->file,");\n");
+        fprintf(out->file,");\n");
         if (out->layout->is_debug_joins()) {
           fprintf(out->file,"l_debug(1,0,");
           this->export_path_mp(out,0,-1,1);
@@ -1059,6 +1098,7 @@ bool thline::export_mp(class thexpmapmpxs * out)
     }
   }
 
+  th2ddataobject::export_mp_end(out);
   return(false);
 }
 
@@ -1256,7 +1296,7 @@ void thline::parse_size(int w, char * ss) {
   double sz;
   if (this->last_point == NULL)
     ththrow(("no line point specified"))
-  char * sizestr = NULL;
+  const char * sizestr = NULL;
   switch (w) {
     case TT_LINE_SIZE:
       sizestr = "size";
@@ -1324,6 +1364,7 @@ void thline::start_insert() {
     case TT_LINE_TYPE_U:
       if (this->m_subtype_str == NULL)
         ththrow(("missing subtype specification for line of user defined type"))
+      this->db->db2d.register_u_symbol(this->get_class_id(), this->m_subtype_str);
       break;
     case TT_LINE_TYPE_SLOPE:
       lp = this->first_point;
@@ -1345,4 +1386,50 @@ void thline::start_insert() {
 
 
 
+void thline::parse_height(char * ss) {
+  switch (this->type) {
+    case TT_LINE_TYPE_PIT:
+      break;
+	case TT_LINE_TYPE_WALL:
+	  if (this->csubtype == TT_LINE_SUBTYPE_PIT)
+		  break;
+    default:
+      ththrow(("-height not valid with type %s", thmatch_string(this->type,thtt_line_types)))
+      break;
+  }
+
+  thsplit_words(& this->db->db2d.mbf,ss);
+  int npar = this->db->db2d.mbf.get_size();
+  char ** pars = this->db->db2d.mbf.get_buffer();
+  int sv, ux;
+  double dv;
+  thtflength lentf;
+  
+  ux = 0;
+  switch (npar) {
+    case 1:
+      break;
+    case 2:
+      ux = 1;
+      break;
+    default:
+      ththrow(("invalid distance -- %s",ss))
+  }
+  this->m_height = thnan;
+  thparse_double(sv,dv,pars[0]);
+  if (sv != TT_SV_NUMBER)
+    ththrow(("not a number -- %s", pars[0]))
+  if (dv <= 0.0)
+	ththrow(("nor a positivie number -- %s", pars[0]))
+  if (ux > 0) {
+    lentf.parse_units(pars[ux]);
+    dv = lentf.transform(dv);
+  }
+  this->m_height = dv;
+
+}
+
+
+
+
 
