@@ -25,7 +25,7 @@
  * --------------------------------------------------------------------
  */
  
-// #include <iomanip>
+#include <iomanip>
 #include <iostream>
 #include <sstream>
 #include <fstream>
@@ -34,6 +34,7 @@
 #include <map>
 #include <set>
 #include <string>
+#include <climits>
 
 // #include <cstring>
 #include <cstdio>
@@ -50,6 +51,10 @@
 #include "thlang.h"
 #include "thversion.h"
 
+#ifdef THMSVC
+#define round(x) floor((x) + 0.5)
+#endif
+
 #ifndef NOTHERION
 #include "thchenc.h"
 #include "thbuffer.h"
@@ -57,6 +62,7 @@
 
 using namespace std;
 
+#define IOerr(F) ((string)"Can't open file "+F+"!\n").c_str()
 
 
 ///////////
@@ -252,6 +258,7 @@ list<sheetrecord>::iterator find_sheet(int x, int y, int z) {
 
 void make_sheets() {
   double llx = 0, lly = 0, urx = 0, ury = 0;
+  double a,b,w,h;
   sheetrecord SHREC;
 
   if (mode == ATLAS) {
@@ -303,6 +310,21 @@ void make_sheets() {
       if (I->X3 > urx) urx = I->X3;
       if (I->X4 > ury) ury = I->X4;
     }
+    for (list<surfpictrecord>::iterator I_sk = I->SKETCHLIST.begin();
+                                          I_sk != I->SKETCHLIST.end(); I_sk++) {
+      for (int i = 0; i<=1; i++) {
+        for (int j = 0; j<=1; j++) {
+          w = i * I_sk->width;
+          h = j * I_sk->height;
+          a = I_sk->xx*w + I_sk->xy*h + I_sk->dx;
+          b = I_sk->yx*w + I_sk->yy*h + I_sk->dy;
+          if (a < llx) llx = a;
+          if (b < lly) lly = b;
+          if (a > urx) urx = a;
+          if (b > ury) ury = b;
+        }
+      }
+    };
 
     if (llx == DBL_MAX || lly == DBL_MAX || urx == -DBL_MAX || ury == -DBL_MAX) 
       therror(("This can't happen -- no data for a scrap!"));
@@ -487,6 +509,7 @@ set<int> find_excluded_pages(string s) {
 //  istrstream S(s.c_str());
   istringstream S(s);
   
+  c = ',';
   while (S >> i) {
     S >> c;
     if (c == ',') excl.insert(i);
@@ -833,7 +856,7 @@ void print_surface_bitmaps (ofstream &PAGEDEF, double shiftx, double shifty) {
   PAGEDEF.precision(6);
   for (list<surfpictrecord>::iterator I = SURFPICTLIST.begin();
                                       I != SURFPICTLIST.end(); I++) {
-    if (mode == MAP && LAYOUT.transparency || LAYOUT.OCG) {
+    if (mode == MAP && (LAYOUT.transparency || LAYOUT.OCG)) {
       PAGEDEF << "\\bitmapcorr{";
     } else {
       PAGEDEF << "\\bitmap{";
@@ -874,15 +897,21 @@ paired rotatedaround(paired x,paired o, double th) {
 }
 
 
-void print_grid(ofstream& PAGEDEF, double LLX,double LLY) {
+void print_grid_pdf(ofstream& PAGEDEF, double LLX, double LLY, double origMINX,double origMINY,double origMAXX, double origMAXY) {
   if (LAYOUT.grid == 0) return;
   PAGEDEF << "\\PL{q}";
   
   paired ll, ur, lr, ul, llrot, urrot, ulrot, lrrot, llnew, urnew, origin;
-  ll.x = LLX;
+/*  ll.x = LLX;
   ll.y = LLY; 
   ur.x = LLX + HS;
-  ur.y = LLY + VS; 
+  ur.y = LLY + VS;  */
+
+  ll.x = origMINX;
+  ll.y = origMINY; 
+  ur.x = origMAXX;
+  ur.y = origMAXY; 
+
   lr.x = ur.x;
   lr.y = ll.y;
   ul.x = ll.x;
@@ -922,6 +951,9 @@ PAGEDEF << "\\PL{ " << rotatedaround(urnew,origin,LAYOUT.gridrot).x-LLX << " " <
 
   double grid_init_x = LAYOUT.hgridsize * floor ((llnew.x-origin.x)/LAYOUT.hgridsize) + origin.x;
   double grid_init_y = LAYOUT.vgridsize * floor ((llnew.y-origin.y)/LAYOUT.vgridsize) + origin.y;
+
+  double G_real_init_x = LAYOUT.XO + LAYOUT.XS * floor ((llnew.x-origin.x)/LAYOUT.hgridsize);
+  double G_real_init_y = LAYOUT.YO + LAYOUT.YS * floor ((llnew.y-origin.y)/LAYOUT.vgridsize);
   
   double cosr = cos(-LAYOUT.gridrot * 3.14159265 / 180);
   double sinr = sin(-LAYOUT.gridrot * 3.14159265 / 180);
@@ -930,11 +962,13 @@ PAGEDEF << "\\PL{ " << rotatedaround(urnew,origin,LAYOUT.gridrot).x-LLX << " " <
 
   if (LAYOUT.proj == 0) {
     paired out,tmp;
-    for (double i = grid_init_x; i < urnew.x + LAYOUT.hgridsize - 0.05; i += LAYOUT.hgridsize) {
-      for (double j = grid_init_y; j < urnew.y + LAYOUT.vgridsize - 0.05; j += LAYOUT.vgridsize) {
-        row = (i == grid_init_x ? 0 : (i >= urnew.x ? 2 : 1));
-        col = (j == grid_init_y ? 0 : (j >= urnew.y ? 2 : 1));
-	elem = row + 3*col;
+    int ii,jj;
+    double i, j;
+    for (i = grid_init_x, ii=0; i < urnew.x + LAYOUT.hgridsize - 0.05; i += LAYOUT.hgridsize, ii++) {
+      for (j = grid_init_y, jj=0; j < urnew.y + LAYOUT.vgridsize - 0.05; j += LAYOUT.vgridsize, jj++) {
+        col = (i == grid_init_x ? 0 : (i >= urnew.x ? 2 : 1));
+        row = (j == grid_init_y ? 0 : (j >= urnew.y ? 2 : 1));
+	elem = col + 3*row;
 /*        tmp.x = i;
         tmp.y = j;
         out = rotatedaround(tmp,origin,LAYOUT.gridrot);
@@ -949,22 +983,58 @@ PAGEDEF << "\\PL{ " << rotatedaround(urnew,origin,LAYOUT.gridrot).x-LLX << " " <
         out.y -= LLY;
         PAGEDEF << "\\PL{q}";
 	PAGEDEF << "\\PL{" << cosr << " " << sinr << " " << -sinr << " " << cosr << " " << out.x << " " << out.y << " cm}";
-	PAGEDEF << "\\PB{0}{0}{\\" << tex_Wname("grid") << u2str(elem+1) << "}%" << endl;
-        PAGEDEF << "\\PL{Q}";
+	PAGEDEF << "\\PB{0}{0}{\\" << tex_Wname("grid") << u2str(elem+1) << "}";
+        PAGEDEF << "\\PL{Q}%" << endl;
+
+        if (LAYOUT.grid_coord_freq==2 || (LAYOUT.grid_coord_freq==1 && elem!=4)) {
+          tmp.x = i;
+          tmp.y = j;
+          out = rotatedaround(tmp,origin,LAYOUT.gridrot);
+          out.x -= LLX;
+          out.y -= LLY;
+          PAGEDEF << "\\PL{q}";
+          PAGEDEF << "\\PL{" << cosr << " " << sinr << " " << -sinr << " " << cosr << " " << out.x << " " << out.y << " cm}";
+	  PAGEDEF << "\\gridcoord{" << (row == 2 ? (col == 2 ? 1 : 3) : (col == 2 ? 7 : 9)) << 
+	      "}{$(" << setprecision(0) << 
+	      G_real_init_x+ii*LAYOUT.XS << "," << 
+              G_real_init_y+jj*LAYOUT.YS << setprecision(2) << ")$}%" << endl;
+          PAGEDEF << "\\PL{Q}";
+        }
+
       }
     }
   }
   else {
     grid_init_x = LLX;
-    for (double j = grid_init_y; j < urnew.y + LAYOUT.vgridsize - 0.05; j += LAYOUT.vgridsize) {
+    int jj;
+    double j;
+    for (j = grid_init_y,jj=0; j < urnew.y + LAYOUT.vgridsize - 0.05; j += LAYOUT.vgridsize,jj++) {
 //      PAGEDEF << "\\PL{q 3 w 0 0 1 RG 0 " << j-LLY << "  m " << HS << " " << j-LLY << " l S Q}";
       for (double i = grid_init_x; i < urnew.x + LAYOUT.hgridsize - 0.05; i += LAYOUT.hgridsize) {
-        row = (i == grid_init_x ? 0 : (i >= urnew.x ? 2 : 1));
-        col = (j == grid_init_y ? 0 : (j >= urnew.y ? 2 : 1));
-	elem = row + 3*col;
+        col = (i == grid_init_x ? 0 : (i >= urnew.x ? 2 : 1));
+        row = (j == grid_init_y ? 0 : (j >= urnew.y ? 2 : 1));
+	elem = col + 3*row;
 	PAGEDEF << "\\PB{" << i-LLX+LAYOUT.gridcell[elem].x << "}{" << 
 	                      j-LLY+LAYOUT.gridcell[elem].y << "}{\\" << 
 			      tex_Wname("grid") << u2str(elem+1) << "}%" << endl;
+
+        if (col == 0 && LAYOUT.grid_coord_freq > 0) {
+          PAGEDEF << "\\PL{q}";
+          PAGEDEF << "\\PL{1 0 0 1 " << i-LLX << " " << j-LLY << " cm}";
+          PAGEDEF << "\\gridcoord{" << (row==2?3:9) << "}{$" << 
+	      setprecision(0) << G_real_init_y+jj*LAYOUT.YS << 
+	      setprecision(2)<< "$}";
+          PAGEDEF << "\\PL{Q}%" << endl;
+	}
+        if (col == 2 && LAYOUT.grid_coord_freq == 2) {
+          PAGEDEF << "\\PL{q}";
+          PAGEDEF << "\\PL{1 0 0 1 " << i-LLX << " " << j-LLY << " cm}";
+          PAGEDEF << "\\gridcoord{" << (row==2?1:7) << "}{$" << 
+	      setprecision(0) << G_real_init_y+jj*LAYOUT.YS <<
+	      setprecision(2)<< "$}";
+          PAGEDEF << "\\PL{Q}%" << endl;
+        }
+
       }
     }
   } 
@@ -1006,7 +1076,7 @@ void print_map(int layer, ofstream& PAGEDEF,
     print_page_bg(PAGEDEF);
     if (LAYOUT.surface == 1) print_surface_bitmaps(PAGEDEF,HSHIFT,VSHIFT);
     print_page_bg_scraps(layer, PAGEDEF, sheet_it);
-    if (LAYOUT.grid == 1) print_grid(PAGEDEF,HSHIFT,VSHIFT);
+    if (LAYOUT.grid == 1) print_grid_pdf(PAGEDEF,HSHIFT,VSHIFT,HSHIFT,VSHIFT,HSHIFT+HS,VSHIFT+VS);
   }
 
   if (mode == ATLAS && !LAYERHASH.find(layer)->second.D.empty()) {
@@ -1044,6 +1114,21 @@ void print_map(int layer, ofstream& PAGEDEF,
         PAGEDEF << "\\PL{Q}%" << endl;            // end of white color for filled bg
       }
     }
+    
+    // sketches
+    PAGEDEF.precision(6);
+    for (list<scraprecord>::iterator K = SCRAPLIST.begin(); K != SCRAPLIST.end(); K++) {
+      for (list<surfpictrecord>::iterator I_sk = K->SKETCHLIST.begin();
+                                          I_sk != K->SKETCHLIST.end(); I_sk++) {
+        PAGEDEF << "\\pdfximage{" << (string) I_sk->filename << "}%" << endl;
+        PAGEDEF << "\\bitmap{" <<
+            I_sk->xx << "}{" << I_sk->yx << "}{" << I_sk->xy << "}{" << I_sk->yy << "}{" << 
+            I_sk->dx - HSHIFT << "}{" << I_sk->dy - VSHIFT  << 
+            "}{\\pdflastximage}%" << endl;
+      };
+    }
+    PAGEDEF.precision(2);
+    // end of sketches
 
     for (list<scraprecord>::iterator K = SCRAPLIST.begin(); K != SCRAPLIST.end(); K++) {
       if (used_scraps.count(K->name) > 0 && K->G != "") {
@@ -1067,7 +1152,7 @@ void print_map(int layer, ofstream& PAGEDEF,
           K->P != "" && K->level >= (I->first)) {
         xc = HSHIFT - K->S1; yc = VSHIFT - K->S2;
         ifstream G((K->P).c_str());
-        if(!G) therror(("Can't open file"));
+        if(!G) therror((IOerr(K->P)));
         while(G >> buffer) {
           if ((buffer == "m") || (buffer == "l") || (buffer == "c")) {
             PAGEDEF << "\\PL{"; 
@@ -1132,7 +1217,7 @@ void print_map(int layer, ofstream& PAGEDEF,
 
   if (mode == ATLAS) {
     if (LAYOUT.surface == 2) print_surface_bitmaps(PAGEDEF,HSHIFT,VSHIFT);
-    if (LAYOUT.grid == 2) print_grid(PAGEDEF,HSHIFT,VSHIFT);
+    if (LAYOUT.grid == 2) print_grid_pdf(PAGEDEF,HSHIFT,VSHIFT,HSHIFT,VSHIFT,HSHIFT+HS,VSHIFT+VS);
   }
 }
 
@@ -1288,17 +1373,17 @@ void print_margins(ofstream& PAGEDEF) {
 void build_pages() {
   
   ofstream PAGEDEF("th_pagedef.tex");
-  if(!PAGEDEF) therror(("Can't write file"));
+  if(!PAGEDEF) therror(("Can't write file th_pagedef.tex"));
   PAGEDEF.setf(ios::fixed, ios::floatfield);
   PAGEDEF.precision(2);
 
   ofstream PAGE("th_pages.tex");
-  if(!PAGE) therror(("Can't write file"));
+  if(!PAGE) therror(("Can't write file th_pages.tex"));
   PAGE.setf(ios::fixed, ios::floatfield);
   PAGE.precision(2);
 
   ofstream PDFRES("th_resources.tex");
-  if(!PDFRES) therror(("Can't write file"));
+  if(!PDFRES) therror(("Can't write file th_resources.tex"));
   if (LAYOUT.transparency || LAYOUT.OCG) {
     PDFRES << "\\ifnum\\pdftexversion<110\\pdfcatalog{ /Version /" <<
       (LAYOUT.OCG ? "1.5" : "1.4") << " }" << 
@@ -1306,8 +1391,15 @@ void build_pages() {
   }
   
   PDFRES << "\\pdfinfo{/Creator (Therion " << THVERSION << ", MetaPost, TeX)}%" << endl;
-  PDFRES << "\\pdfcatalog{ /ViewerPreferences << /DisplayDocTitle true >> }" << endl;
+  PDFRES << "\\pdfcatalog{ /ViewerPreferences << /DisplayDocTitle true /PrintScaling /None >> }" << endl;
   
+  if(ENC_NEW.NFSS != 0) PDFRES << "\\input thfonts.map" << endl;
+
+  PDFRES << "\\ifnum\\pdftexversion>139" << endl;
+  PDFRES << "  \\newread\\testin" << endl;
+  PDFRES << "  \\openin\\testin=glyphtounicode.tex" << endl;
+  PDFRES << "  \\ifeof\\testin\\message{No glyph to unicode mapping found!}\\else\\closein\\testin\\input glyphtounicode.tex\\pdfgentounicode=1\\fi" << endl;
+  PDFRES << "\\fi" << endl;
 
   if (LAYOUT.transparency) {
     PDFRES << "\\opacity{" << LAYOUT.opacity << "}%" << endl;
@@ -1396,8 +1488,6 @@ void build_pages() {
 //    PDFRES << "\\legendcomment={" << utf2tex(LAYOUT.doc_comment) << "}" << endl;
 //  }
 
-  PDFRES << "\\pdfcatalog { /TeXsetup <" << pdf_info() << "> }" << endl;
-
   if (!LEGENDLIST.empty()) {  // zmenit test na LAYOUT.legend???
     PDFRES << "\\legendtrue" << endl;
   }
@@ -1434,6 +1524,8 @@ void build_pages() {
     i++;
   }
 
+  double origMINX=0, origMINY=0, origMAXX=0, origMAXY=0;
+
   if (mode == ATLAS) {
     HS = LAYOUT.hsize + 2*LAYOUT.overlap;
     VS = LAYOUT.vsize + 2*LAYOUT.overlap;
@@ -1446,7 +1538,9 @@ void build_pages() {
       MINY = LAYOUT.vgridsize * floor ((MINY-LAYOUT.vgridorigin)/LAYOUT.vgridsize) + LAYOUT.vgridorigin;
       MAXY = LAYOUT.vgridsize * ceil  ((MAXY-LAYOUT.vgridorigin)/LAYOUT.vgridsize) + LAYOUT.vgridorigin;
     }
+    origMINX = MINX; origMINY = MINY; origMAXX = MAXX; origMAXY = MAXY;
     if (LAYOUT.map_grid) {
+//      origMINX = MINX; origMINY = MINY; origMAXX = MAXX; origMAXY = MAXY;
       MINX = LAYOUT.hsize * floor (MINX/LAYOUT.hsize);
       MINY = LAYOUT.vsize * floor (MINY/LAYOUT.vsize);
       MAXX = LAYOUT.hsize * ceil (MAXX/LAYOUT.hsize);
@@ -1502,9 +1596,10 @@ void build_pages() {
     PAGEDEF << "\\advance\\x by \\extraW" << endl;
     PAGEDEF << "\\advance\\y by \\extraN" << endl;
     PAGEDEF << "\\advance\\y by \\extraS" << endl;
-    PAGEDEF << "\\wd\\xxx=\\x" << endl;
-    PAGEDEF << "\\ht\\xxx=\\y" << endl;
-    PAGEDEF << "\\immediate\\pdfxform\\xxx" << endl;
+    PAGEDEF << "\\newbox\\xxxx\\setbox\\xxxx=\\hbox to \\x{\\kern\\extraW\\raise\\extraS\\box\\xxx\\hss}%\\dp\\xxx=0bp" << endl;
+    PAGEDEF << "\\wd\\xxxx=\\x" << endl;
+    PAGEDEF << "\\ht\\xxxx=\\y" << endl;
+    PAGEDEF << "\\immediate\\pdfxform\\xxxx" << endl;
     PAGEDEF << "\\newcount\\THmaplegend\\THmaplegend=\\pdflastxform" << endl;
 
     PAGEDEF << "\\advance\\pdfhorigin by \\extraW" << endl;
@@ -1523,6 +1618,18 @@ void build_pages() {
     PAGEDEF << "\\x=\\extraW\\advance\\x by \\overlap" << endl;
     PAGEDEF << "\\dimtobp{\\the\\x}\\edef\\wsize{\\tmpdef}%" << endl;
 
+    PAGEDEF << "\\newdimen\\overlaphalf\\overlaphalf=\\overlap\\divide\\overlaphalf by 2%" << endl;
+    PAGEDEF << "\\newdimen\\framew\\framew=\\pdfpagewidth\\advance\\framew by -\\overlap" << endl;
+    PAGEDEF << "\\newdimen\\frameh\\frameh=\\pdfpageheight\\advance\\frameh by -\\overlap" << endl;
+    PAGEDEF << "\\newdimen\\framex\\framex=\\extraW\\advance\\framex by \\overlaphalf" << endl;
+    PAGEDEF << "\\newdimen\\framey\\framey=\\extraN\\advance\\framey by \\overlaphalf" << endl;
+
+    PAGEDEF << "\\dimtobp{\\framew}\\edef\\Framew{\\tmpdef}%" << endl;
+    PAGEDEF << "\\dimtobp{\\frameh}\\edef\\Frameh{\\tmpdef}%" << endl;
+    PAGEDEF << "\\dimtobp{\\framex}\\edef\\Framex{\\tmpdef}%" << endl;
+    PAGEDEF << "\\dimtobp{\\framey}\\edef\\Framey{\\tmpdef}%" << endl;
+
+
     PAGEDEF << "\\adjustedHS=" << HS << "bp" <<
       "\\advance\\adjustedHS by \\extraE" << 
       "\\advance\\adjustedHS by \\extraW" <<
@@ -1535,6 +1642,19 @@ void build_pages() {
       "\\advance\\adjustedVS by \\overlap" << 
       "\\advance\\adjustedVS by \\overlap" << endl;
 
+    //calibration
+    PAGEDEF.precision(10);
+    for (int i=0; i<9; i++) {
+      PAGEDEF << "\\calibrX{" << LAYOUT.calibration_local[i].x - MINX << "bp}\\tmpdimenX=\\tmpdimen" << endl;
+      PAGEDEF << "\\calibrY{" << LAYOUT.calibration_local[i].y - MINY << "bp}\\tmpdimenY=\\tmpdimen" << endl;
+    
+      PAGEDEF << "\\pdfcatalog { /thCalibrate" << i << 
+                 " (X=\\the\\tmpdimenX, Y=\\the\\tmpdimenY, L=" << LAYOUT.calibration_latlong[i].x << 
+                 ", F=" << LAYOUT.calibration_latlong[i].y << ")}" << endl;
+    }
+    PAGEDEF << "\\pdfcatalog { /thCalibrate (HS=\\the\\adjustedHS, VS=\\the\\adjustedVS, HD=" << LAYOUT.calibration_hdist << ")}";
+    PAGEDEF.precision(2);
+
     PAGEDEF << "\\tmpdimen=\\extraW\\advance\\tmpdimen by \\overlap" << endl;
     PAGEDEF << "\\dimtobp{\\tmpdimen}\\edef\\adjustedX{\\tmpdef}%" << endl;
     PAGEDEF << "\\tmpdimen=\\extraS\\advance\\tmpdimen by \\overlap" << endl;
@@ -1549,6 +1669,12 @@ void build_pages() {
 			    "\\ysize\\space" << 
                             " re f Q}%" << endl;
 
+    PAGEDEF << "\\ifdim\\framethickness>0mm\\dimtobp{\\framethickness}\\edef\\Framethickness{\\tmpdef}" <<
+               "\\PL{q 0 0 0 RG 1 J 1 j \\Framethickness\\space w " << 
+               "-\\Framex\\space\\Framey\\space\\Framew\\space-\\Frameh\\space" << 
+               " re s Q}\\fi" << endl;
+
+
 //    PAGEDEF << "\\leavevmode\\setbox\\xxx=\\hbox to " << HS << "bp{%" << endl;
     PAGEDEF << "\\leavevmode\\setbox\\xxx=\\hbox to 0bp{%" << endl;
 
@@ -1568,7 +1694,7 @@ void build_pages() {
       }
     }
 
-    if (LAYOUT.grid == 1) print_grid(PAGEDEF,MINX,MINY);
+    if (LAYOUT.grid == 1) print_grid_pdf(PAGEDEF,MINX,MINY,origMINX,origMINY,origMAXX,origMAXY);
 
     if (!MAP_PREVIEW_DOWN.empty()) print_preview(0,PAGEDEF,MINX,MINY);
     for (map<int,layerrecord>::iterator I = LAYERHASH.begin();
@@ -1591,19 +1717,34 @@ void build_pages() {
     if (!MAP_PREVIEW_UP.empty()) print_preview(1,PAGEDEF,MINX,MINY);
 
     if (LAYOUT.surface == 2) print_surface_bitmaps(PAGEDEF,MINX,MINY);
-    if (LAYOUT.grid == 2) print_grid(PAGEDEF,MINX,MINY);
+    if (LAYOUT.grid == 2) print_grid_pdf(PAGEDEF,MINX,MINY,origMINX,origMINY,origMAXX,origMAXY);
 
     if (LAYOUT.map_grid) {
-      PAGEDEF << "\\PL{q .4 w}%" << endl;
+      PAGEDEF << "\\PL{q .4 w 0.6 g 0.6 G }%" << endl;
       PAGEDEF << "\\PL{0 0 " << HS << " " << VS << " re S}%" << endl;
       for (double i=0; i <= HS; i += LAYOUT.hsize) {
         PAGEDEF << "\\PL{" << i << " 0 m " << i << " " << VS << " l S}%" << endl;
+	if (i<HS) {
+          PAGEDEF << "\\PL{q 1 0 0 1 " << i+LAYOUT.hsize/2 << 
+	    " 0 cm}\\gridcoord{8}{\\size[24]" << grid_name(LAYOUT.labelx,round((MINX+i)/LAYOUT.hsize)) << 
+	    "}\\PL{Q}%" << endl;
+          PAGEDEF << "\\PL{q 1 0 0 1 " << i+LAYOUT.hsize/2 << " " << VS <<
+	    " cm}\\gridcoord{2}{\\size[24]" << grid_name(LAYOUT.labelx,round((MINX+i)/LAYOUT.hsize)) << 
+	    "}\\PL{Q}%" << endl;
+	}
       }
       for (double i=0; i <= VS; i += LAYOUT.vsize) {
         PAGEDEF << "\\PL{0 " << i << " m " << HS << " " << i << " l S}%" << endl;
+	if (i<VS) {
+          PAGEDEF << "\\PL{q 1 0 0 1 0 " << i+LAYOUT.vsize/2 << 
+    	    " cm}\\gridcoord{6}{\\size[24]" << grid_name(LAYOUT.labely,round((MINY+VS-i)/LAYOUT.vsize)-1) << 
+	    "}\\PL{Q}%" << endl;
+          PAGEDEF << "\\PL{q 1 0 0 1 " << HS << " " << i+LAYOUT.vsize/2 << 
+	    " cm}\\gridcoord{4}{\\size[24]" << grid_name(LAYOUT.labely,round((MINY+VS-i)/LAYOUT.vsize)-1) << 
+	    "}\\PL{Q}%" << endl;
+	}
       }
       PAGEDEF << "\\PL{Q}%" << endl;
-
     }
 ////    PAGEDEF << "\\setbox\\xxx=\\hbox to " << HS << "bp{";      // map legend
 ////    PAGEDEF << "\\maplayout\\hfill}\\ht\\xxx=" << VS << "bp\\dp\\xxx=0bp" << endl;
