@@ -56,7 +56,6 @@ thpoint::thpoint()
   this->align = TT_POINT_ALIGN_C;
   
   this->text = NULL;
-  this->code = NULL;
 }
 
 
@@ -92,21 +91,21 @@ int thpoint::get_cmd_nargs()
 }
 
 
-char * thpoint::get_cmd_end()
+const char * thpoint::get_cmd_end()
 {
   // insert endcommand if multiline command
   return NULL;
 }
 
 
-char * thpoint::get_cmd_name()
+const char * thpoint::get_cmd_name()
 {
   // insert command name here
   return "point";
 }
 
 
-thcmd_option_desc thpoint::get_cmd_option_desc(char * opts)
+thcmd_option_desc thpoint::get_cmd_option_desc(const char * opts)
 {
   int id = thmatch_token(opts, thtt_point_opt);
   if (id == TT_POINT_UNKNOWN)
@@ -120,6 +119,7 @@ void thpoint::start_insert() {
   if (this->type == TT_POINT_TYPE_U) {
     if (this->m_subtype_str == NULL)
       ththrow(("missing subtype specification for point of user defined type"))
+    this->db->db2d.register_u_symbol(this->get_class_id(), this->m_subtype_str);
   }
 }
 
@@ -167,9 +167,8 @@ void thpoint::set(thcmd_option_desc cod, char ** args, int argenc, unsigned long
       this->parse_text(this->db->buff_enc.get_buffer());
       break;
 
-    case TT_POINT_CODE:
-      thencode(&(this->db->buff_enc), *args, argenc);
-      this->parse_code(this->db->buff_enc.get_buffer());
+    case TT_POINT_EXPLORED:
+      this->parse_explored(*args);
       break;
 
     case TT_POINT_ALIGN:
@@ -368,7 +367,7 @@ void thpoint::parse_from(char * estr)
 }
 
 
-char * thpoint_export_mp_align2mp(int a) {
+const char * thpoint_export_mp_align2mp(int a) {
   switch (a) {
     case TT_POINT_ALIGN_R:
       return ".rt";
@@ -395,10 +394,12 @@ char * thpoint_export_mp_align2mp(int a) {
 bool thpoint::export_mp(class thexpmapmpxs * out)
 {
   th2ddataobject::export_mp(out);
+  thattr * station_attr = NULL;
+  long station_attr_id = 0;
 
-  bool postprocess = true;
+  bool postprocess = true, expsym = false;
   std::string attr_text;
-  int macroid = -1;
+  int macroid = -1, omacroid = -1, cmark;
   int postprocess_label = -1;
   this->db->buff_enc.guarantee(8128);
 //  char * buff = this->db->buff_enc.get_buffer();
@@ -434,6 +435,7 @@ bool thpoint::export_mp(class thexpmapmpxs * out)
         if (out->file == NULL)
           return(true);
         out->symset->get_mp_macro(macroid);
+        out->symset->export_mp_symbol_options(out->file, macroid);
         fprintf(out->file,"p_label%s(btex ",thpoint_export_mp_align2mp(thdb2d_rotate_align(this->align, xrr)));
         switch (this->type) {
           case TT_POINT_TYPE_STATION_NAME:
@@ -441,28 +443,28 @@ bool thpoint::export_mp(class thexpmapmpxs * out)
             break;
           case TT_POINT_TYPE_LABEL:
             fprintf(out->file,"\\thlabel");
-            switch (this->scale) {
-              case TT_2DOBJ_SCALE_XL:
-                fprintf(out->file,"\\thhugesize ");
-                break;
-              case TT_2DOBJ_SCALE_L:
-                fprintf(out->file,"\\thlargesize ");
-                break;
-              case TT_2DOBJ_SCALE_S:
-                fprintf(out->file,"\\thsmallsize ");
-                break;
-              case TT_2DOBJ_SCALE_XS:
-                fprintf(out->file,"\\thtinysize ");
-                break;
-              default:
-                fprintf(out->file,"\\thnormalsize ");
-            }
             break;
           case TT_POINT_TYPE_REMARK:
             fprintf(out->file,"\\thremark ");
             break;
         }
-        fprintf(out->file,"%s etex,",utf2tex(this->text));        
+        switch (this->scale) {
+          case TT_2DOBJ_SCALE_XL:
+            fprintf(out->file,"\\thhugesize ");
+            break;
+          case TT_2DOBJ_SCALE_L:
+            fprintf(out->file,"\\thlargesize ");
+            break;
+          case TT_2DOBJ_SCALE_S:
+            fprintf(out->file,"\\thsmallsize ");
+            break;
+          case TT_2DOBJ_SCALE_XS:
+            fprintf(out->file,"\\thtinysize ");
+            break;
+          default:
+            fprintf(out->file,"\\thnormalsize ");
+        }
+        fprintf(out->file,"%s etex,",ths2tex(this->text, out->layout->lang).c_str());        
         if (this->type == TT_POINT_TYPE_STATION_NAME)
           postprocess_label = 7;
         else
@@ -475,30 +477,87 @@ bool thpoint::export_mp(class thexpmapmpxs * out)
       switch (this->subtype) {
         case TT_POINT_SUBTYPE_FIXED:
           macroid = SYMP_STATION_FIXED;
+          cmark = TT_DATAMARK_FIXED;
           break;
         case TT_POINT_SUBTYPE_NATURAL:
           macroid = SYMP_STATION_NATURAL;
+          cmark = TT_DATAMARK_NATURAL;
           break;
         case TT_POINT_SUBTYPE_PAINTED:
           macroid = SYMP_STATION_PAINTED;
+          cmark = TT_DATAMARK_PAINTED;
           break;
         default:
           macroid = SYMP_STATION_TEMPORARY;
+          cmark = TT_DATAMARK_TEMP;
       }
+      
+      expsym = out->symset->assigned[macroid];
+      omacroid = macroid;
+
       if (this->context >= 0) 
         macroid = this->context;
-      if (out->symset->assigned[macroid]) {
-        if (out->file == NULL)
-          return(true);
-        fprintf(out->file,"%s(",out->symset->get_mp_macro(macroid));
+
+      {
+        thdb1ds * pst = NULL;
+        std::string commentstr("0");
+        if (this->station_name.id != 0) {
+          pst = &(this->db->db1d.station_vec[this->station_name.id - 1]);
+          station_attr_id = this->station_name.id;
+          station_attr = &(thdb.db1d.m_station_attr);
+          station_attr->export_mp_object_begin(out->file, station_attr_id);
+        }
+        if (pst != NULL) {
+#define thexpmatselected_stationflag(flag,mid) if (((pst->flags & flag) == flag) && out->symset->assigned[mid]) expsym = true;
+          thexpmatselected_stationflag(TT_STATIONFLAG_ENTRANCE, SYMP_FLAG_ENTRANCE)
+          thexpmatselected_stationflag(TT_STATIONFLAG_SINK, SYMP_FLAG_SINK)
+          thexpmatselected_stationflag(TT_STATIONFLAG_SPRING, SYMP_FLAG_SPRING)
+          thexpmatselected_stationflag(TT_STATIONFLAG_DOLINE, SYMP_FLAG_DOLINE)
+          thexpmatselected_stationflag(TT_STATIONFLAG_DIG, SYMP_FLAG_DIG)
+          thexpmatselected_stationflag(TT_STATIONFLAG_ARCH, SYMP_FLAG_ARCH)
+          thexpmatselected_stationflag(TT_STATIONFLAG_OVERHANG, SYMP_FLAG_OVERHANG)
+          thexpmatselected_stationflag(TT_STATIONFLAG_CONT, SYMP_FLAG_CONTINUATION)
+          thexpmatselected_stationflag(TT_STATIONFLAG_AIRDRAUGHT, SYMP_FLAG_AIRDRAUGHT)
+          thexpmatselected_stationflag(TT_STATIONFLAG_AIRDRAUGHT_SUMMER, SYMP_FLAG_AIRDRAUGHT)
+          thexpmatselected_stationflag(TT_STATIONFLAG_AIRDRAUGHT_WINTER, SYMP_FLAG_AIRDRAUGHT)
+        }
+
+        if (out->file == NULL) return(expsym);
+
+        if ((pst != NULL) && (pst->comment != NULL) && (strlen(pst->comment) > 0)) {
+          commentstr = "btex \\thcomment ";
+          commentstr += ths2tex(pst->comment, out->layout->lang);
+          commentstr += " etex";
+        }
+        out->symset->export_mp_symbol_options(out->file, omacroid);
+        fprintf(out->file,"p_station(");
         this->point->export_mp(out);
+        fprintf(out->file,",%d,%s,\"\"",
+          out->symset->assigned[macroid] ? cmark : 0,
+          commentstr.c_str()
+          );
+#define thexpmat_stationflag(flag,mid,str) if (((pst->flags & flag) == flag) && out->symset->assigned[mid]) fprintf(out->file,",\"%s\"", str);
+        if (pst != NULL) {
+          thexpmat_stationflag(TT_STATIONFLAG_ENTRANCE, SYMP_FLAG_ENTRANCE, "entrance")
+          thexpmat_stationflag(TT_STATIONFLAG_SINK, SYMP_FLAG_SINK, "sink")
+          thexpmat_stationflag(TT_STATIONFLAG_SPRING, SYMP_FLAG_SPRING, "spring")
+          thexpmat_stationflag(TT_STATIONFLAG_DOLINE, SYMP_FLAG_DOLINE, "doline")
+          thexpmat_stationflag(TT_STATIONFLAG_DIG, SYMP_FLAG_DIG, "dig")
+          thexpmat_stationflag(TT_STATIONFLAG_ARCH, SYMP_FLAG_ARCH, "arch")
+          thexpmat_stationflag(TT_STATIONFLAG_OVERHANG, SYMP_FLAG_OVERHANG, "overhang")
+          thexpmat_stationflag(TT_STATIONFLAG_CONT, SYMP_FLAG_CONTINUATION, "continuation")
+          thexpmat_stationflag(TT_STATIONFLAG_AIRDRAUGHT, SYMP_FLAG_AIRDRAUGHT, "air-draught")
+          thexpmat_stationflag(TT_STATIONFLAG_AIRDRAUGHT_SUMMER, SYMP_FLAG_AIRDRAUGHT, "air-draught:summer")
+          thexpmat_stationflag(TT_STATIONFLAG_AIRDRAUGHT_WINTER, SYMP_FLAG_AIRDRAUGHT, "air-draught:winter")
+        }
         fprintf(out->file,");\n");
-        if (out->layout->is_debug_stations()) {
-          fprintf(out->file,"p_debug(0,1,");
-          this->point->export_mp(out,0);
-          fprintf(out->file,");\n");
-        }        
       }
+
+      if (out->symset->assigned[macroid] && out->layout->is_debug_stations()) {
+        fprintf(out->file,"p_debug(0,1,");
+        this->point->export_mp(out,0);
+        fprintf(out->file,");\n");
+      }        
       postprocess = false;
       break;
 
@@ -511,8 +570,24 @@ bool thpoint::export_mp(class thexpmapmpxs * out)
         if (out->file == NULL)
           return(true);
         out->symset->get_mp_macro(SYMP_ALTITUDE);    
-        fprintf(out->file,"p_label%s(btex \\thaltitude %s etex,",
-					thpoint_export_mp_align2mp(thdb2d_rotate_align(this->align, xrr)),
+        out->symset->export_mp_symbol_options(out->file, SYMP_ALTITUDE);
+        fprintf(out->file,"p_label%s(btex \\thaltitude ",
+					thpoint_export_mp_align2mp(thdb2d_rotate_align(this->align, xrr)));
+        switch (this->scale) {
+          case TT_2DOBJ_SCALE_XL:
+            fprintf(out->file,"\\thhugesize ");
+            break;
+          case TT_2DOBJ_SCALE_L:
+            fprintf(out->file,"\\thlargesize ");
+            break;
+          case TT_2DOBJ_SCALE_S:
+            fprintf(out->file,"\\thsmallsize ");
+            break;
+          case TT_2DOBJ_SCALE_XS:
+            fprintf(out->file,"\\thtinysize ");
+            break;
+        }
+        fprintf(out->file,"%s etex,",
 					utf2tex(out->layout->units.format_length(this->xsize - out->layout->goz)));
         postprocess_label = 1;
       }
@@ -546,14 +621,28 @@ bool thpoint::export_mp(class thexpmapmpxs * out)
           return(true);
 
         out->symset->get_mp_macro(macroid);
-        fprintf(out->file,"p_label%s(btex ",thpoint_export_mp_align2mp(thdb2d_rotate_align(this->align, xrr)));
+        out->symset->export_mp_symbol_options(out->file, macroid);
+        fprintf(out->file,"p_label%s(btex \\thheight ",thpoint_export_mp_align2mp(thdb2d_rotate_align(this->align, xrr)));
+
+        switch (this->scale) {
+          case TT_2DOBJ_SCALE_XL:
+            fprintf(out->file,"\\thhugesize ");
+            break;
+          case TT_2DOBJ_SCALE_L:
+            fprintf(out->file,"\\thlargesize ");
+            break;
+          case TT_2DOBJ_SCALE_S:
+            fprintf(out->file,"\\thsmallsize ");
+            break;
+          case TT_2DOBJ_SCALE_XS:
+            fprintf(out->file,"\\thtinysize ");
+            break;
+        }
 
         if ((this->tags & TT_POINT_TAG_HEIGHT_P) != 0)
           fprintf(out->file,"\\thheightpos ");
         else if ((this->tags & TT_POINT_TAG_HEIGHT_N) != 0)
           fprintf(out->file,"\\thheightneg ");
-        else
-          fprintf(out->file,"\\thheight ");  
 
         if (!thisnan(this->xsize)) {
           //if (double(int(this->xsize)) != this->xsize)
@@ -584,8 +673,24 @@ bool thpoint::export_mp(class thexpmapmpxs * out)
         if (out->file == NULL)
           return(true);
         out->symset->get_mp_macro(SYMP_DATE);    
-        fprintf(out->file,"p_label%s(btex \\thdate %s etex,",
-            thpoint_export_mp_align2mp(thdb2d_rotate_align(this->align, xrr)),
+        out->symset->export_mp_symbol_options(out->file, SYMP_DATE);
+        fprintf(out->file,"p_label%s(btex \\thdate ",
+            thpoint_export_mp_align2mp(thdb2d_rotate_align(this->align, xrr)));
+        switch (this->scale) {
+          case TT_2DOBJ_SCALE_XL:
+            fprintf(out->file,"\\thhugesize ");
+            break;
+          case TT_2DOBJ_SCALE_L:
+            fprintf(out->file,"\\thlargesize ");
+            break;
+          case TT_2DOBJ_SCALE_S:
+            fprintf(out->file,"\\thsmallsize ");
+            break;
+          case TT_2DOBJ_SCALE_XS:
+            fprintf(out->file,"\\thtinysize ");
+            break;
+        }
+        fprintf(out->file,"%s etex,",
             utf2tex(((thdate *)this->text)->get_str(TT_DATE_FMT_UTF8_ISO)));
         postprocess_label = 0;
       }
@@ -622,7 +727,7 @@ bool thpoint::export_mp(class thexpmapmpxs * out)
           return(true);
         
         out->symset->get_mp_macro(macroid);
-            
+        out->symset->export_mp_symbol_options(out->file, macroid);
         fprintf(out->file,"p_label%s(btex ",thpoint_export_mp_align2mp(thdb2d_rotate_align(this->align, xrr)));
         switch (this->tags & (TT_POINT_TAG_HEIGHT_P |
         TT_POINT_TAG_HEIGHT_N | TT_POINT_TAG_HEIGHT_U)) {
@@ -645,19 +750,41 @@ bool thpoint::export_mp(class thexpmapmpxs * out)
         }
 
         if (!thisnan(this->xsize)) {
-          //if (double(int(this->xsize)) != this->xsize)
-          //  sprintf(buff,"%.1f",this->xsize);
-          //else
-          //  sprintf(buff,"%.0f",this->xsize);
-          fprintf(out->file,"{%s}", utf2tex(out->layout->units.format_human_length(this->xsize)));
+          fprintf(out->file,"{");
+          switch (this->scale) {
+            case TT_2DOBJ_SCALE_XL:
+              fprintf(out->file,"\\thhugesize ");
+              break;
+            case TT_2DOBJ_SCALE_L:
+              fprintf(out->file,"\\thlargesize ");
+              break;
+            case TT_2DOBJ_SCALE_S:
+              fprintf(out->file,"\\thsmallsize ");
+              break;
+            case TT_2DOBJ_SCALE_XS:
+              fprintf(out->file,"\\thtinysize ");
+              break;
+          }
+          fprintf(out->file,"%s}", utf2tex(out->layout->units.format_human_length(this->xsize)));
         }
         
         if (!thisnan(this->ysize)) {
-          //if (double(int(this->ysize)) != this->ysize)
-          //  sprintf(buff,"%.1f",this->ysize);
-          //else
-          //  sprintf(buff,"%.0f",this->ysize);
-          fprintf(out->file,"{%s}", utf2tex(out->layout->units.format_human_length(this->ysize)));
+          fprintf(out->file,"{");
+          switch (this->scale) {
+            case TT_2DOBJ_SCALE_XL:
+              fprintf(out->file,"\\thhugesize ");
+              break;
+            case TT_2DOBJ_SCALE_L:
+              fprintf(out->file,"\\thlargesize ");
+              break;
+            case TT_2DOBJ_SCALE_S:
+              fprintf(out->file,"\\thsmallsize ");
+              break;
+            case TT_2DOBJ_SCALE_XS:
+              fprintf(out->file,"\\thtinysize ");
+              break;
+          }
+          fprintf(out->file,"%s}", utf2tex(out->layout->units.format_human_length(this->ysize)));
         }        
         
         fprintf(out->file," etex,");
@@ -696,17 +823,14 @@ bool thpoint::export_mp(class thexpmapmpxs * out)
       macroid = SYMP_CONTINUATION;
       {
         std::string tmp;
-        if ((this->code != NULL) && (strlen(this->code) > 0)) {
-          tmp += this->code;
-        }
         if ((this->text != NULL) && (strlen(this->text) > 0)) {
           if (tmp.length() > 0)
             tmp += " -- ";
           tmp += this->text;
         }
         if (tmp.length() > 0) {
-          attr_text = "btex \\thcontinuation ";
-          attr_text += utf2tex(tmp);
+          attr_text = "btex \\thcomment ";
+          attr_text += ths2tex(tmp, out->layout->lang);
           attr_text += "etex";
         }
       }
@@ -728,12 +852,14 @@ bool thpoint::export_mp(class thexpmapmpxs * out)
     thpoint_type_export_mp(TT_POINT_TYPE_NO_EQUIPMENT,SYMP_NOEQUIPMENT)
     thpoint_type_export_mp(TT_POINT_TYPE_ANCHOR,SYMP_ANCHOR)
     thpoint_type_export_mp(TT_POINT_TYPE_CAMP,SYMP_CAMP)
+    thpoint_type_export_mp(TT_POINT_TYPE_DIG,SYMP_DIG)
 
 // ukoncenia chodby
     thpoint_type_export_mp(TT_POINT_TYPE_NARROW_END,SYMP_NARROWEND)
     thpoint_type_export_mp(TT_POINT_TYPE_LOW_END,SYMP_LOWEND)
     thpoint_type_export_mp(TT_POINT_TYPE_FLOWSTONE_CHOKE,SYMP_FLOWSTONECHOKE)
     thpoint_type_export_mp(TT_POINT_TYPE_BREAKDOWN_CHOKE,SYMP_BREAKDOWNCHOKE)
+    thpoint_type_export_mp(TT_POINT_TYPE_CLAY_CHOKE,SYMP_CLAYCHOKE)
 
 // vypln
     thpoint_type_export_mp(TT_POINT_TYPE_FLOWSTONE,SYMP_FLOWSTONE)
@@ -741,6 +867,9 @@ bool thpoint::export_mp(class thexpmapmpxs * out)
     thpoint_type_export_mp(TT_POINT_TYPE_STALACTITE,SYMP_STALACTITE)
     thpoint_type_export_mp(TT_POINT_TYPE_STALAGMITE,SYMP_STALAGMITE)
     thpoint_type_export_mp(TT_POINT_TYPE_PILLAR,SYMP_PILLAR)
+    thpoint_type_export_mp(TT_POINT_TYPE_ICE_STALACTITE,SYMP_ICESTALACTITE)
+    thpoint_type_export_mp(TT_POINT_TYPE_ICE_STALAGMITE,SYMP_ICESTALAGMITE)
+    thpoint_type_export_mp(TT_POINT_TYPE_ICE_PILLAR,SYMP_ICEPILLAR)
     thpoint_type_export_mp(TT_POINT_TYPE_CURTAIN,SYMP_CURTAIN)
     thpoint_type_export_mp(TT_POINT_TYPE_HELICTITE,SYMP_HELICTITE)
     thpoint_type_export_mp(TT_POINT_TYPE_SODA_STRAW,SYMP_SODASTRAW)
@@ -759,6 +888,8 @@ bool thpoint::export_mp(class thexpmapmpxs * out)
     thpoint_type_export_mp(TT_POINT_TYPE_SCALLOP,SYMP_SCALLOP)
     thpoint_type_export_mp(TT_POINT_TYPE_FLUTE,SYMP_FLUTE)
     thpoint_type_export_mp(TT_POINT_TYPE_RAFT_CONE,SYMP_RAFT)
+    thpoint_type_export_mp(TT_POINT_TYPE_CLAY_TREE,SYMP_CLAYTREE)
+
   
 // plosne vyplne
     thpoint_type_export_mp(TT_POINT_TYPE_BEDROCK,SYMP_BEDROCK)
@@ -799,9 +930,16 @@ bool thpoint::export_mp(class thexpmapmpxs * out)
       if (out->file == NULL)
         return(true);
       if (attr_text.length() > 0) {
-        fprintf(out->file, "ATTR__text_x := true;\nATTR__text := %s;\n", attr_text.c_str());
+        fprintf(out->file, "picture ATTR__text;\nATTR__text := %s;\n", attr_text.c_str());
+      }
+      if (this->type == TT_POINT_TYPE_U) {
+         out->symset->export_mp_symbol_options(out->file, -1);
+         fprintf(out->file,"p_u_%s(",this->m_subtype_str);
+         this->db->db2d.use_u_symbol(this->get_class_id(), this->m_subtype_str);
+      } else {
+         out->symset->export_mp_symbol_options(out->file, macroid);
+         fprintf(out->file,"%s(",out->symset->get_mp_macro(macroid));
       }
-      fprintf(out->file,"%s(",out->symset->get_mp_macro(macroid));
     }
     else
       postprocess = false;
@@ -824,7 +962,7 @@ bool thpoint::export_mp(class thexpmapmpxs * out)
         scl = 0.5;
         break;
     }
-    char * al = "(0,0)";
+    const char * al = "(0,0)";
     switch (thdb2d_rotate_align(this->align, xrr)) {
       case TT_POINT_ALIGN_B:
         al = "(0,-1)";
@@ -853,11 +991,7 @@ bool thpoint::export_mp(class thexpmapmpxs * out)
     }
     fprintf(out->file,",%.1f,%.2f,%s",
         (thisnan(this->orient) ? 0 : 360 - this->orient - out->rr),scl,al);
-    if (this->type == TT_POINT_TYPE_U) {
-      fprintf(out->file, ",\"%s\");\n", (this->m_subtype_str == NULL) ? "" : this->m_subtype_str);
-    } else {
-      fprintf(out->file, ");\n");
-    }
+    fprintf(out->file, ");\n");
 
     if (out->layout->is_debug_stations() || out->layout->is_debug_joins()) {
       fprintf(out->file,"p_debug(-1,0,");
@@ -875,10 +1009,14 @@ bool thpoint::export_mp(class thexpmapmpxs * out)
       fprintf(out->file,");\n");
     }
     if (attr_text.length() > 0) {
-      fprintf(out->file, "ATTR__text_x := false;\n");
+      fprintf(out->file, "save ATTR__text;\n");
     }
   }
   
+  if (station_attr != NULL) {
+    station_attr->export_mp_object_end(out->file, station_attr_id);
+  }
+  th2ddataobject::export_mp_end(out);
   return(false);
 }
 
@@ -912,16 +1050,18 @@ void thpoint::parse_text(char * ss) {
 }
 
 
-void thpoint::parse_code(char * ss) {
+void thpoint::parse_explored(char * ss) {
   switch (this->type) {
     case TT_POINT_TYPE_CONTINUATION:
       break;
     default:
-      ththrow(("-code not valid with type %s", thmatch_string(this->type,thtt_point_types)))
+      ththrow(("-explored not valid with type %s", thmatch_string(this->type,thtt_point_types)))
       break;
   }
-  if (strlen(ss) > 0)
-    this->code = this->db->strstore(ss);
+  int sv;
+  thparse_length(sv, this->xsize, ss);
+  if (sv == TT_SV_UNKNOWN)
+      ththrow(("ivalid explored length -- %s", ss));
 }
 
 
