@@ -129,21 +129,21 @@ int thscrap::get_cmd_nargs()
 }
 
 
-char * thscrap::get_cmd_end()
+const char * thscrap::get_cmd_end()
 {
   // insert endcommand if multiline command
   return NULL;
 }
 
 
-char * thscrap::get_cmd_name()
+const char * thscrap::get_cmd_name()
 {
   // insert command name here
   return "scrap";
 }
 
 
-thcmd_option_desc thscrap::get_cmd_option_desc(char * opts)
+thcmd_option_desc thscrap::get_cmd_option_desc(const char * opts)
 {
   int id = thmatch_token(opts, thtt_scrap_opt);
   if (id == TT_SCRAP_UNKNOWN)
@@ -158,9 +158,9 @@ thcmd_option_desc thscrap::get_cmd_option_desc(char * opts)
 
 
 static const thstok thtt_scrap_flips[] = {
-  {"none", TT_SCRAP_FLIP_NONE},
   {"horiz", TT_SCRAP_FLIP_HORIZ},
   {"horizontal", TT_SCRAP_FLIP_HORIZ},
+  {"none", TT_SCRAP_FLIP_NONE},
   {"vert", TT_SCRAP_FLIP_VERT},
   {"vertical", TT_SCRAP_FLIP_VERT},
   {NULL, TT_SCRAP_FLIP_UNKNOWN},
