@@ -38,6 +38,8 @@
 #include "thscrapen.h"
 #include "thmapstat.h"
 #include "thdb3d.h"
+#include "thsketch.h"
+#include "thtrans.h"
 
 /**
  * scrap command options tokens.
@@ -49,6 +51,15 @@ enum {
   TT_SCRAP_SCALE = 2002,
   TT_SCRAP_STATIONS = 2003,
   TT_SCRAP_3D = 2004,
+  TT_SCRAP_FLIP = 2005,
+  TT_SCRAP_SKETCH = 2006,
+};
+
+enum {
+  TT_SCRAP_FLIP_UNKNOWN,
+  TT_SCRAP_FLIP_NONE,
+  TT_SCRAP_FLIP_HORIZ,
+  TT_SCRAP_FLIP_VERT,
 };
 
 
@@ -57,9 +68,11 @@ enum {
  */
  
 static const thstok thtt_scrap_opt[] = {
+  {"flip", TT_SCRAP_FLIP},
   {"proj", TT_SCRAP_PROJECTION},
   {"projection", TT_SCRAP_PROJECTION},
   {"scale", TT_SCRAP_SCALE},
+  {"sketch", TT_SCRAP_SKETCH},
   {"stations", TT_SCRAP_STATIONS},
   {"walls", TT_SCRAP_3D},
   {NULL, TT_SCRAP_UNKNOWN},
@@ -109,16 +122,19 @@ class thscrap : public thdataobject {
   thscrapen * ends_first;
   
   thmapstat_datamap adata;
+  thmorph2trans morph;
   
-  int d3;
+  int d3, flip;
   thdb3ddata d3_outline;
   
   void parse_scale(char * ss);
   void parse_stations(char * ss);
   void process_3d();
-  thscraplp * polygon_insert();
+  thscraplp * polygon_insert(thscraplp * before = NULL);
   
   thdb1ds * get_nearest_station(thdb2dpt * pt);
+
+  thsketch_list sketch_list;
   
   double z;
     
@@ -149,7 +165,7 @@ class thscrap : public thdataobject {
    * Return class name.
    */
    
-  virtual char * get_class_name() {return "thscrap";};
+  virtual const char * get_class_name() {return "thscrap";};
   
   
   /**
@@ -170,21 +186,21 @@ class thscrap : public thdataobject {
    * Return command name.
    */
    
-  virtual char * get_cmd_name();
+  virtual const char * get_cmd_name();
   
   
   /**
    * Return command end option.
    */
    
-  virtual char * get_cmd_end();
+  virtual const char * get_cmd_end();
   
   
   /**
    * Return option description.
    */
    
-  virtual thcmd_option_desc get_cmd_option_desc(char * opts);
+  virtual thcmd_option_desc get_cmd_option_desc(const char * opts);
   
   
   /**
@@ -198,6 +214,13 @@ class thscrap : public thdataobject {
   virtual void set(thcmd_option_desc cod, char ** args, int argenc, unsigned long indataline);
 
 
+  /**
+   * Parse sketch option.
+   */
+
+  void parse_sketch(char ** args, int argenc);
+
+
   /**
    * Delete this object.
    *
@@ -207,6 +230,13 @@ class thscrap : public thdataobject {
   virtual void self_delete();
 
 
+  /**
+   * Create a sketch linked to current scrap.
+   */
+
+  void insert_sketch(char ** args);
+
+
   /**
    * Print object properties.
    */
@@ -276,6 +306,9 @@ class thscrap : public thdataobject {
  
  void update_limits(double x, double y);
 
+ virtual void start_insert();
+
+
 };
 
 
