@@ -45,12 +45,15 @@ class thscraplp {
     
   thobjectname station_name;
   thdb1ds * station, * ustation;
+  class thdb1d_tree_arrow * arrow;
   
   thscraplp(); ///< Default constructor
   
   double stx, sty, stz, lnx1, lnx2, lny1, lny2, lnz1, lnz2;
   bool lnio;
   int type;
+
+  void export_mp(class thexpmapmpxs * out, class thscrap * scrap);
   
 };
 
