@@ -25,7 +25,7 @@
  * --------------------------------------------------------------------
  */
  
-// #include <iomanip>
+#include <iomanip>
 #include <iostream>
 #include <fstream>
 #include <list>
@@ -55,12 +55,13 @@ using namespace std;
 
 static const char base64_tab[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
 
-void base64_encode(char * fname, ofstream & fout) {
+void base64_encode(const char * fname, ofstream & fout) {
   int llength = 0;
   char in_buffer[3];
   unsigned char out_buffer[4];
 
   ifstream fin(fname, ios::binary);
+  if (!fin) therror((((string)"Can't read file "+fname+"!\n").c_str()));
 
   do {
       for (int i = 0; i < 3; i++) in_buffer[i] = '\x0';
@@ -90,6 +91,7 @@ void base64_encode(char * fname, ofstream & fout) {
 
 void find_dimensions(double & MINX,double & MINY,double & MAXX,double & MAXY) {
   double llx = 0, lly = 0, urx = 0, ury = 0;
+  double a,b,w,h;
   MINX=DBL_MAX, MINY=DBL_MAX, MAXX=-DBL_MAX, MAXY=-DBL_MAX;
   for (list<scraprecord>::iterator I = SCRAPLIST.begin(); 
                                   I != SCRAPLIST.end(); I++) {
@@ -131,6 +133,22 @@ void find_dimensions(double & MINX,double & MINY,double & MAXX,double & MAXY) {
       if (I->X3 > urx) urx = I->X3;
       if (I->X4 > ury) ury = I->X4;
     }
+    for (list<surfpictrecord>::iterator I_sk = I->SKETCHLIST.begin();
+                                          I_sk != I->SKETCHLIST.end(); I_sk++) {
+      for (int i = 0; i<=1; i++) {
+        for (int j = 0; j<=1; j++) {
+          w = i * I_sk->width;
+          h = j * I_sk->height;
+          a = I_sk->xx*w + I_sk->xy*h + I_sk->dx;
+          b = I_sk->yx*w + I_sk->yy*h + I_sk->dy;
+          if (a < llx) llx = a;
+          if (b < lly) lly = b;
+          if (a > urx) urx = a;
+          if (b > ury) ury = b;
+        }
+      }
+    };
+
 
     if (llx == DBL_MAX || lly == DBL_MAX || urx == -DBL_MAX || ury == -DBL_MAX) 
       therror(("This can't happen -- no data for a scrap!"));
@@ -158,7 +176,7 @@ void find_dimensions(double & MINX,double & MINY,double & MAXX,double & MAXY) {
 }
 
 
-void print_preview(int up,ofstream& F) {
+void print_preview(int up,ofstream& F, string unique_prefix) {
   set<int> used_layers;
   set<string> used_scraps;
   
@@ -184,12 +202,12 @@ void print_preview(int up,ofstream& F) {
         if (used_scraps.count(K->name) > 0) {
           if (up) {
             if (K->B != "" && K->sect == 0) {
-              F << "<use x=\"" << K->B1 << "\" y=\"" << K->B2 << "\" xlink:href=\"#B_" << K->name << "\" />" << endl;
+              F << "<use x=\"" << K->B1 << "\" y=\"" << K->B2 << "\" xlink:href=\"#B_" << K->name << "_" << unique_prefix << "\" />" << endl;
             }
           }
           else {
             if (K->I != "" && K->sect == 0) {
-              F << "<use x=\"" << K->I1 << "\" y=\"" << K->I2 << "\" xlink:href=\"#I_" << K->name << "\" />" << endl;
+              F << "<use x=\"" << K->I1 << "\" y=\"" << K->I2 << "\" xlink:href=\"#I_" << K->name << "_" << unique_prefix << "\" />" << endl;
             }
           }
         }
@@ -200,7 +218,7 @@ void print_preview(int up,ofstream& F) {
 }
 
 
-void print_grid(ofstream& PAGEDEF, double LLX,double LLY,double URX,double URY) {
+void print_grid(ofstream& PAGEDEF, double LLX,double LLY,double URX,double URY, string unique_prefix) {
   if (LAYOUT.grid == 0) return;
   PAGEDEF << "<g>" << endl;
   
@@ -215,7 +233,7 @@ void print_grid(ofstream& PAGEDEF, double LLX,double LLY,double URX,double URY)
   ul.y = ur.y;
   origin.x = LAYOUT.hgridorigin;
   origin.y = LAYOUT.vgridorigin;
-  
+//cout << endl << origin.x << " " << origin.y << endl;
   llrot = rotatedaround(ll,origin,-LAYOUT.gridrot);
   urrot = rotatedaround(ur,origin,-LAYOUT.gridrot);
   lrrot = rotatedaround(lr,origin,-LAYOUT.gridrot);
@@ -228,7 +246,10 @@ void print_grid(ofstream& PAGEDEF, double LLX,double LLY,double URX,double URY)
   
   double grid_init_x = LAYOUT.hgridsize * floor ((llnew.x-origin.x)/LAYOUT.hgridsize) + origin.x;
   double grid_init_y = LAYOUT.vgridsize * floor ((llnew.y-origin.y)/LAYOUT.vgridsize) + origin.y;
-  
+
+double G_real_init_x = LAYOUT.XO + LAYOUT.XS * floor ((llnew.x-origin.x)/LAYOUT.hgridsize);
+double G_real_init_y = LAYOUT.YO + LAYOUT.YS * floor ((llnew.y-origin.y)/LAYOUT.vgridsize);
+
   double cosr = cos(-LAYOUT.gridrot * 3.14159265 / 180);
   double sinr = sin(-LAYOUT.gridrot * 3.14159265 / 180);
 
@@ -236,11 +257,13 @@ void print_grid(ofstream& PAGEDEF, double LLX,double LLY,double URX,double URY)
 
   if (LAYOUT.proj == 0) {
     paired out,tmp;
-    for (double i = grid_init_x; i < urnew.x + LAYOUT.hgridsize - 0.05; i += LAYOUT.hgridsize) {
-      for (double j = grid_init_y; j < urnew.y + LAYOUT.vgridsize - 0.05; j += LAYOUT.vgridsize) {
-        row = (i == grid_init_x ? 0 : (i >= urnew.x ? 2 : 1));
-        col = (j == grid_init_y ? 0 : (j >= urnew.y ? 2 : 1));
-	elem = row + 3*col;
+    int ii,jj;
+    double i, j;
+    for (i = grid_init_x, ii=0; i < urnew.x + LAYOUT.hgridsize - 0.05; i += LAYOUT.hgridsize, ii++) {
+      for (j = grid_init_y, jj=0; j < urnew.y + LAYOUT.vgridsize - 0.05; j += LAYOUT.vgridsize, jj++) {
+        col = (i == grid_init_x ? 0 : (i >= urnew.x ? 2 : 1));
+        row = (j == grid_init_y ? 0 : (j >= urnew.y ? 2 : 1));
+	elem = col + 3*row;
         tmp.x = i+LAYOUT.gridcell[elem].x;
         tmp.y = j+LAYOUT.gridcell[elem].y;
         out = rotatedaround(tmp,origin,LAYOUT.gridrot);
@@ -249,21 +272,56 @@ void print_grid(ofstream& PAGEDEF, double LLX,double LLY,double URX,double URY)
         PAGEDEF << "<g transform=\"matrix(" << 
                    cosr << " " << sinr << " " << -sinr << " " << cosr << " " << 
                    out.x << " " << out.y << ")\">";
-	PAGEDEF << "<use xlink:href=\"#grid_" << u2str(elem+1) << "\" />";
+	PAGEDEF << "<use xlink:href=\"#grid_" << u2str(elem+1) << "_" << unique_prefix << "\" />";
         PAGEDEF << "</g>" << endl;
+
+        if (LAYOUT.grid_coord_freq==2 || (LAYOUT.grid_coord_freq==1 && elem!=4)) {
+          tmp.x = i+ (col==2 ? -2 : 2);
+          tmp.y = j+ (row==2 ? -9 : 2);
+          out = rotatedaround(tmp,origin,LAYOUT.gridrot);
+          PAGEDEF << "<text fill=\"black\" stroke=\"none\" font-size=\"8\" " <<
+	          (col == 2 ? "text-anchor=\"end\" " : "") <<
+                  "transform=\"matrix(" << 
+                   cosr << " " << sinr << " " << sinr << " " << -cosr << " " << 
+                   out.x << " " << out.y << ")\">(" << 
+	      setprecision(0) << G_real_init_x+ii*LAYOUT.XS << "," << 
+              G_real_init_y+jj*LAYOUT.YS << setprecision(3) << ")</text>" << endl;
+        }
+  
+/*  PAGEDEF << "<text font-family=\"arial\" font-size=\"10\" " <<
+       "transform=\"matrix(" << 
+                   1 << " " << 0 << " " << 0 << " " << -1 << " " << 
+                   out.x << " " << out.y << ")\">";
+  PAGEDEF << i/72*2.54/100*200 << " " << j/72*2.54/100*200;
+  PAGEDEF << "</text>" << endl;*/
+	
       }
     }
   }
   else {
     grid_init_x = LLX;
-    for (double j = grid_init_y; j < urnew.y + LAYOUT.vgridsize - 0.05; j += LAYOUT.vgridsize) {
-      for (double i = grid_init_x; i < urnew.x + LAYOUT.hgridsize - 0.05; i += LAYOUT.hgridsize) {
-        row = (i == grid_init_x ? 0 : (i >= urnew.x ? 2 : 1));
-        col = (j == grid_init_y ? 0 : (j >= urnew.y ? 2 : 1));
-	elem = row + 3*col;
+    int jj;
+    double i, j;
+    for (j = grid_init_y, jj=0; j < urnew.y + LAYOUT.vgridsize - 0.05; j += LAYOUT.vgridsize, jj++) {
+      for (i = grid_init_x; i < urnew.x + LAYOUT.hgridsize - 0.05; i += LAYOUT.hgridsize) {
+        col = (i == grid_init_x ? 0 : (i >= urnew.x ? 2 : 1));
+        row = (j == grid_init_y ? 0 : (j >= urnew.y ? 2 : 1));
+	elem = col + 3*row;
 	PAGEDEF << "<use x=\"" << i/*-LLX*/+LAYOUT.gridcell[elem].x << "\" y=\"" << 
-	                      j/*-LLY*/+LAYOUT.gridcell[elem].y << "\" xlink:href=\"#grid_" << 
+	                      j/*-LLY*/+LAYOUT.gridcell[elem].y << "\" xlink:href=\"#grid_" << "_" << unique_prefix << 
 			      u2str(elem+1) << "\" />" << endl;
+        if (col == 0 && LAYOUT.grid_coord_freq > 0) {
+          PAGEDEF << "<text fill=\"black\" stroke=\"none\" font-size=\"8\" " << 
+	      "transform=\"matrix(1,0,0,-1," << i << "," << 
+	      j+1 /* podvihnutie o 1 bp */ << ")\">" << 
+	      setprecision(0) << G_real_init_y+jj*LAYOUT.YS << "</text>" << endl;
+	}
+        if (col == 2 && LAYOUT.grid_coord_freq == 2) {
+          PAGEDEF << "<text fill=\"black\" stroke=\"none\" font-size=\"8\" " << 
+	      "transform=\"matrix(1,0,0,-1," << i << "," << 
+	      j+1 /* podvihnutie o 1 bp */ << ")\" text-anchor=\"end\">" << 
+	      setprecision(0) << G_real_init_y+jj*LAYOUT.YS << "</text>" << endl;
+        }
       }
     }
   } 
@@ -274,6 +332,7 @@ void print_grid(ofstream& PAGEDEF, double LLX,double LLY,double URX,double URY)
 void print_surface_bitmaps (ofstream &F) {
   if (LAYOUT.transparency) 
     F << "<g opacity=\"" << LAYOUT.surface_opacity << "\">" << endl;
+  F.precision(8);
   for (list<surfpictrecord>::iterator I = SURFPICTLIST.begin();
                                       I != SURFPICTLIST.end(); I++) {
     F << "<g transform=\"matrix(";
@@ -286,27 +345,29 @@ void print_surface_bitmaps (ofstream &F) {
     F << "\" />";
     F << "</g>" << endl;
   };
+  F.precision(3);
   if (LAYOUT.transparency) 
     F << "</g>" << endl;
 }
 
 
 
-#define ginit(ID) F << "<g id=\"" << ID << "\">" << endl;
+#define ginit(ID) F << "<g id=\"" << ID << "_" << unique_prefix << "\">" << endl;
 #define gend  F << "</g>" << endl;
 
 
 
-void thsvg(char * fname, int fmt, legenddata ldata = legenddata::legenddata()) {
+void thsvg(const char * fname, int fmt, legenddata ldata) {
   if (fmt == 0)
     thprintf("making svg map ... ");
   else
     thprintf("making svg (xhtml) map ... ");
   string bgcol;
+  string unique_prefix = fname;
 
   ofstream F(fname);
-  //F.setf(ios::fixed, ios::floatfield);
-  //F.precision(2);
+  F.setf(ios::fixed, ios::floatfield);  // dolezite pre velke suradnice
+  F.precision(3);
   
   double llx=0, lly=0, urx=0, ury=0;
   find_dimensions(llx, lly, urx, ury);
@@ -315,14 +376,15 @@ void thsvg(char * fname, int fmt, legenddata ldata = legenddata::legenddata()) {
     lly = LAYOUT.vgridsize * floor ((lly-LAYOUT.vgridorigin)/LAYOUT.vgridsize) + LAYOUT.vgridorigin;
     ury = LAYOUT.vgridsize * ceil  ((ury-LAYOUT.vgridorigin)/LAYOUT.vgridsize) + LAYOUT.vgridorigin;
   }
-  double llxo(0.0), llyo(0.0), urxo(0.0), uryo(0.0);  // rozmery mapy bez overlapu
   if (LAYOUT.map_grid) {
-    llxo = llx = LAYOUT.hsize * floor (llx/LAYOUT.hsize);
-    llyo = lly = LAYOUT.vsize * floor (lly/LAYOUT.vsize);
-    urxo = urx = LAYOUT.hsize * ceil (urx/LAYOUT.hsize);
-    uryo = ury = LAYOUT.vsize * ceil (ury/LAYOUT.vsize);
+    llx = LAYOUT.hsize * floor (llx/LAYOUT.hsize);
+    lly = LAYOUT.vsize * floor (lly/LAYOUT.vsize);
+    urx = LAYOUT.hsize * ceil (urx/LAYOUT.hsize);
+    ury = LAYOUT.vsize * ceil (ury/LAYOUT.vsize);
   }
 
+  double llxo(llx), llyo(lly), urxo(urx), uryo(ury);  // rozmery mapy bez overlapu
+
   llx -= LAYOUT.overlap;
   urx += LAYOUT.overlap;
   lly -= LAYOUT.overlap;
@@ -356,12 +418,12 @@ void thsvg(char * fname, int fmt, legenddata ldata = legenddata::legenddata()) {
     if (!ldata.copyrights.empty()) F << "<p>" << ldata.copyrights << "</p>" << endl;
     if (LAYOUT.scalebar != "") {
       F << "<p>" << endl;
-      ScBar.print_svg(F);
+      ScBar.print_svg(F,unique_prefix);
       F << "</p>" << endl;
     }
     if (LAYOUT.northarrow != "") {
       F << "<p>" << endl;
-      NArrow.print_svg(F);
+      NArrow.print_svg(F,unique_prefix);
       F << "</p>" << endl;
     }
   }
@@ -381,35 +443,37 @@ void thsvg(char * fname, int fmt, legenddata ldata = legenddata::legenddata()) {
   // if the pattern definition exceeds the BBox in MetaPost
   for (list<pattern>::iterator I = PATTERNLIST.begin();
                                I != PATTERNLIST.end(); I++) {
-    F << "<pattern id=\"patt_" << I->name << 
-         "\" patternUnits=\"userSpaceOnUse\"" << 
-         " width=\"" << I->xstep <<   
-         "\" height=\"" << I->ystep << 
-         "\" patternTransform=\"matrix(" << I->xx << " " << I->xy << " " 
-                                         << I->yx << " " << I->yy << " " 
-                                         << I->x <<  " " << I->y  << 
-         ")\">" << endl;
-    F << "<g transform=\"translate(" 
-                  << I->llx1-I->llx << " " << I->lly1-I->lly << ")\">" << endl;
-    I->data.MP.print_svg(F);
-    F << "</g>" << endl;
-    F << "</pattern>" << endl;
+    if (I->used) {
+      F << "<pattern id=\"patt_" << I->name << "_" << unique_prefix <<
+           "\" patternUnits=\"userSpaceOnUse\"" << 
+           " width=\"" << I->xstep <<   
+           "\" height=\"" << I->ystep << 
+           "\" patternTransform=\"matrix(" << I->xx << " " << I->xy << " " 
+                                           << I->yx << " " << I->yy << " " 
+                                           << I->x <<  " " << I->y  << 
+           ")\">" << endl;
+      F << "<g transform=\"translate(" 
+                    << I->llx1-I->llx << " " << I->lly1-I->lly << ")\">" << endl;
+      I->data.MP.print_svg(F,unique_prefix);
+      F << "</g>" << endl;
+      F << "</pattern>" << endl;
+    }
   }
   // scraps:
   for(list<scraprecord>::iterator I = SCRAPLIST.begin(); 
                                   I != SCRAPLIST.end(); I++) {
-    ginit("F_" + I->name); I->Fc.MP.print_svg(F); gend;
-    ginit("G_" + I->name); I->Gc.MP.print_svg(F); gend;
-    ginit("B_" + I->name); I->Bc.MP.print_svg(F); gend;
-    ginit("I_" + I->name); I->Ic.MP.print_svg(F); gend;
-    ginit("E_" + I->name); I->Ec.MP.print_svg(F); gend;
-    ginit("X_" + I->name); I->Xc.MP.print_svg(F); gend;
+    ginit("F_" + I->name); I->Fc.MP.print_svg(F,unique_prefix); gend;
+    ginit("G_" + I->name); I->Gc.MP.print_svg(F,unique_prefix); gend;
+    ginit("B_" + I->name); I->Bc.MP.print_svg(F,unique_prefix); gend;
+    ginit("I_" + I->name); I->Ic.MP.print_svg(F,unique_prefix); gend;
+    ginit("E_" + I->name); I->Ec.MP.print_svg(F,unique_prefix); gend;
+    ginit("X_" + I->name); I->Xc.MP.print_svg(F,unique_prefix); gend;
   }
   // grid:
   int i=0;
   for (list<converted_data>::iterator I = GRIDLIST.begin();
                                       I != GRIDLIST.end(); I++) {
-    ginit("grid_" + u2str(++i)); I->MP.print_svg(F); gend;
+    ginit("grid_" + u2str(++i)); I->MP.print_svg(F,unique_prefix); gend;
   }
   // clip to initial viewBox
   // (browsers mostly ignore clip="auto" overflow="hidden" root svg attributes)
@@ -439,10 +503,10 @@ void thsvg(char * fname, int fmt, legenddata ldata = legenddata::legenddata()) {
 
   // white scrap backgrounds (when transparency added):
   // grid:
-  if (LAYOUT.grid == 1) print_grid(F,llx,lly,urx,ury);
+  if (LAYOUT.grid == 1) print_grid(F,llxo,llyo,urxo,uryo,unique_prefix);
 
   // preview down:
-  if (!MAP_PREVIEW_DOWN.empty()) print_preview(0,F);
+  if (!MAP_PREVIEW_DOWN.empty()) print_preview(0,F,unique_prefix);
   
   // map export:
   for (map<int,layerrecord>::iterator J = LAYERHASH.begin();
@@ -475,31 +539,53 @@ void thsvg(char * fname, int fmt, legenddata ldata = legenddata::legenddata()) {
           }
           if (used_scraps.count(K->name) > 0 && K->I != "") {
             F << "<g fill=\"" << bgcol << "\">" << endl;
-            F << "<use x=\"" << K->I1 << "\" y=\"" << K->I2 << "\" xlink:href=\"#I_" << K->name << "\" />" << endl;
+            F << "<use x=\"" << K->I1 << "\" y=\"" << K->I2 << "\" xlink:href=\"#I_" << K->name << "_" << unique_prefix << "\" />" << endl;
             F << "</g>" << endl;
           }
         }
 
-//    F << "<use x=\"" << I->G1 << "\" y=\"" << -I->G2 << "\" xlink:href=\"#G_" << I->name << "\" />" << endl;
+//    F << "<use x=\"" << I->G1 << "\" y=\"" << -I->G2 << "\" xlink:href=\"#G_" << I->name << "_" << unique_prefix << "\" />" << endl;
+
+        // sketches
+        F.precision(8);
+        for (list<scraprecord>::iterator K = SCRAPLIST.begin(); 
+                                         K != SCRAPLIST.end(); K++) {
+          for (list<surfpictrecord>::iterator I_sk = K->SKETCHLIST.begin();
+                                              I_sk != K->SKETCHLIST.end(); I_sk++) {
+            F << "<g transform=\"matrix(";
+            F << I_sk->xx << " " << I_sk->yx << " " << -I_sk->xy << " " << -I_sk->yy << " " << 
+                 I_sk->dx << " " << I_sk->dy << ")\">";
+            F << "<image x=\"0\" y=\"" << -I_sk->height << "\" width=\"" << I_sk->width << 
+                 "\" height=\"" << I_sk->height << "\" xlink:href=\"data:image/" << 
+                 I_sk->type << ";base64," << endl;
+            base64_encode(I_sk->filename, F);
+            F << "\" />";
+            F << "</g>" << endl;
+          };
+        }
+        F.precision(3);
+        
+        // end of sketches
+
 
         for (list<scraprecord>::iterator K = SCRAPLIST.begin(); 
                                          K != SCRAPLIST.end(); K++) {
           if (used_scraps.count(K->name) > 0 && K->F != "") {
-            F << "<use x=\"" << K->F1 << "\" y=\"" << K->F2 << "\" xlink:href=\"#F_" << K->name << "\" />" << endl;
+            F << "<use x=\"" << K->F1 << "\" y=\"" << K->F2 << "\" xlink:href=\"#F_" << K->name << "_" << unique_prefix << "\" />" << endl;
           }
         }
 
         for (list<scraprecord>::iterator K = SCRAPLIST.begin(); 
                                          K != SCRAPLIST.end(); K++) {
           if (used_scraps.count(K->name) > 0 && K->E != "") {
-            F << "<use x=\"" << K->E1 << "\" y=\"" << K->E2 << "\" xlink:href=\"#E_" << K->name << "\" />" << endl;
+            F << "<use x=\"" << K->E1 << "\" y=\"" << K->E2 << "\" xlink:href=\"#E_" << K->name << "_" << unique_prefix << "\" />" << endl;
           }
         }
 
         for (list<scraprecord>::iterator K = SCRAPLIST.begin(); 
                                          K != SCRAPLIST.end(); K++) {
           if (used_scraps.count(K->name) > 0 && K->X != "") {
-            F << "<use x=\"" << K->X1 << "\" y=\"" << K->X2 << "\" xlink:href=\"#X_" << K->name << "\" />" << endl;
+            F << "<use x=\"" << K->X1 << "\" y=\"" << K->X2 << "\" xlink:href=\"#X_" << K->name << "_" << unique_prefix << "\" />" << endl;
           }
         }
 
@@ -508,13 +594,13 @@ void thsvg(char * fname, int fmt, legenddata ldata = legenddata::legenddata()) {
   }
 
   // preview up:
-  if (!MAP_PREVIEW_UP.empty()) print_preview(1,F);
+  if (!MAP_PREVIEW_UP.empty()) print_preview(1,F,unique_prefix);
   
   // surface:
   if (LAYOUT.surface == 2) print_surface_bitmaps(F);
 
   // grid:
-  if (LAYOUT.grid == 2) print_grid(F,llx,lly,urx,ury);
+  if (LAYOUT.grid == 2) print_grid(F,llxo,llyo,urxo,uryo,unique_prefix);
 
   // map grid:
   if (LAYOUT.map_grid) {
@@ -572,7 +658,6 @@ void thsvg(char * fname, int fmt, legenddata ldata = legenddata::legenddata()) {
       int rows = (int) ceil(double(legendbox_num) / columns);
       int pos = 0;
 
-      int i_patt = 0;
       F << "<table cellspacing=\"5\">" << endl;
       for (int i = 0; i < rows; i++) {
         F << "<tr>" << endl;
@@ -580,7 +665,7 @@ void thsvg(char * fname, int fmt, legenddata ldata = legenddata::legenddata()) {
           F << "<td>" << endl;
           pos = i + j * rows;
           if (pos < legendbox_num) {
-            L[pos].ldata.print_svg(F,++i_patt);
+            L[pos].ldata.print_svg(F,unique_prefix);
             F << L[pos].descr;
             // F << "</p>" << endl;
           }
