@@ -28,9 +28,13 @@
 
 #include <string>
 #include <list>
+#include <vector>
+#include <map>
 
 using namespace std;
 
+string select_lang(string s, string lang);
+
 struct fontrecord {
   int id;
   string rm,it,bf,ss,si;
@@ -38,7 +42,7 @@ struct fontrecord {
 
 extern list <fontrecord> FONTS;
 
-int get_enc_id(char* enc);
+int get_enc_id(const char* enc);
 void print_tex_encodings (void);
 void print_fonts_setup(void);
 
@@ -49,8 +53,29 @@ string utf2texhex(string s);
 string utf2tex (string s, bool b=false);
 int tex2uni(string font, int ch);
 
+struct encodings_new {
+ private:
+  vector<int> v_fon;  // tex - unicode
+  map<int,int> m_fon; // unicode - tex
+  int i_fon; // last position
+  
+ public:
+  int NFSS, t1_convert;
+  string otf_file[5]; 
+  
+  encodings_new();
+  
+  int get_enc_pos(int ch);
+  int get_enc_count();
+  int get_uni (int f, int ch);
+  void write_enc_files();
+};
+
+extern encodings_new ENC_NEW;
+
+
 #ifndef NOTHERION
-const char * utf2tex (char * s, bool b = false);
+const char * utf2tex (const char * s, bool b = false);
 #endif
 
 #endif
