@@ -41,7 +41,8 @@ set xth(gui,bacw) ".xth_bac"
 set xth(gui,dbg) ".xth_dbg"
 set xth(gui,help) ".xth_help"
 set xth(gui,message) ".xthmsg"
-set xth(gui,minsize) {480 300}
+#set xth(gui,minsize) {480 300}
+set xth(gui,minsize) {720 576}
 set xth(gui,balloons) 0
 set xth(gui,toolbar) 1
 set xth(te,template) {}
@@ -57,8 +58,12 @@ set xth(gui,auto_backup_ext) "~"
 set xth(gui,check_update) 1
 
 
-set xth(encodings) { iso8859-1 iso8859-2 iso8859-5 iso8859-7 utf-8 }
-set xth(kbencodings) {utf-8 iso8859-1 iso8859-2 cp1250 macCentEuro unicode}
+set xth(encodings) { iso8859-1 iso8859-2 iso8859-5 iso8859-7 utf-8 cp1250 cp1251 cp1252 cp1253}
+set xth(kbencodings) {utf-8 iso8859-1 iso8859-2 cp1250 cp1251 macCentEuro unicode}
+if {[lsearch -exact $xth(kbencodings) [encoding system]] < 0} {
+  lappend xth(kbencodings) [encoding system]
+}
+
 set xth(length_units) {m cm in ft yd}
 set xth(angle_units) {deg min grad}
 set xth(point_types) {}
@@ -98,12 +103,13 @@ set xth(datafmt,dimensions) {4.1fx {-}}
 set xth(gui,initdir) ""
 set xth(app,active) ""
 set xth(app,list) {}
-set xth(app,all,relw) -1
+set xth(app,all,tbwidth) 256
+set xth(app,all,relw) 380
 set xth(app,all,wmwd) 180
 set xth(app,all,wpsw) 1
 
 set xth(app,fencoding) utf-8
-set xth(app,sencoding) iso8859-2
+set xth(app,sencoding) utf-8
 
 # autodetect some options
 frame .def
@@ -147,6 +153,7 @@ set xth(gui,xvi_walls_oclr) gray60
 set xth(gui,xvi_shot_clr) gray50
 set xth(gui,xvi_station_fclr) black
 set xth(gui,xvi_station_oclr) black
+set xth(gui,xvi_sketch_line_width) 1
 
 set xth(gui,me,nozoom) 1
 
@@ -188,6 +195,7 @@ set xth(gui,me,controlfill) blue
 set xth(gui,me,highlightfill) cyan
 
 set xth(gui,me,typelistwidth) 16
+set xth(gui,rmb) 3
 
 set xth(gui,bindinsdel) 1
 
@@ -222,7 +230,7 @@ case $tcl_platform(platform) {
     set xth(gui,efont) "Courier 16 roman bold"
     set xth(gui,platform) windows
     set xth(gui,cursor) arrow
-    set xth(app,sencoding) cp1250
+    set xth(app,sencoding) [encoding system]
     set xth(gui,bindinsdel) 0
     if {[catch {
       set fid [open "|cmd.exe /c" r]
@@ -244,6 +252,15 @@ case $tcl_platform(platform) {
     set xth(app,sencoding) utf-8
   }
 }
+
+
+case $tcl_platform(os) {
+  Darwin {
+    set xth(gui,rmb) 2
+  }
+}
+
+
 # end of platform dependend settings
 
 # words to translate
@@ -463,18 +480,38 @@ proc xth_incr_station_name {oname iii} {
   } else {
     set svname {}
   }
-  if {[regexp {^\d+$} $oname]} {
-    incr oname $iii
-    return "$oname$svname"
-  } elseif {[regexp {^(.*\D)(\d+)$} $oname dumm s1 s2]} {
+  if {[regexp {^0*(\d+)$} $oname dum s2]} {
+    set numlen [string length $dum]
     incr s2 $iii
-    return "$s1$s2$svname"
-  } elseif {[regexp {^(\d+)(\D.*)$} $oname dumm s2 s1]} {
-    incr s2 $iii
-    return "$s2$s1$svname"
-  } else {
-    return "$oname$svname"
+    if {[string length $s2] < $numlen} {
+      set s2 [format [join [list "%0" $numlen d] ""] $s2]
+    }
+    return "$s2$svname"
   }
+  if {[regexp {^(.*\D)(0*(\d+))$} $oname dumm s1 s3 s2]} {
+    set numlen [string length $s3]
+    incr s2 $iii
+    if {[string length $s2] < $numlen} {
+      set s2 [format [join [list "%0" $numlen d] ""] $s2]
+    }
+    return "$s1$s2$svname"
+  } 
+  if {[regexp {^(.*)(\D)$} $oname dumm s2 s1]} {
+    set avs1 [scan $s1 %c]
+    if {(($avs1 >= 65) && ($avs1 < 90)) || (($avs1 >= 97) && ($avs1 < 122))} {
+      incr avs1
+      return "$s2[format %c $avs1]$svname"
+    }
+  } 
+  if {[regexp {^(0*(\d+))(\D.*)$} $oname dumm s3 s2 s1]} {
+    set numlen [string length $s3]
+    incr s2 $iii
+    if {[string length $s2] < $numlen} {
+      set s2 [format [join [list "%0" $numlen d] ""] $s2]
+    }
+    return "$s2$s1$svname"
+  } 
+  return "$oname$svname"
 }
 
 
