@@ -51,6 +51,9 @@ proc xth_me_reset_defaults {} {
     
 }
 
+set xth(me,acttheme) 0
+set xth(ctrl,me,acttheme) [lindex $xth(me,themes,showlist) 0]
+
 xth_me_reset_defaults
 
 proc xth_me_bind_entry_focusin {wlist} {
@@ -303,6 +306,8 @@ proc xth_me_create_file {} {
   $xth(me,menu,file) entryconfigure [mc "Save as"] -state normal
   $xth(me,menu,file) entryconfigure [mc "Auto save"] -state normal
   $xth(me,menu,file) entryconfigure [mc "Close"] -state normal
+
+  $xth(me,menu,file) entryconfigure [mc "Import"] -state normal
   
   $xth(me,menu) entryconfigure [mc "Edit"] -state normal
 
@@ -367,6 +372,8 @@ proc xth_me_destroy_file {} {
 
     set xth(ctrl,me,images,posx) ""
     set xth(ctrl,me,images,posy) ""
+    set xth(ctrl,me,images,newposx) 0
+    set xth(ctrl,me,images,newposy) 0
     set xth(ctrl,me,images,vis) 0
     $xth(ctrl,me,images).ic.insp configure -state disabled
     $xth(ctrl,me,area).l configure -state disabled -text ""
@@ -384,6 +391,8 @@ proc xth_me_destroy_file {} {
     $xth(me,menu,file) entryconfigure [mc "Auto save"] -state disabled
     $xth(me,menu,file) entryconfigure [mc "Close"] -state disabled
 
+    $xth(me,menu,file) entryconfigure [mc "Import"] -state disabled
+
     $xth(me,menu) entryconfigure [mc "Edit"] -state disabled
 
     $xth(ctrl,me,area).xmin configure -state disabled
@@ -435,9 +444,9 @@ proc xth_me_before_close_file {btns} {
       -font $xth(gui,lfont)]
     switch $wtd {
       0 {
-        if {[xth_me_save_file 0] == 0} {
-          return 0
-        }
+	if {[xth_me_save_file 0] == 0} {
+	  return 0
+	}
       }
       1 {}
       default {return 0}
@@ -476,17 +485,17 @@ proc xth_me_read_file {pth changebs} {
     incr flnn
     if {[regexp {^\s*encoding\s+(\S+)\s*$} $fln encln enc]} {
       if {$encspc} {
-        set success 0
-        set nm [format [mc "%s \[%s\] -- multiple encoding commands in file"] $pth $flnn]
-        break
+	set success 0
+	set nm [format [mc "%s \[%s\] -- multiple encoding commands in file"] $pth $flnn]
+	break
       }
       set encspc 1
       set rxp "\\s+($enc)\\s+"
       set validenc [regexp -nocase $rxp $xth(encodings) dum curenc]
       if {$validenc == 0} {
-        set success 0
-        set nm [format [mc "%s \[%s\] -- unknown encoding -- %s"] $pth $flnn $enc]
-        break
+	set success 0
+	set nm [format [mc "%s \[%s\] -- unknown encoding -- %s"] $pth $flnn $enc]
+	break
       }
       fconfigure $fid -encoding $curenc
       set lastln ""
@@ -494,19 +503,19 @@ proc xth_me_read_file {pth changebs} {
       lappend cmds $cmmd
       set lastln ""
       if {[regexp {^\s*\#\#BEGIN\#\#\s*$} $cmmd]} {
-        fconfigure $fid -encoding utf-8
+	fconfigure $fid -encoding utf-8
       }
       if {[regexp {^\s*\#\#END\#\#\s*$} $cmmd]} {
-        fconfigure $fid -encoding $curenc
+	fconfigure $fid -encoding $curenc
       }
     } else {
       if {$changebs && [regexp {(.*)\\\s*$} $lastln dumln prevln]} {
-        set fln "$prevln$fln"
-        if {[llength $lns] > 1} {
-          set lns [lrange $lns 0 [expr [llength $lns] - 2]]
-        } else {
-          set lns {}
-        }
+	set fln "$prevln$fln"
+	if {[llength $lns] > 1} {
+	  set lns [lrange $lns 0 [expr [llength $lns] - 2]]
+	} else {
+	  set lns {}
+	}
       }
       lappend lns $fln
       set lastln $fln
@@ -535,7 +544,7 @@ proc xth_me_write_file {pth} {
     set nm $errorInfo
     return [list $success $nm]
   }
-  fconfigure $fid -encoding utf-8 -translation {auto lf}
+  fconfigure $fid -encoding utf-8 -translation {auto auto}
   puts $fid "encoding  utf-8"
   # now let's put special commands
   puts $fid "##XTHERION## xth_me_area_adjust $xth(me,area,xmin) $xth(me,area,ymin) $xth(me,area,xmax) $xth(me,area,ymax)"
@@ -546,6 +555,7 @@ proc xth_me_write_file {pth} {
     set xxlist [linsert $xxlist 0 $imgx]
   }
   foreach imgx $xxlist {
+    if {$xth(me,imgs,$imgx,XVIimg)} continue
     set vsb $xth(me,imgs,$imgx,vsb)
     set gamma $xth(me,imgs,$imgx,gamma)
     if {$vsb < 0} {
@@ -556,11 +566,11 @@ proc xth_me_write_file {pth} {
     set ypos [expr [lindex $xth(me,imgs,$imgx,position) 1]]
     switch [llength $root] {
       1 {
-        set ypos [list $ypos [lindex $root 0]]
+	set ypos [list $ypos [lindex $root 0]]
       }
       3 {
-        set xpos [lindex $root 1]
-        set ypos [list [lindex $root 2] [lindex $root 0]]
+	set xpos [lindex $root 1]
+	set ypos [list [lindex $root 2] [lindex $root 0]]
       }
       0 {}
     }
@@ -609,9 +619,9 @@ proc xth_me_open_file {dialogid fname fline} {
   set fdata [xth_me_read_file $fname 1]
   if {[lindex $fdata 0] == 0} {
       MessageDlg $xth(gui,message) -parent $xth(gui,main) \
-        -icon error -type ok \
-        -message [lindex $fdata 1] \
-        -font $xth(gui,lfont)
+	-icon error -type ok \
+	-message [lindex $fdata 1] \
+	-font $xth(gui,lfont)
       xth_status_bar_pop me
       return 0
   }
@@ -679,7 +689,7 @@ proc xth_me_save_file {dialogid} {
     set fname [tk_getSaveFile -filetypes $xth(app,me,filetypes) \
       -parent $xth(gui,main) \
       -initialfile [file tail $fname] -defaultextension {.th2} \
-      -initialdir [file dirname $fname]]
+      -initialdir [xth_getdir $fname]]
   }
   
   if {[string length $fname] == 0} {
@@ -704,9 +714,9 @@ proc xth_me_save_file {dialogid} {
   set fdata [xth_me_write_file $fname]
   if {[lindex $fdata 0] == 0} {
       MessageDlg $xth(gui,message) -parent $xth(gui,main) \
-        -icon error -type ok \
-        -message [lindex $fdata 1] \
-        -font $xth(gui,lfont)
+	-icon error -type ok \
+	-message [lindex $fdata 1] \
+	-font $xth(gui,lfont)
       xth_status_bar_pop me
       return 0
   }
@@ -786,6 +796,50 @@ proc xth_me_limitize {limits x y} {
 }
 
 
+proc xth_me_image_newpos {} {
+
+  global xth 
+  set limits {}
+
+  # scan limits of pictures
+  foreach imgx $xth(me,imgs,xlist) {
+    set px [lindex $xth(me,imgs,$imgx,position) 0]
+    set py [lindex $xth(me,imgs,$imgx,position) 1]
+    set limits [xth_me_limitize $limits $px $py]
+    if {$xth(me,imgs,$imgx,XVI)} {
+      set gv $xth(me,imgs,$imgx,XVIgrid)
+      set v1x [expr [lindex $gv 2] * ([lindex $gv 6] - 1.0)]
+      set v1y [expr [lindex $gv 3] * ([lindex $gv 6] - 1.0)]
+      set v2x [expr [lindex $gv 4] * ([lindex $gv 7] - 1.0)]
+      set v2y [expr [lindex $gv 5] * ([lindex $gv 7] - 1.0)]
+      set limits [xth_me_limitize $limits [expr $px + $v1x] [expr $py + $v1y]]
+      set limits [xth_me_limitize $limits [expr $px + $v2x] [expr $py + $v2y]]
+      set limits [xth_me_limitize $limits [expr $px + $v1x + $v2x] [expr $py + $v1y + $v2y]]
+    } elseif {[string length $xth(me,imgs,$imgx,image)] > 0} {
+      set sx [image width $xth(me,imgs,$imgx,image)]
+      set sy [image height $xth(me,imgs,$imgx,image)]
+      set limits [xth_me_limitize $limits [expr $px + $sx] [expr $py - $sy]]
+    }
+  }
+
+  # scan limits of commands
+  set cmdlim [$xth(me,can) bbox command]
+  if {[llength $cmdlim] == 4} {
+    set limits [xth_me_limitize $limits [xth_me_can2realx [lindex $cmdlim 0]] [xth_me_can2realy [lindex $cmdlim 1]]]
+    set limits [xth_me_limitize $limits [xth_me_can2realx [lindex $cmdlim 2]] [xth_me_can2realy [lindex $cmdlim 3]]]
+  }
+
+  if {[llength $limits] > 0} {
+    set xth(ctrl,me,images,newposx) [lindex $limits 0]
+    set xth(ctrl,me,images,newposy) [expr 32.0 + [lindex $limits 3]]
+  } else {
+    set xth(ctrl,me,images,newposx) 0
+    set xth(ctrl,me,images,newposy) 0
+  }
+    
+}
+
+
 proc xth_me_area_auto_adjust {} {
   
   global xth 
@@ -811,7 +865,7 @@ proc xth_me_area_auto_adjust {} {
       set limits [xth_me_limitize $limits [expr $px + $sx] [expr $py - $sy]]
     }
   }
-  
+
   # scan limits of commands
   set cmdlim [$xth(me,can) bbox command]
   if {[llength $cmdlim] == 4} {
@@ -870,6 +924,9 @@ proc xth_me_area_adjust {x1 y1 x2 y2} {
   xth_me_area_scroll_adjust
   
   catch {$xth(ctrl,me,area).l configure -text [format "%.0f:%.0f - %.0f:%.0f" $x1 $y1 $x2 $y2]}
+
+  set xth(ctrl,me,images,newposx) $x1
+  set xth(ctrl,me,images,newposy) $y1
   
 }
 
@@ -959,29 +1016,29 @@ proc xth_me_area_zoom_to {zv} {
     xth_me_progbar_prog $ccmd
     switch $xth(me,cmds,$id,ct) {
       4 {
-        if {$id == $xth(me,cmds,selid)} {
-          xth_me_cmds_move_scrap_xctrl 1 [lindex $xth(me,cmds,$id,scale) 0] \
-            [lindex $xth(me,cmds,$id,scale) 1] 
-          xth_me_cmds_move_scrap_xctrl 2 [lindex $xth(me,cmds,$id,scale) 2] \
-            [lindex $xth(me,cmds,$id,scale) 3]
-        }
+	if {$id == $xth(me,cmds,selid)} {
+	  xth_me_cmds_move_scrap_xctrl 1 [lindex $xth(me,cmds,$id,scale) 0] \
+	    [lindex $xth(me,cmds,$id,scale) 1] 
+	  xth_me_cmds_move_scrap_xctrl 2 [lindex $xth(me,cmds,$id,scale) 2] \
+	    [lindex $xth(me,cmds,$id,scale) 3]
+	}
       }
       3 {
-        xth_me_cmds_move_line $id
-        if {$id == $xth(me,cmds,selid)} {
-          xth_me_cmds_show_line_xctrl $id
-          set pid $xth(me,cmds,selpid)
-          if {$pid != 0} {
-            xth_me_cmds_show_linept_xctrl $id $pid
-          }
-        }
+	xth_me_cmds_move_line $id
+	if {$id == $xth(me,cmds,selid)} {
+	  xth_me_cmds_show_line_xctrl $id
+	  set pid $xth(me,cmds,selpid)
+	  if {$pid != 0} {
+	    xth_me_cmds_show_linept_xctrl $id $pid
+	  }
+	}
       }
       2 {
-        $xth(me,can) coords pt$id [xth_me_cmds_calc_point_coords $id]
-        if {$id == $xth(me,cmds,selid)} {
-          xth_me_cmds_move_point_xctrl $id
-          xth_me_cmds_move_point_fill_xctrl $id $xth(me,cmds,$id,rotation) $xth(me,cmds,$id,xsize) $xth(me,cmds,$id,ysize)
-        }
+	$xth(me,can) coords pt$id [xth_me_cmds_calc_point_coords $id]
+	if {$id == $xth(me,cmds,selid)} {
+	  xth_me_cmds_move_point_xctrl $id
+	  xth_me_cmds_move_point_fill_xctrl $id $xth(me,cmds,$id,rotation) $xth(me,cmds,$id,xsize) $xth(me,cmds,$id,ysize)
+	}
       }
     }
   }
@@ -1019,8 +1076,8 @@ proc xth_me_area_start_drag {tagOrId imgx x y} {
   $xth(me,can) configure -cursor plus
   $xth(me,can) bind $tagOrId <Shift-B1-Motion> "xth_me_area_drag %x %y"
   $xth(me,can) bind $tagOrId <Shift-B1-ButtonRelease> "xth_me_area_end_drag $tagOrId \"$imgx\" %x %y"
-  $xth(me,can) bind $tagOrId <B3-Motion> "xth_me_area_drag %x %y"
-  $xth(me,can) bind $tagOrId <B3-ButtonRelease> "xth_me_area_end_drag $tagOrId \"$imgx\" %x %y"
+  $xth(me,can) bind $tagOrId <B$xth(gui,rmb)-Motion> "xth_me_area_drag %x %y"
+  $xth(me,can) bind $tagOrId <B$xth(gui,rmb)-ButtonRelease> "xth_me_area_end_drag $tagOrId \"$imgx\" %x %y"
   update idletasks
 }
 
@@ -1040,8 +1097,8 @@ proc xth_me_area_end_drag {tagOrId imgx x y} {
   $xth(me,can) configure -cursor crosshair
   $xth(me,can) bind $tagOrId <Shift-B1-Motion> ""
   $xth(me,can) bind $tagOrId <Shift-B1-ButtonRelease> ""
-  $xth(me,can) bind $tagOrId <B3-Motion> ""
-  $xth(me,can) bind $tagOrId <B3-ButtonRelease> ""
+  $xth(me,can) bind $tagOrId <B$xth(gui,rmb)-Motion> ""
+  $xth(me,can) bind $tagOrId <B$xth(gui,rmb)-ButtonRelease> ""
   if {[string length $imgx] > 0} {
     xth_me_image_choose $imgx
   }
@@ -1061,13 +1118,13 @@ proc xth_me_bind_area_drag {tagOrId imgx} {
   $xth(me,can) bind $tagOrId <1> "xth_me_cmds_click_area $tagOrId %x %y"
   $xth(me,can) bind $tagOrId <Motion> "xth_me_area_motion %x %y"
   $xth(me,can) bind $tagOrId <Shift-1> "xth_me_area_start_drag $tagOrId \"$imgx\" %x %y"
-  $xth(me,can) bind $tagOrId <3> "xth_me_area_start_drag $tagOrId \"$imgx\" %x %y"
+  $xth(me,can) bind $tagOrId <$xth(gui,rmb)> "xth_me_area_start_drag $tagOrId \"$imgx\" %x %y"
 }
 
 proc xth_me_bind_area_only_drag {tagOrId} {
   global xth
   $xth(me,can) bind $tagOrId <Shift-1> "xth_me_area_start_drag $tagOrId {} %x %y"
-  $xth(me,can) bind $tagOrId <3> "xth_me_area_start_drag $tagOrId {} %x %y"
+  $xth(me,can) bind $tagOrId <$xth(gui,rmb)> "xth_me_area_start_drag $tagOrId {} %x %y"
 }
 
 
@@ -1082,8 +1139,8 @@ proc xth_me_image_start_drag {tagOrId imgx x y} {
   $xth(me,can) configure -cursor fleur
   $xth(me,can) bind $tagOrId <Shift-B1-Motion> "xth_me_image_drag $imgx %x %y"
   $xth(me,can) bind $tagOrId <Shift-B1-ButtonRelease> "xth_me_image_end_drag $tagOrId $imgx %x %y"
-  $xth(me,can) bind $tagOrId <B3-Motion> "xth_me_image_drag $imgx %x %y"
-  $xth(me,can) bind $tagOrId <B3-ButtonRelease> "xth_me_image_end_drag $tagOrId $imgx %x %y"
+  $xth(me,can) bind $tagOrId <B$xth(gui,rmb)-Motion> "xth_me_image_drag $imgx %x %y"
+  $xth(me,can) bind $tagOrId <B$xth(gui,rmb)-ButtonRelease> "xth_me_image_end_drag $tagOrId $imgx %x %y"
   update idletasks
 }
 
@@ -1102,8 +1159,8 @@ proc xth_me_image_end_drag {tagOrId imgx x y} {
   $xth(me,can) configure -cursor crosshair
   $xth(me,can) bind $tagOrId <Shift-B1-Motion> ""
   $xth(me,can) bind $tagOrId <Shift-B1-ButtonRelease> ""
-  $xth(me,can) bind $tagOrId <B3-Motion> ""
-  $xth(me,can) bind $tagOrId <B3-ButtonRelease> ""
+  $xth(me,can) bind $tagOrId <B$xth(gui,rmb)-Motion> ""
+  $xth(me,can) bind $tagOrId <B$xth(gui,rmb)-ButtonRelease> ""
   xth_me_image_choose $imgx
   xth_me_unredo_action [mc "dragging image"] \
     "xth_me_image_move $imgx $xth(me,imgs,drag_px) $xth(me,imgs,drag_py)" \
@@ -1115,7 +1172,7 @@ proc xth_me_image_end_drag {tagOrId imgx x y} {
 proc xth_me_bind_image_drag {tagOrId imgx} {
   global xth
   $xth(me,can) bind $tagOrId <Double-Shift-1> "xth_me_image_start_drag $tagOrId $imgx %x %y"
-  $xth(me,can) bind $tagOrId <Double-3> "xth_me_image_start_drag $tagOrId $imgx %x %y"
+  $xth(me,can) bind $tagOrId <Double-$xth(gui,rmb)> "xth_me_image_start_drag $tagOrId $imgx %x %y"
 }
 
 
@@ -1155,6 +1212,8 @@ set xth(me,snai) 1
 
 set xth(ctrl,me,images,posx) ""
 set xth(ctrl,me,images,posy) ""
+set xth(ctrl,me,images,newposx) 0
+set xth(ctrl,me,images,newposy) 0
 set xth(ctrl,me,images,vis) 0
 set xth(ctrl,me,images,gamma) 0.0
 set xth(ctrl,me,area,xmin) ""
@@ -1178,6 +1237,8 @@ set xth(ctrl,me,scrap,ry2) ""
 set xth(ctrl,me,scrap,units) ""
 set xth(ctrl,me,scrap,pp1) {}
 set xth(ctrl,me,scrap,pp2) {}
+set xth(ctrl,me,scrap,sklempty) {}
+
 
 set xth(ctrl,me,point,x) {}
 set xth(ctrl,me,point,y) {}
@@ -1250,6 +1311,12 @@ set xth(me,canid,scrap,scp1) [$xth(me,can) create rectangle 0 0 3 3 \
   -fill red -outline red -width 1 -state hidden -tags {cmd_ctrl}]
 set xth(me,canid,scrap,scp2) [$xth(me,can) create rectangle 0 0 3 3 \
   -fill red -outline red -width 1 -state hidden -tags {cmd_ctrl}]
+set xth(me,canid,scrap,scpA) [$xth(me,can) create line 0 0 3 3 \
+  -fill red -width 5 -state hidden -arrow last -arrowshape {18 20 5} -tags {cmd_ctrl}]
+
+set xth(me,canid,scrap,scp1,pos) {0 0}
+set xth(me,canid,scrap,scp2,pos) {0 0}
+set xth(me,cmds,scrap_scale) 0
 
 xth_me_bind_area_drag $xth(me,canid,area) {}
 xth_me_area_adjust 0 0 1600 1200
@@ -1577,7 +1644,8 @@ grid $xth(ctrl,me,images).ic -column 0 -row 1 -sticky news
 Button $xth(ctrl,me,images).ic.insp -text [mc "Insert"] -anchor center -font $xth(gui,lfont) \
   -state disabled -command {
 		set nimgs [llength $xth(me,imgs,xlist)]
-    xth_me_image_insert $xth(ctrl,me,images,posx) $xth(ctrl,me,images,posy) {} 0 {}
+    xth_me_image_newpos
+    xth_me_image_insert $xth(ctrl,me,images,newposx) $xth(ctrl,me,images,newposy) {} 0 {}
 		set nnimgs [llength $xth(me,imgs,xlist)]
 		if {$nnimgs > $nimgs} {
 	    xth_me_area_auto_adjust
@@ -1690,7 +1758,7 @@ xth_status_bar me $sfm.name [mc "Scrap name."]
 
 Label $sfm.projl -text [mc "projection"] -anchor e -font $xth(gui,lfont) -state disabled
 xth_status_bar me $sfm.projl [mc "Scrap projection."]
-ComboBox $sfm.proj -values $xth(scrap_projections) \
+ComboBox $sfm.proj -values $xth(scrap_projections) -autocomplete 1 \
   -font $xth(gui,lfont) -height 4 -state disabled -width 4 \
   -textvariable xth(ctrl,me,scrap,projection) -command {xth_me_cmds_update {}}
 
@@ -1705,8 +1773,8 @@ Entry $sfm.opt -font $xth(gui,lfont) -state disabled -width 4 \
 xth_status_bar me $sfm.opt [mc "Other scrap options."] 
 
 #Separator $sfm.s1 -orient horizontal
-Label $sfm.scl -text [mc "scale"] -anchor sw -font $xth(gui,lfont) -state disabled
-xth_status_bar me $sfm.scl [mc "Scrap scale definition."]
+Button $sfm.scl -text [mc "Scale"] -anchor center -font $xth(gui,lfont) -state disabled -command xth_me_cmds_scrap_scale_start
+xth_status_bar me $sfm.scl [mc "Scrap scaling - push to click picture calibration points."]
 Button $sfm.scpb -text [mc "Update scrap"] -anchor center -font $xth(gui,lfont) \
   -state disabled -width 4 -command {xth_me_cmds_update {}}
 xth_status_bar me $sfm.scpb [mc "Press this button to apply your changes to current scrap."]
@@ -1750,6 +1818,42 @@ xth_me_bind_entry_focus_return "$sfm.scx1r $sfm.scy1r $sfm.scx2r $sfm.scy2r" {xt
 xth_me_bind_entry_return "$sfm.name $sfm.opt" {xth_me_cmds_update {}}
 xth_me_bind_entry_focusin "$sfm.name $sfm.opt $sfm.scx1p $sfm.scy1p $sfm.scx2p $sfm.scy2p $sfm.scx1r $sfm.scy1r $sfm.scx2r $sfm.scy2r"
 
+
+# scrap background sketches
+Label $sfm.skl -text [mc "background sketches"] -anchor w -font $xth(gui,lfont) -state disabled
+xth_status_bar me $sfm.skl [mc "List of image files used as scrap background."]
+
+
+set plf $sfm.skll
+frame $plf
+listbox $plf.l -height 2 -selectmode single -takefocus 0 \
+  -yscrollcommand "xth_scroll $plf.sv" \
+  -xscrollcommand "xth_scroll $plf.sh" \
+  -font $xth(gui,lfont) -exportselection no \
+  -selectborderwidth 0
+scrollbar $plf.sv -orient vert  -command "$plf.l yview" \
+  -takefocus 0 -width $xth(gui,sbwidth) -borderwidth $xth(gui,sbwidthb)
+scrollbar $plf.sh -orient horiz  -command "$plf.l xview" \
+  -takefocus 0 -width $xth(gui,sbwidth) -borderwidth $xth(gui,sbwidthb)
+bind $plf.l <B1-ButtonRelease> "focus $plf.l"
+
+Button $sfm.skins -text [mc "Insert"] -anchor center -font $xth(gui,lfont) \
+  -state disabled -width 10 -command xth_me_cmds_scrap_sketch_start
+xth_status_bar me $sfm.skins [mc "Press this button to insert sketch by clicking on it."]
+Button $sfm.skdel -text [mc "Delete"] -anchor center -font $xth(gui,lfont) \
+  -state disabled -width 10 -command {xth_me_cmds_delete_scrap_sketch {} {}}
+xth_status_bar me $sfm.skdel [mc "Delete sketch from scrap."]
+
+grid columnconf $plf 0 -weight 1
+grid rowconf $plf 0 -weight 1
+grid $plf.l -column 0 -row 0 -sticky news
+xth_scroll_showcmd $plf.sv "grid $plf.sv -column 1 -row 0 -sticky news"
+xth_scroll_hidecmd $plf.sv "grid forget $plf.sv"
+xth_scroll_showcmd $plf.sh "grid $plf.sh -column 0 -row 1 -sticky news"
+xth_scroll_hidecmd $plf.sh "grid forget $plf.sh"
+xth_status_bar me $plf [mc "Select line in area."]
+
+
 grid columnconf $sfm 0 -weight 1
 grid columnconf $sfm 1 -weight 1
 grid columnconf $sfm 2 -weight 1
@@ -1770,11 +1874,17 @@ grid $sfm.scx1r $sfm.scy1r $sfm.scx2r $sfm.scy2r -row 7 -sticky news -padx 1
 grid $sfm.scul -row 8 -column 0 -columnspan 2 -sticky news
 grid $sfm.scu  -row 8 -column 2 -columnspan 2 -sticky news -padx 2
 
+grid $sfm.skl -column 0 -row 9 -sticky news -columnspan 4
+grid $sfm.skll -column 0 -row 10 -sticky news -columnspan 4
+grid $sfm.skins -row 11 -column 0 -columnspan 2 -sticky news
+grid $sfm.skdel -row 11 -column 2 -columnspan 2 -sticky news
+
 
 # point control
 xth_about_status [mc "loading point module ..."]
 
 set ptc $xth(ctrl,me,point)
+
 Label $ptc.posl -text [mc "position"] -anchor e -font $xth(gui,lfont) -state disabled -width 8
 xth_status_bar me $ptc.posl [mc "Point position."]
 Entry $ptc.posx -font $xth(gui,lfont) -state disabled -width 4 -textvariable xth(ctrl,me,point,x)
@@ -1786,11 +1896,23 @@ Button $ptc.upd -text [mc "Update point"] -anchor center -font $xth(gui,lfont) \
   -state disabled -command {xth_me_cmds_update {}}
 xth_status_bar me $ptc.upd [mc "Click this button to apply point changes."] 
 
+ComboBox $ptc.theme -values $xth(me,themes,showlist) -autocomplete 1 -editable 0 \
+  -font $xth(gui,lfont) -height $xth(gui,me,typelistwidth) -state disabled -width 4 \
+  -modifycmd "xth_me_set_theme \[$ptc.theme getvalue\]" -fg blue \
+  -textvariable xth(ctrl,me,acttheme)
+xth_status_bar me $ptc.theme [mc "Actual symbol theme."]
+ComboBox $ptc.themetype -values $xth(me,themes,all-symbols,point,showlist) -autocomplete 1 -editable 0 \
+  -font $xth(gui,lfont) -height $xth(gui,me,typelistwidth) -state disabled -width 4  -fg blue \
+  -textvariable xth(ctrl,me,point,themetype) \
+  -modifycmd xth_me_point_themetype_modified
+xth_status_bar me $ptc.themetype [mc "Point type."]
+
 Label $ptc.typl -text [mc "type"] -anchor e -font $xth(gui,lfont) -state disabled -width 8
 xth_status_bar me $ptc.typl [mc "Point type."] 
-ComboBox $ptc.typ -values $xth(point_types) \
+ComboBox $ptc.typ -values $xth(point_types) -autocomplete 1 \
   -font $xth(gui,lfont) -height $xth(gui,me,typelistwidth) -state disabled -width 4 \
-  -textvariable xth(ctrl,me,point,type) -command {xth_me_cmds_update {}}
+  -textvariable xth(ctrl,me,point,type) -command {xth_me_cmds_update {}} \
+  -modifycmd {xth_me_cmds_update {}}
 xth_status_bar me $ptc.typ [mc "Point type."]
 xth_me_bind_typecbx_hint $ptc.typ point
 
@@ -1805,6 +1927,7 @@ xth_status_bar me $ptc.optl [mc "Other point options."]
 Entry $ptc.opt -font $xth(gui,lfont) -state disabled -width 4 \
   -textvariable xth(ctrl,me,point,opts)
 xth_status_bar me $ptc.opt [mc "I.e, if type is label, write here: -text \042this will be displayed (P100)\042"]
+
 Separator $ptc.s1 -orient horizontal
 
 checkbutton $ptc.rotc -text [mc "orientation"] -anchor w -font $xth(gui,lfont) -state disabled \
@@ -1837,24 +1960,36 @@ grid columnconf $ptc 0 -weight 1
 grid columnconf $ptc 1 -weight 1
 grid columnconf $ptc 2 -weight 1
 grid columnconf $ptc 3 -weight 1
-grid $ptc.posl -row 0 -column 0 -columnspan 2 -sticky news
-grid $ptc.posx -row 0 -column 2 -sticky news -padx 1
-grid $ptc.posy -row 0 -column 3 -sticky news -padx 1
-grid $ptc.typl -row 1 -column 0 -columnspan 2 -sticky news
-grid $ptc.typ -row 1 -column 2 -columnspan 2 -sticky news -padx 2
-grid $ptc.namel -row 2 -column 0 -columnspan 2 -sticky news
-grid $ptc.name -row 2 -column 2 -columnspan 2 -sticky news -padx 1
-grid $ptc.optl -row 3 -column 0 -columnspan 2 -sticky news
-grid $ptc.opt -row 3 -column 2 -columnspan 2 -sticky news -padx 1
-grid $ptc.s1 -row 4 -column 0 -columnspan 4 -sticky news -pady 3
-grid $ptc.rotc -row 5 -column 0 -columnspan 2 -sticky news
-grid $ptc.rot -row 5 -column 2 -columnspan 2 -sticky news -padx 1
+
+set crow 0
+grid $ptc.theme -row $crow -column 0 -columnspan 2 -sticky news -padx 2 -pady 2
+grid $ptc.themetype -row $crow -column 2 -columnspan 2 -sticky news -padx 2 -pady 2
+incr crow
+grid $ptc.typl -row $crow -column 0 -columnspan 2 -sticky news
+grid $ptc.typ -row $crow -column 2 -columnspan 2 -sticky news -padx 2
+incr crow
+grid $ptc.s1 -row $crow -column 0 -columnspan 4 -sticky news -pady 2
+incr crow
+grid $ptc.posl -row $crow -column 0 -columnspan 2 -sticky news
+grid $ptc.posx -row $crow -column 2 -sticky news -padx 2 -pady 1
+grid $ptc.posy -row $crow -column 3 -sticky news -padx 2 -pady 1
+incr crow
+grid $ptc.namel -row $crow -column 0 -columnspan 2 -sticky news
+grid $ptc.name -row $crow -column 2 -columnspan 2 -sticky news -padx 2 -pady 1
+incr crow
+grid $ptc.optl -row $crow -column 0 -columnspan 2 -sticky news
+grid $ptc.opt -row $crow -column 2 -columnspan 2 -sticky news -padx 2 -pady 1
+incr crow
+grid $ptc.rotc -row $crow -column 0 -columnspan 2 -sticky news
+grid $ptc.rot -row $crow -column 2 -columnspan 2 -sticky news -padx 2 -pady 1
+incr crow
 ## DISABLED
 #grid $ptc.xszc -row 6 -column 0 -columnspan 2 -sticky news
 #grid $ptc.xsz -row 6 -column 2 -columnspan 2 -sticky news -padx 1
 #grid $ptc.yszc -row 7 -column 0 -columnspan 2 -sticky news
 #grid $ptc.ysz -row 7 -column 2 -columnspan 2 -sticky news -padx 1
-grid $ptc.upd -row 8 -column 0 -columnspan 4 -sticky news
+grid $ptc.upd -row $crow -column 0 -columnspan 4 -sticky news
+incr crow
 
 
 # line control
@@ -1862,15 +1997,28 @@ xth_about_status [mc "loading line module ..."]
 
 set lnc $xth(ctrl,me,line)
 
+ComboBox $lnc.theme -values $xth(me,themes,showlist) -autocomplete 1 -editable 0 \
+  -font $xth(gui,lfont) -height $xth(gui,me,typelistwidth) -state disabled -width 4 \
+  -modifycmd "xth_me_set_theme \[$ptc.theme getvalue\]" -fg blue \
+  -textvariable xth(ctrl,me,acttheme)
+xth_status_bar me $lnc.theme [mc "Actual symbol theme."]
+ComboBox $lnc.themetype -values $xth(me,themes,all-symbols,line,showlist) -autocomplete 1 -editable 0 \
+  -font $xth(gui,lfont) -height $xth(gui,me,typelistwidth) -state disabled -width 4 -fg blue \
+  -textvariable xth(ctrl,me,line,themetype) \
+  -modifycmd xth_me_line_themetype_modified
+xth_status_bar me $lnc.themetype [mc "Line type."]
+
 Label $lnc.typl -text [mc "type"] -anchor e -font $xth(gui,lfont) -state disabled
 xth_status_bar me $lnc.typl [mc "Line type."]
-ComboBox $lnc.typ -values $xth(line_types) \
+ComboBox $lnc.typ -values $xth(line_types) -autocomplete 1  \
   -font $xth(gui,lfont) -height $xth(gui,me,typelistwidth) -state disabled -width 4 \
-  -textvariable xth(ctrl,me,line,type) \
+  -textvariable xth(ctrl,me,line,type) -modifycmd {xth_me_cmds_update {}} \
   -command {xth_me_cmds_update {}}
 xth_status_bar me $lnc.typ [mc "Line type."]
 xth_me_bind_typecbx_hint $lnc.typ line
 
+Separator $lnc.s1
+
 Label $lnc.namel -text [mc "id"] -anchor e -font $xth(gui,lfont) -state disabled
 xth_status_bar me $lnc.namel [mc "Line identifier."]
 Entry $lnc.name -font $xth(gui,lfont) -state disabled -width 4 \
@@ -1912,8 +2060,8 @@ bind $plf.l <<ListboxSelect>> {
     [lindex [%W curselection] 0]]
     if {$xth(me,cmds,selpid) > 0} {
       xth_me_center_to [list \
-        $xth(me,cmds,$xth(me,cmds,selid),$xth(me,cmds,selpid),x) \
-        $xth(me,cmds,$xth(me,cmds,selid),$xth(me,cmds,selpid),y)]
+	$xth(me,cmds,$xth(me,cmds,selid),$xth(me,cmds,selpid),x) \
+	$xth(me,cmds,$xth(me,cmds,selid),$xth(me,cmds,selpid),y)]
     }
   }
 }
@@ -1948,17 +2096,30 @@ xth_status_bar me $plf [mc "Select line point."]
 
 grid columnconf $lnc 0 -weight 1
 grid columnconf $lnc 1 -weight 1
-grid $lnc.typl -row 0 -column 0 -sticky news
-grid $lnc.typ -row 0 -column 1 -sticky news -padx 2
-grid $lnc.namel -row 1 -column 0 -sticky news
-grid $lnc.name -row 1 -column 1 -sticky news -padx 1
-grid $lnc.optl -row 2 -column 0 -sticky news
-grid $lnc.opt -row 2 -column 1 -sticky news -padx 1
-grid $lnc.rev -row 3 -column 0 -sticky news
-grid $lnc.cls -row 3 -column 1 -sticky news
-grid $plf -row 4 -column 0 -columnspan 2 -sticky news
-grid $lnc.lpa -row 5 -column 0 -sticky news
-grid $lnc.upd -row 5 -column 1 -sticky news
+set crow 0
+grid $lnc.theme -row $crow -column 0 -sticky news -padx 2 -pady 2
+grid $lnc.themetype -row $crow -column 1 -sticky news -padx 2 -pady 2
+incr crow
+grid $lnc.typl -row $crow -column 0 -sticky news
+grid $lnc.typ -row $crow -column 1 -sticky news -padx 2
+incr crow
+grid $lnc.s1 -row $crow -column 0 -columnspan 2 -sticky news -padx 2 -pady 2
+incr crow
+grid $lnc.namel -row $crow -column 0 -sticky news
+grid $lnc.name -row $crow -column 1 -sticky news -padx 2 -pady 1
+incr crow
+grid $lnc.optl -row $crow -column 0 -sticky news
+grid $lnc.opt -row $crow -column 1 -sticky news -padx 2 -pady 1
+incr crow
+grid $lnc.rev -row $crow -column 0 -sticky news
+grid $lnc.cls -row $crow -column 1 -sticky news
+incr crow
+grid $plf -row $crow -column 0 -columnspan 2 -sticky news
+incr crow
+grid $lnc.lpa -row $crow -column 0 -sticky news
+grid $lnc.upd -row $crow -column 1 -sticky news
+incr crow
+
 #grid $lnc.insp -row 5 -column 0 -sticky news
 #grid $lnc.delp -row 5 -column 1 -sticky news
 
@@ -2137,11 +2298,23 @@ xth_about_status [mc "loading area module ..."]
 
 set lnc $xth(ctrl,me,ac)
 
+ComboBox $lnc.theme -values $xth(me,themes,showlist) -autocomplete 1 -editable 0 \
+  -font $xth(gui,lfont) -height $xth(gui,me,typelistwidth) -state disabled -width 4 \
+  -modifycmd "xth_me_set_theme \[$ptc.theme getvalue\]" -fg blue \
+  -textvariable xth(ctrl,me,acttheme)
+xth_status_bar me $lnc.theme [mc "Actual symbol theme."]
+ComboBox $lnc.themetype -values $xth(me,themes,all-symbols,area,showlist) -autocomplete 1 -editable 0 \
+  -font $xth(gui,lfont) -height $xth(gui,me,typelistwidth) -state disabled -width 4 -fg blue \
+  -textvariable xth(ctrl,me,ac,themetype) \
+  -modifycmd xth_me_area_themetype_modified
+xth_status_bar me $lnc.themetype [mc "Area type."]
+Separator $lnc.s1
+
 Label $lnc.typl -text [mc "type"] -anchor e -font $xth(gui,lfont) -state disabled
 xth_status_bar me $lnc.typl [mc "Area type."]
-ComboBox $lnc.typ -values $xth(area_types) \
+ComboBox $lnc.typ -values $xth(area_types) -autocomplete 1 \
   -font $xth(gui,lfont) -height $xth(gui,me,typelistwidth) -state disabled -width 4 \
-  -textvariable xth(ctrl,me,ac,type) \
+  -textvariable xth(ctrl,me,ac,type) -modifycmd {xth_me_cmds_update {}} \
   -command {xth_me_cmds_update {}}
 xth_status_bar me $lnc.typ [mc "Area type."] 
 xth_me_bind_typecbx_hint $lnc.typ area
@@ -2163,7 +2336,7 @@ scrollbar $plf.sv -orient vert  -command "$plf.l yview" \
   -takefocus 0 -width $xth(gui,sbwidth) -borderwidth $xth(gui,sbwidthb)
 scrollbar $plf.sh -orient horiz  -command "$plf.l xview" \
   -takefocus 0 -width $xth(gui,sbwidth) -borderwidth $xth(gui,sbwidthb)
-bind $plf.l <<ListboxSelect>> {}
+bind $plf.l <<ListboxSelect>> {xth_me_cmds_show_current_area}
 bind $plf.l <B1-ButtonRelease> "focus $plf.l"
 
 Button $lnc.ins -text [mc "Select lines"] -anchor center -font $xth(gui,lfont) \
@@ -2198,18 +2371,29 @@ xth_status_bar me $plf [mc "Select line in area."]
 
 grid columnconf $lnc 0 -weight 1
 grid columnconf $lnc 1 -weight 1
-grid $lnc.typl -row 0 -column 0 -sticky news
-grid $lnc.typ -row 0 -column 1 -sticky news -padx 2
-grid $lnc.optl -row 1 -column 0 -sticky news
-grid $lnc.opt -row 1 -column 1 -sticky news -padx 1
-grid $lnc.shw -row 2 -column 0 -sticky news
-grid $lnc.upd -row 2 -column 1 -sticky news
-grid $plf -row 3 -column 0 -columnspan 2 -sticky news
-grid $lnc.ins -row 4 -column 0 -sticky news
-grid $lnc.del -row 4 -column 1 -sticky news
-grid $lnc.insid -row 5 -column 0 -sticky news
-grid $lnc.inside -row 5 -column 1 -sticky news
-
+set crow 0
+grid $lnc.theme -row $crow -column 0 -sticky news -padx 2 -pady 2
+grid $lnc.themetype -row $crow -column 1 -sticky news -padx 2 -pady 2
+incr crow
+grid $lnc.typl -row $crow -column 0 -sticky news
+grid $lnc.typ -row $crow -column 1 -sticky news -padx 2 -pady 1
+incr crow
+grid $lnc.s1 -row $crow -column 0 -columnspan 2 -sticky news -padx 2 -pady 2
+incr crow
+grid $lnc.optl -row $crow -column 0 -sticky news
+grid $lnc.opt -row $crow -column 1 -sticky news -padx 2 -pady 1
+incr crow
+grid $lnc.shw -row $crow -column 0 -sticky news
+grid $lnc.upd -row $crow -column 1 -sticky news
+incr crow
+grid $plf -row $crow -column 0 -columnspan 2 -sticky news
+incr crow
+grid $lnc.ins -row $crow -column 0 -sticky news
+grid $lnc.del -row $crow -column 1 -sticky news
+incr crow
+grid $lnc.insid -row $crow -column 0 -sticky news
+grid $lnc.inside -row $crow -column 1 -sticky news
+incr crow
 xth_me_bind_entry_return "$lnc.opt" {xth_me_cmds_update {}}
 xth_me_bind_entry_focusin "$lnc.opt"
 
@@ -2223,14 +2407,14 @@ $xth(me,menu,file) add command -label [mc "New"] -command xth_me_create_file \
 $xth(me,menu,file) add command -label [mc "Open"] -underline 0 \
   -accelerator "$xth(gui,controlk)-o" -state normal \
   -font $xth(gui,lfont) -command {
-			set xth(gui,openxp) 0
-			xth_me_open_file 1 {} 1
+		        set xth(gui,openxp) 0
+		        xth_me_open_file 1 {} 1
 	}
 $xth(me,menu,file) add command -label [mc "Open (no pics)"] -underline 10 \
   -state normal -font $xth(gui,lfont) -command {
 	    set xth(gui,openxp) 1
-			xth_me_open_file 1 {} 1
-			set xth(gui,openxp) 0
+		        xth_me_open_file 1 {} 1
+		        set xth(gui,openxp) 0
 	}
 $xth(me,menu,file) add command -label [mc "Save"] -underline 0 \
   -accelerator "$xth(gui,controlk)-s" -state disabled \
@@ -2245,6 +2429,10 @@ $xth(me,menu,file) add command -label [mc "Close"] -underline 0 \
   -font $xth(gui,lfont) \
   -command xth_me_close_file
 
+$xth(me,menu,file) add separator
+$xth(me,menu,file) add command -label [mc "Import"] -underline 0 \
+  -state disabled -font $xth(gui,lfont) -command {xth_me_import_file {} {}}
+
 set xth(me,menu,edit) $xth(me,menu).edit
 menu $xth(me,menu,edit) -tearoff 0
 menu $xth(me,menu,edit).ins -tearoff 0
@@ -2309,7 +2497,15 @@ $xth(me,menu,edit) add command -label [mc "Auto adjust area"] \
   -font $xth(gui,lfont) -command xth_me_area_auto_adjust
 $xth(me,menu,edit) add command -label [mc "Insert image"] \
   -font $xth(gui,lfont) \
-  -command {xth_me_image_insert $xth(ctrl,me,images,posx) $xth(ctrl,me,images,posy) {} 0 {}}
+  -command {
+  set nimgs [llength $xth(me,imgs,xlist)]
+  xth_me_image_newpos
+  xth_me_image_insert $xth(ctrl,me,images,newposx) $xth(ctrl,me,images,newposy) {} 0 {}
+  set nnimgs [llength $xth(me,imgs,xlist)]
+  if {$nnimgs > $nimgs} {
+    xth_me_area_auto_adjust
+  }
+}
 set xth(me,menu,edit,undo) [$xth(me,menu,edit) index [mc "Undo"]]
 set xth(me,menu,edit,redo) [$xth(me,menu,edit) index [mc "Redo"]]
 set xth(me,menu,edit,zoom) [$xth(me,menu,edit) index [mc "Zoom 100 %"]]
@@ -2323,7 +2519,7 @@ grid columnconf $barfm 0 -weight 1
 grid rowconf $barfm 0 -weight 1
 
 set xth(me,mbar) $barfm.mbar
-Label $xth(me,mbar) -text "" -width 17 -relief sunken -font $xth(gui,lfont) \
+Label $xth(me,mbar) -text "" -width 20 -relief sunken -font $xth(gui,lfont) \
   -anchor center -state disabled
 grid $xth(me,mbar) -column 0 -row 0 -sticky news
 set xth(me,mbar,bg) [$xth(me,mbar) cget -bg]
@@ -2381,6 +2577,7 @@ set xth(ctrl,me,area,ymin) ""
 set xth(ctrl,me,area,xmax) ""
 set xth(ctrl,me,area,ymax) ""
 
+xth_me_set_theme 0
 xth_about_status [mc "loading line procs ..."]
 xth_status_bar_status me [mc "Map editor is not active. To activate it, open existing file or create new one."]
 
