@@ -1547,36 +1547,36 @@ proc xth_me_cmds_create_point {ix mode x y type opts} {
     }
   }
 
-  # nastavit xsize
-  if {$mode} {
-    set xth(me,cmds,$id,xsize) $xth(me,dflt,point,xsize)
-  } else {
+#  # nastavit xsize
+#  if {$mode} {
+#    set xth(me,cmds,$id,xsize) $xth(me,dflt,point,xsize)
+#  } else {
     set xth(me,cmds,$id,xsize) {}
-  }
-  set optl [xth_me_cmds_get_line_option $opts size]
-  if {[lindex $optl 2]} {
-    set xth(me,cmds,$id,xsize) [lindex $optl 0]
-    set xth(me,cmds,$id,ysize) [lindex $optl 0]
-    set opts [lindex $optl 1]
-  }
-  set optl [xth_me_cmds_get_line_option $opts "x-size"]
-  if {[lindex $optl 2]} {
-    set xth(me,cmds,$id,xsize) [lindex $optl 0]
-    set opts [lindex $optl 1]
-  }
-
-  # nastavit ysize
-  if {$mode} {
-    set xth(me,cmds,$id,ysize) $xth(me,dflt,point,ysize)
-  } else {
+#  }
+#  set optl [xth_me_cmds_get_line_option $opts size]
+#  if {[lindex $optl 2]} {
+#    set xth(me,cmds,$id,xsize) [lindex $optl 0]
+#    set xth(me,cmds,$id,ysize) [lindex $optl 0]
+#    set opts [lindex $optl 1]
+#  }
+#  set optl [xth_me_cmds_get_line_option $opts "x-size"]
+#  if {[lindex $optl 2]} {
+#    set xth(me,cmds,$id,xsize) [lindex $optl 0]
+#    set opts [lindex $optl 1]
+#  }
+#
+#  # nastavit ysize
+#  if {$mode} {
+#    set xth(me,cmds,$id,ysize) $xth(me,dflt,point,ysize)
+#  } else {
     set xth(me,cmds,$id,ysize) {}
-  }
-  set optl [xth_me_cmds_get_line_option $opts "y-size"]
-  if {[lindex $optl 2]} {
-    set xth(me,cmds,$id,ysize) [lindex $optl 0]
-    set opts [lindex $optl 1]
-  }
-  
+#  }
+#  set optl [xth_me_cmds_get_line_option $opts "y-size"]
+#  if {[lindex $optl 2]} {
+#    set xth(me,cmds,$id,ysize) [lindex $optl 0]
+#    set opts [lindex $optl 1]
+#  }
+#  
 #  if {([string length $xth(me,cmds,$id,ysize)] > 0) &&
 #      ([string length $xth(me,cmds,$id,xsize)] == 0)} {
 #    set xth(me,cmds,$id,xsize) $xth(me,cmds,$id,ysize)
@@ -1986,7 +1986,7 @@ proc xth_me_cmds_draw_point {id} {
   $xth(me,can) bind pt$id <Enter> "$xth(me,can) itemconfigure pt$id -fill cyan; xth_status_bar_push me; xth_status_bar_status me \"\$xth(me,cmds,$id,listix): \$xth(me,cmds,$id,sbar)\""
   $xth(me,can) bind pt$id <Leave> "$xth(me,can) itemconfigure pt$id -fill \[$xth(me,can) itemcget pt$id -outline\]; xth_status_bar_pop me"
   $xth(me,can) bind pt$id <1> "xth_me_cmds_click $id pt$id \$xth(me,cmds,$id,x) \$xth(me,cmds,$id,y) %x %y"
-  $xth(me,can) bind pt$id <3> "xth_me_cmds_special_select $id %x %y"  
+  $xth(me,can) bind pt$id <$xth(gui,rmb)> "xth_me_cmds_special_select $id %x %y"  
   $xth(me,can) bind pt$id <Shift-1> "xth_me_cmds_special_select $id %x %y"  
   $xth(me,can) bind pt$id <$xth(kb_control)-1> "xth_me_cmds_click_area pt$id %x %y"
 }
